@@ -1,7 +1,7 @@
 #ifndef _COLORS_H_
 #define _COLORS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -683,8 +683,7 @@ StringX4Int ColorString[] = {
   { NULL                       ,  0  ,   0,   0, 255 }
 } ;
 
-int Get_ColorForString(StringX4Int SX4I[], int alpha, 
-		       char * string, int * FlagError);
-
+int Get_ColorForString(StringX4Int SX4I[], int alpha, const char *string,
+                       int *FlagError);
 
 #endif
