@@ -1,7 +1,7 @@
 #ifndef _DEFAULT_OPTIONS_H_
 #define _DEFAULT_OPTIONS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,14 +15,12 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 // USA.
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "GmshDefines.h"
 #include "Options.h"
-#include "Views.h"
 
 //          PLEASE RESPECT THE ALPHABETICAL ORDER WHEN 
 //                    INSERTING A NEW OPTION!
@@ -78,7 +76,7 @@ StringXString GeneralOptions_String[] = {
 #if defined(WIN32)
     "notepad.exe %s" , 
 #elif defined(__APPLE__)
-    "open -e %s" ,
+    "open -t %s" ,
 #else
     "emacs %s &" ,
 #endif
@@ -104,8 +102,6 @@ StringXString GeometryOptions_String[] = {
 } ;
 
 StringXString MeshOptions_String[] = {
-  { F|O, "TriangleOptions" , opt_mesh_triangle_options , "praqzBPY" , 
-    "Options for Jonathan Shewchuk's Triangle isotropic algorithm" },
   { 0, NULL , NULL , NULL , NULL }
 } ;
 
@@ -122,7 +118,7 @@ StringXString SolverOptions_String[] = {
     "Name of solver 0" },
   { F|O, "Help0" , opt_solver_help0 , 
     "A General environment for the treatment of\nDiscrete Problems\n\n"
-    "Copyright (C) 1997-2007\nPatrick Dular and Christophe Geuzaine\n\n"
+    "Copyright (C) 1997-2008\nPatrick Dular and Christophe Geuzaine\n\n"
     "Visit http://www.geuz.org/getdp/ for more info",
     "Help string for solver 0" },
   { F|O, "Executable0" , opt_solver_executable0 , 
@@ -450,6 +446,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Relative radius of arrow stem" },
   { F|O, "Axes" , opt_general_axes , 0. ,
     "Axes (0=none, 1=simple axes, 2=box, 3=full grid, 4=open grid, 5=ruler)" },
+  { F|O, "AxesMikado" , opt_general_axes_mikado , 0. , 
+    "Mikado axes style" }, 
   { F|O, "AxesAutoPosition" , opt_general_axes_auto_position , 1. , 
     "Position the axes automatically" }, 
   { F|O, "AxesMaxX" , opt_general_axes_xmax , 1. , 
@@ -540,8 +538,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Horizontal position (in pixels) of the upper left corner of the clipping planes window" }, 
   { F|S, "ClipPositionY" , opt_general_clip_position1 , 150. , 
     "Vertical position (in pixels) of the upper left corner of the clipping planes window" }, 
-  { F|O, "ColorScheme", opt_general_color_scheme , 0. ,
-    "Default color scheme (0, 1 or 2)" },
+  { F|O, "ColorScheme", opt_general_color_scheme , 1. ,
+    "Default color scheme (0=dark, 1=light or 2=grayscale)" },
   { F|O, "ConfirmOverwrite" , opt_general_confirm_overwrite, 1. , 
     "Ask confirmation before overwriting files?" }, 
   { F|S, "ContextPositionX" , opt_general_context_position0 , 650. , 
@@ -559,6 +557,14 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "FastRedraw" , opt_general_fast_redraw, 0. ,
     "Draw simplified model while rotating, panning and zooming" },
+  { F|S, "FieldPositionX" , opt_general_field_position0 , 650. , 
+    "Horizontal position (in pixels) of the upper left corner of the field window" }, 
+  { F|S, "FieldPositionY" , opt_general_field_position1 , 550. , 
+    "Vertical position (in pixels) of the upper left corner of the field window" }, 
+  { F|S, "FieldHeight" , opt_general_field_size1 , 300. , 
+    "Height (in pixels) of the field window" }, 
+  { F|S, "FieldWidth" , opt_general_field_size0 , 300. , 
+    "Width (in pixels) of the field window" }, 
   { F|S, "FileChooserPositionX" , opt_general_file_chooser_position0 , 200. , 
     "Horizontal position (in pixels) of the upper left corner of the file chooser windows" }, 
   { F|S, "FileChooserPositionY" , opt_general_file_chooser_position1 , 200. ,
@@ -566,7 +572,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "FontSize" , opt_general_fontsize , -1. , 
     "Size of the font in the user interface (-1=automatic)" },
 
-  { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 16. ,
+  { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 17. ,
     "Size of the font in the graphic window" }, 
   { F|S, "GraphicsHeight" , opt_general_viewport3 , 600. , 
     "Height (in pixels) of the graphic window" }, 
@@ -659,11 +665,11 @@ StringXNumber GeneralOptions_Number[] = {
     "Vertical position (in pixels) of the upper left corner of the menu window" }, 
   { F|S, "MessagePositionX" , opt_general_message_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the message window" }, 
-  { F|S, "MessagePositionY" , opt_general_message_position1 , 150. , 
+  { F|S, "MessagePositionY" , opt_general_message_position1 , 490. , 
     "Vertical position (in pixels) of the upper left corner of the message window" }, 
-  { F|S, "MessageHeight" , opt_general_message_size1 , 350. , 
+  { F|S, "MessageHeight" , opt_general_message_size1 , 300. , 
     "Height (in pixels) of the message window" }, 
-  { F|S, "MessageWidth" , opt_general_message_size0 , 450. , 
+  { F|S, "MessageWidth" , opt_general_message_size0 , 400. , 
     "Width (in pixels) of the message window" }, 
   { F,   "MinX" , opt_general_xmin , 0. , 
     "Minimum model coordinate along the X-axis (read-only)" }, 
@@ -671,9 +677,13 @@ StringXNumber GeneralOptions_Number[] = {
     "Minimum model coordinate along the Y-axis (read-only)" }, 
   { F,   "MinZ" , opt_general_zmin , 0. , 
     "Minimum model coordinate along the Z-axis (read-only)" }, 
+  { F|O, "MouseHoverMeshes" , opt_general_mouse_hover_meshes , 0. ,
+    "Enable mouse hover on meshes" },
   { F|O, "MouseSelection" , opt_general_mouse_selection , 1. ,
-    "Mouse hover and selection mode (0=none, 1=hover and select geometry but only select mesh, 2=hover and select geometry and mesh)" },
+    "Enable mouse selection" },
 
+  { F|S, "NonModalWindows" , opt_general_non_modal_windows , 1. ,
+    "Force all control windows to be on top of the graphic window (\"non-modal\")" },
   { F|O, "NoPopup" , opt_general_nopopup , 0. , 
     "Disable interactive dialog windows in scripts (and use default values instead)" },
 
@@ -688,16 +698,20 @@ StringXNumber GeneralOptions_Number[] = {
     "Horizontal position (in pixels) of the upper left corner of the plugin window" }, 
   { F|S, "PluginPositionY" , opt_general_plugin_position1 , 550. , 
     "Vertical position (in pixels) of the upper left corner of the plugin window" }, 
-  { F|S, "PluginWidth" , opt_general_plugin_size0 , 100. , 
-    "Width (in pixels) of the plugin window" }, 
-  { F|S, "PluginHeight" , opt_general_plugin_size1 , 100. , 
+  { F|S, "PluginHeight" , opt_general_plugin_size1 , 300. , 
     "Height (in pixels) of the plugin window" }, 
-
+  { F|S, "PluginWidth" , opt_general_plugin_size0 , 300. , 
+    "Width (in pixels) of the plugin window" }, 
   { F|O, "PointSize" , opt_general_point_size , 3. , 
     "Display size of points (in pixels)" },
   { F|O, "PolygonOffsetAlwaysOn" , opt_general_polygon_offset_always , 0. , 
     "Always apply polygon offset, instead of trying to detect when it is required" },
-  { F|O, "PolygonOffsetFactor" , opt_general_polygon_offset_factor , 1.0 ,
+  { F|O, "PolygonOffsetFactor" , opt_general_polygon_offset_factor , 
+#if defined(__APPLE__)
+    0.5 ,
+#else
+    1.0 ,
+#endif
     "Polygon offset factor (offset = factor * DZ + r * units)" },
   { F|O, "PolygonOffsetUnits" , opt_general_polygon_offset_units , 1. , 
     "Polygon offset units (offset = factor * DZ + r * units)" },
@@ -756,7 +770,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "Terminal" , opt_general_terminal , 
 #if defined(HAVE_FLTK)
     0. ,
-#else		      
+#else                 
     1. ,
 #endif
     "Should information be printed on the terminal (if available)?" },
@@ -779,7 +793,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F,   "TranslationZ" , opt_general_translation2 , 0.0 , 
     "Z-axis translation (in model units)" },
 
-  { F|O, "VectorType" , opt_general_vector_type , DRAW_POST_ARROW3D ,
+  { F|O, "VectorType" , opt_general_vector_type , 4 ,
     "Default vector display type (for normals, etc.)" },
   { F|O, "Verbosity" , opt_general_verbosity , 3. ,
     "Level of information printed during processing (0=no information)" },
@@ -811,6 +825,8 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "Light" , opt_geometry_light , 1. , 
     "Enable lighting for the geometry" },
+  { F|O, "LightTwoSide" , opt_geometry_light_two_side , 1. , 
+    "Light both sides of surfaces (leads to slower rendering)" },
   { F|O, "Lines" , opt_geometry_lines , 1. , 
     "Display geometry curves?" },
   { F|O, "LineNumbers" , opt_geometry_lines_num , 0. , 
@@ -818,7 +834,7 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "LineSelectWidth" , opt_geometry_line_sel_width , 2. , 
     "Display width of selected lines (in pixels)" },
   { F|O, "LineType" , opt_geometry_line_type , 0. , 
-    "Display lines as solid color segments (0) or 3D cylinders (1)" },
+    "Display lines as solid color segments (0), 3D cylinders (1) or tapered cylinders (2)" },
   { F|O, "LineWidth" , opt_geometry_line_width , 2. , 
     "Display width of lines (in pixels)" },
 
@@ -845,7 +861,7 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "PointSize" , opt_geometry_point_size , 4. , 
     "Display size of points (in pixels)" },
   { F|O, "PointType" , opt_geometry_point_type , 0. , 
-    "Display points as solid color dots (0) or 3D spheres (1)" },
+    "Display points as solid color dots (0), 3D spheres (1) or scaled spheres (2)" },
 
   { F|O, "ScalingFactor" , opt_geometry_scaling_factor , 1.0 , 
     "Global geometry scaling factor" },
@@ -859,6 +875,8 @@ StringXNumber GeometryOptions_Number[] = {
     "Display geometry surfaces?" },
   { F|O, "SurfaceNumbers" , opt_geometry_surfaces_num , 0. , 
     "Display surface numbers?" },
+  { F|O, "SurfaceType" , opt_geometry_surface_type , 2. , 
+    "Surface display type (0=cross, 1=wireframe, 2=solid" },
 
   { F|O, "Tangents" , opt_geometry_tangents , 0. ,
     "Display size of tangent vectors (in pixels)" },
@@ -885,18 +903,28 @@ StringXNumber MeshOptions_Number[] = {
     "3D mesh algorithm (1=Tetgen+Delaunay, 4=Netgen)" }, 
   { F|O, "AngleSmoothNormals" , opt_mesh_angle_smooth_normals , 30.0 ,
     "Threshold angle below which normals are not smoothed" }, 
+  { F|O, "AllowSwapAngle" , opt_mesh_allow_swap_edge_angle , 10.0 ,
+    "Treshold angle (in degrees) between faces normals under which we allow an edge swap" }, 
 
   { F|O, "BdfFieldFormat" , opt_mesh_bdf_field_format , 1. , 
     "Field format for Nastran BDF files (0=free, 1=small, 2=large)" },
 
+  { F|O, "C1Continuity" , opt_mesh_c1 , 0. ,
+    "Impose C1 continuity to high order meshes (only valid in 2D plane and ElemenOrder = 2)" },
+  { F|O, "CharacteristicLengthExtendFromBoundary" , opt_mesh_lc_extend_from_boundary, 1. ,
+    "Extend characteristic lengths from the boundaries inside the surface/volume" },
   { F|O, "CharacteristicLengthFactor" , opt_mesh_lc_factor , 1.0 ,
     "Factor applied to all characteristic lengths" },
+  { F|O, "CharacteristicLengthMin" , opt_mesh_lc_min, 0.0 ,
+    "Minimum characteristic length" },
+  { F|O, "CharacteristicLengthMax" , opt_mesh_lc_max, 1.e22 ,
+    "Maximum characteristic length" },
   { F|O, "CharacteristicLengthFromCurvature" , opt_mesh_lc_from_curvature , 0. ,
-    "Compute characteritic lenghts automatically from curvatures" },
+    "Compute characteristic lengths from curvature" },
+  { F|O, "CharacteristicLengthFromPoints" , opt_mesh_lc_from_points , 1. ,
+    "Compute characteristic lengths from values given at geometry points" },
   { F|O, "ColorCarousel" , opt_mesh_color_carousel , 1. ,
     "Mesh coloring (0=by element type, 1=by elementary entity, 2=by physical entity, 3=by partition)" },
-  { F|O, "ConstrainedBackgroundMesh" , opt_mesh_constrained_bgmesh, 0. ,
-    "Should the background mesh be constrained by the characteristic lengths associated with the geometry?" },
   { F,   "CpuTime" , opt_mesh_cpu_time , 0. , 
     "CPU time (in seconds) for the generation of the current mesh (read-only)" },
   { F,   "CutPlane" , opt_mesh_use_cut_plane , 0 ,
@@ -914,15 +942,13 @@ StringXNumber MeshOptions_Number[] = {
   { F,   "CutPlaneD" , opt_mesh_cut_planed , 0.0 , 
     "Fourth cut plane equation coefficient (`D' in `AX+BY+CZ+D=0')" },
 
+  { F|O, "DrawSkinOnly" , opt_mesh_draw_skin_only , 0. ,
+    "Draw only the skin of 3D meshes?" },
   { F|O, "Dual" , opt_mesh_dual , 0. ,
     "Display the dual mesh obtained by barycentric subdivision" },
 
   { F|O, "ElementOrder" , opt_mesh_order , 1. , // "Order" is a reserved token in the parser
     "Element order (1=linear elements, N (<6) = elements of higher order)" },
-  { F|O, "C1Continuity" , opt_mesh_c1 , 0. , // "Order" is a reserved token in the parser
-    "Impose C1 continuity to high order meshes, only valid in 2D plane and ElemenOrder = 2 and 3 (todo) (Default : C0 continuity)" },
-  { F|O, "SmoothInternalEdges" , opt_mesh_smooth_internal_edges , 0 , // "Order" is a reserved token in the parser
-    "Number of smoothing steps of internal edges for high order meshes" },
   { F|O, "Explode" , opt_mesh_explode , 1.0 ,
     "Element shrinking factor (between 0 and 1)" },
 
@@ -941,7 +967,7 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "LightLines" , opt_mesh_light_lines , 1. , 
     "Enable lighting for mesh lines (element edges)" },
   { F|O, "LightTwoSide" , opt_mesh_light_two_side , 1. , 
-    "Light both sides of mesh elements (leads to slower rendering)" },
+    "Light both sides of surfaces (leads to slower rendering)" },
   { F|O, "Lines" , opt_mesh_lines , 0. , 
     "Display mesh lines (1D elements)?" },
   { F|O, "LineNumbers" , opt_mesh_lines_num , 0. , 
@@ -951,6 +977,8 @@ StringXNumber MeshOptions_Number[] = {
 
   { F|O, "MinimumCirclePoints" , opt_mesh_min_circ_points, 7. ,
     "Minimum number of points used to mesh a circle" },
+  { F|O, "MinimumCurvePoints" , opt_mesh_min_curv_points, 3. ,
+    "Minimum number of points used to mesh a (non-straight) curve" },
   { F|O, "MshBinary" , opt_mesh_msh_binary , 0. , 
     "Write MSH files in binary format?" },
   { F|O, "MshFileVersion" , opt_mesh_msh_file_version , 2.0 , 
@@ -974,6 +1002,8 @@ StringXNumber MeshOptions_Number[] = {
     "Display size of normal vectors (in pixels)" }, 
 
   { F|O, "Optimize" , opt_mesh_optimize , 0. , 
+    "Optimize the mesh to improve the quality of tetrahedral elements" },
+  { F|O, "OptimizeNetgen" , opt_mesh_optimize_netgen , 0. , 
     "Optimize the mesh using Netgen to improve the quality of tetrahedral elements" },
 
   { F|O, "Points" , opt_mesh_points , 0. , 
@@ -1004,6 +1034,8 @@ StringXNumber MeshOptions_Number[] = {
     "Only display elements whose longest edge is smaller than RadiusSup" },
   { F|O, "RandomFactor" , opt_mesh_rand_factor , 1.e-9 ,
     "Random factor used in 2D and 3D meshing algorithm (test other values when the algorithm fails)" },
+  { F|O, "RefineSteps" , opt_mesh_refine_steps , 10 ,
+    "Number of refinement steps in the MeshAdapt-based 2D algorithms" }, 
   { F|O, "RecombineAlgo" , opt_mesh_recombine_algo , 1 ,
     "Recombine algorithm (1=mixed triangles-quadrangles, 2=all quadrangles)" }, 
   { F|O, "ReverseAllNormals" , opt_mesh_reverse_all_normals , 0. , 
@@ -1015,12 +1047,18 @@ StringXNumber MeshOptions_Number[] = {
     "Save groups of nodes for each physical line and surface (UNV mesh format only)" },
   { F|O, "ScalingFactor" , opt_mesh_scaling_factor , 1.0 ,
     "Global scaling factor applied to the saved mesh" },
+  { F|O, "SecondOrderExperimental" , opt_mesh_second_order_experimental , 0. ,
+    "Use experimental code to generate second order mesh" },
   { F|O, "SecondOrderIncomplete" , opt_mesh_second_order_incomplete , 1. ,
     "Create incomplete second order elements? (8-node quads, 20-node hexas, etc.)" },
   { F|O, "SecondOrderLinear" , opt_mesh_second_order_linear , 0. ,
     "Should second order vertices simply be created by linear interpolation?" },
+  { F|O, "LcIntegrationPrecision" , opt_mesh_lc_integration_precision, 1.e-9 ,
+    "Accuracy of evaluation of the LC field for 1D mesh generation" },
   { F|O, "Smoothing" , opt_mesh_nb_smoothing , 1. ,
     "Number of smoothing steps applied to the final mesh" },
+  { F|O, "SmoothInternalEdges" , opt_mesh_smooth_internal_edges , 0 ,
+    "Number of smoothing steps of internal edges for high order meshes" },
   { F|O, "SmoothNormals" , opt_mesh_smooth_normals , 0. , 
     "Smooth the mesh normals?" },
   { F|O, "StlBinary" , opt_mesh_stl_binary , 0. , 
@@ -1125,9 +1163,6 @@ StringXNumber PostProcessingOptions_Number[] = {
   { F|O, "Smoothing" , opt_post_smooth , 0. ,
     "Apply (non-reversible) smoothing to post-processing view when merged" },
 
-  { F|O, "VertexArrays" , opt_post_vertex_arrays , 1. ,
-    "Use OpenGL vertex arrays to draw triangles in post-processing views?" },
-
   { 0, NULL , NULL , 0. }
 } ;
 
@@ -1148,6 +1183,8 @@ StringXNumber ViewOptions_Number[] = {
     "Position the scale or 2D plot automatically" }, 
   { F|O, "Axes" , opt_view_axes , 0 ,
     "Axes (0=none, 1=simple axes, 2=box, 3=full grid, 4=open grid, 5=ruler)" },
+  { F|O, "AxesMikado" , opt_view_axes_mikado , 0. , 
+    "Mikado axes style" }, 
   { F|O, "AxesAutoPosition" , opt_view_axes_auto_position , 1. , 
     "Position the axes automatically" }, 
   { F|O, "AxesMaxX" , opt_view_axes_xmax , 1. , 
@@ -1172,6 +1209,8 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "Boundary" , opt_view_boundary , 0. ,
     "Draw the `N minus b'-dimensional boundary of the element (N=element dimension, b=option value)" },
 
+  { F|O, "CenterGlyphs" , opt_view_center_glyphs , 0, 
+    "Center glyphs (arrows, numbers, etc.)" },
   { F|O, "ColormapAlpha" , opt_view_colormap_alpha , 1.0 ,
     "Colormap alpha channel value (used only if != 1)" },
   { F|O, "ColormapAlphaPower" , opt_view_colormap_alpha_power , 0.0 ,
@@ -1211,6 +1250,8 @@ StringXNumber ViewOptions_Number[] = {
     "Display post-processing quadrangles?" },
   { F|O, "DrawScalars" , opt_view_draw_scalars , 1. , 
     "Display scalar values?" },
+  { F|O, "DrawSkinOnly" , opt_view_draw_skin_only , 0. , 
+    "Draw only the skin of 3D scalar views?" },
   { F|O, "DrawStrings" , opt_view_draw_strings , 1. ,
     "Display post-processing annotation strings?" },
   { F|O, "DrawTensors" , opt_view_draw_tensors , 1. , 
@@ -1234,26 +1275,28 @@ StringXNumber ViewOptions_Number[] = {
     "Generalized raise amplification factor" },
   { F|O, "GeneralizedRaiseView" , opt_view_gen_raise_view , -1. ,
     "Index of the view used for generalized raise (-1=self)" },
-  { F|O, "GlyphLocation" , opt_view_glyph_location , DRAW_POST_LOCATE_COG , 
+  { F|O, "GlyphLocation" , opt_view_glyph_location , 1, 
     "Glyph (arrow, number, etc.) location (1=center of gravity, 2=node)" },
 
   { F|O, "Height" , opt_view_size1 , 200. , 
     "Height (in pixels) of the scale or 2D plot" }, 
 
-  { F|O, "IntervalsType" , opt_view_intervals_type , DRAW_POST_CONTINUOUS ,
+  { F|O, "IntervalsType" , opt_view_intervals_type , 2 ,
     "Type of interval display (1=iso, 2=continuous, 3=discrete, 4=numeric)" },
 
   { F|O, "Light" , opt_view_light , 1. ,
     "Enable lighting for the view" },
-  { F|O, "LightLines" , opt_view_light_lines , 0. , 
+  { F|O, "LightLines" , opt_view_light_lines , 1. , 
     "Light element edges" },
   { F|O, "LightTwoSide" , opt_view_light_two_side , 1. , 
-    "Light both sides of view elements (leads to slower rendering)" },
+    "Light both sides of surfaces (leads to slower rendering)" },
   { F|O, "LineType" , opt_view_line_type , 0. , 
     "Display lines as solid color segments (0) or 3D cylinders (1)" },
   { F|O, "LineWidth" , opt_view_line_width , 1.0 , 
     "Display width of lines (in pixels)" },
 
+  { F|O,   "MaxRecursionLevel" , opt_view_max_recursion_level , 0. , 
+    "Maximum recursion level for adaptive views" },
   { F,   "Max" , opt_view_max , 0. , 
     "Maximum value in the view (read-only)" },
   { F,   "MaxX" , opt_view_xmax , 0. , 
@@ -1275,6 +1318,8 @@ StringXNumber ViewOptions_Number[] = {
     "Number of intervals" },
   { F,   "NbTimeStep" , opt_view_nb_timestep , 1. ,
     "Number of time steps in the view (do not change this!)" },
+  { F,   "NormalRaise" , opt_view_normal_raise , 0. , 
+    "Elevation of the view along the normal (in model coordinates)" },
   { F|O, "Normals" , opt_view_normals , 0. ,
     "Display size of normal vectors (in pixels)" }, 
 
@@ -1300,18 +1345,18 @@ StringXNumber ViewOptions_Number[] = {
     "Elevation of the view along Y-axis (in model coordinates)" },
   { F,   "RaiseZ" , opt_view_raise2 , 0. , 
     "Elevation of the view along Z-axis (in model coordinates)" },
-  { F|O, "RangeType" , opt_view_range_type , DRAW_POST_RANGE_DEFAULT ,
+  { F|O, "RangeType" , opt_view_range_type , 1 ,
     "Value scale range type (1=default, 2=custom, 3=per time step)" },
 
   { F|O, "SaturateValues" , opt_view_saturate_values , 0. ,
     "Saturate the view values to custom min and max (1=true, 0=false)" },
-  { F|O, "ScaleType" , opt_view_scale_type , DRAW_POST_LINEAR ,
+  { F|O, "ScaleType" , opt_view_scale_type , 1 ,
     "Value scale type (1=linear, 2=logarithmic, 3=double logarithmic)" },
   { F|O, "ShowElement" , opt_view_show_element , 0. ,
     "Show element boundaries?" },
   { F|O, "ShowScale" , opt_view_show_scale , 1. ,
     "Show value scale?" },
-  { F|O, "ShowTime" , opt_view_show_time , 3. ,
+  { F|O, "ShowTime" , opt_view_show_time , 1. ,
     "Time display mode (0=hidden, 1=value if multi-step, 2=value always, 3=step if multi-step, 4=step always)" },
   { F|O, "SmoothNormals" , opt_view_smooth_normals , 0. ,
     "Smooth the normals?" },
@@ -1320,6 +1365,8 @@ StringXNumber ViewOptions_Number[] = {
 
   { F|O, "Tangents" , opt_view_tangents , 0. ,
     "Display size of tangent vectors (in pixels)" }, 
+  { F|O, "TargetError" , opt_view_target_error , 0.01 ,
+    "Target representation error for adaptive views" }, 
   { F|O, "TensorType" , opt_view_tensor_type , 1. ,
     "Tensor Visualization Type" },
   { F,   "TimeStep" , opt_view_timestep , 0. ,
@@ -1342,13 +1389,13 @@ StringXNumber ViewOptions_Number[] = {
     "Element (3,2) of the 3x3 coordinate transformation matrix" },
   { F,   "Transform33" , opt_view_transform22 , 1. ,
     "Element (3,3) of the 3x3 coordinate transformation matrix" },
-  { F,   "Type" , opt_view_type , DRAW_POST_3D ,
+  { F,   "Type" , opt_view_type , 1 ,
     "Type of plot (1=3D, 2=2D space, 3=2D time)" },
 
   { F|O, "UseGeneralizedRaise" , opt_view_use_gen_raise , 0 ,
     "Use generalized raise?" },
 
-  { F|O, "VectorType" , opt_view_vector_type , DRAW_POST_ARROW3D ,
+  { F|O, "VectorType" , opt_view_vector_type , 4 ,
     "Vector display type (1=segment, 2=arrow, 3=pyramid, 4=3D arrow, 5=displacement)" },
   { F,   "Visible" , opt_view_visible , 1. ,
     "Is the view visible?" },
@@ -1396,6 +1443,17 @@ StringXNumber PrintOptions_Number[] = {
   { F|O, "JpegSmoothing" , opt_print_jpeg_smoothing , 0. ,
     "JPEG smoothing (between 0 and 100)" },
 
+  { F|O, "PostElementary" , opt_print_pos_elementary , 1. ,
+    "Save elementary region tags in mesh statistics exported as post-processing views" },
+  { F|O, "PostElement" , opt_print_pos_element , 0. ,
+    "Save element numbers in mesh statistics exported as post-processing views" },
+  { F|O, "PostGamma" , opt_print_pos_gamma , 0. ,
+    "Save Gamma quality measure in mesh statistics exported as post-processing views" },
+  { F|O, "PostEta" , opt_print_pos_eta , 0. ,
+    "Save Eta quality measure in mesh statistics exported as post-processing views" },
+  { F|O, "PostRho" , opt_print_pos_rho , 0. ,
+    "Save Rho quality measure in mesh statistics exported as post-processing views" },
+
   { F|O, "TexAsEquation" , opt_print_tex_as_equation , 0. ,
     "Print all TeX strings as equations" },
   { F|O, "Text" , opt_print_text , 1. ,
@@ -1408,12 +1466,15 @@ StringXNumber PrintOptions_Number[] = {
 
 // CONTRARY TO THE PRECEDING OPTIONS, THE ORDERING OF COLOR OPTIONS MATTERS !
 
+// Solid Works (light blue to light gray): 94,198,255 -> 232,232,232
+// Catia (dark grey-blue to light gray-blue): 63,62,119 -> 181,182,202
+
 StringXColor GeneralOptions_Color[] = {
   { F|O, "Background" , opt_general_color_background ,
     {0, 0, 0, 255}, {255, 255, 255, 255}, {255, 255, 255, 255}, 
     "Background color" },
   { F|O, "BackgroundGradient" , opt_general_color_background_gradient ,
-    {0, 0, 125, 255}, {0, 0, 125, 255}, {125, 125, 125, 255}, 
+    {0, 0, 128, 255}, {128, 147, 255, 255}, {125, 125, 125, 255}, 
     "Background gradient color" },
   { F|O, "Foreground" , opt_general_color_foreground ,
     {170, 170, 170, 255}, {85, 85, 85, 255}, {128, 128, 128, 255},
@@ -1441,7 +1502,7 @@ StringXColor GeneralOptions_Color[] = {
 
 StringXColor GeometryOptions_Color[] = {
   { F|O, "Points" , opt_geometry_color_points , 
-    {178, 182, 129, 255}, {178, 182, 129, 255}, {0, 0, 0, 255}, 
+    {178, 182, 129, 255}, {90, 90, 90, 255}, {0, 0, 0, 255}, 
     "Normal geometry point color" },
   { F|O, "Lines" , opt_geometry_color_lines ,
     {0, 0, 255, 255}, {0, 0, 255, 255}, {0, 0, 0, 255},
@@ -1470,6 +1531,9 @@ StringXColor GeometryOptions_Color[] = {
   { F|O, "Normals" , opt_geometry_color_normals ,
     {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255},
     "Normal geometry vectors color" },
+  { F|O, "Projection" , opt_geometry_color_projection ,
+    {0, 255, 0, 255}, {0, 255, 0, 255}, {0, 0, 0, 255},
+    "Projection surface color" },
   { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
 } ;
 
