@@ -1,7 +1,7 @@
 #ifndef _GMSH_H_
 #define _GMSH_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,20 +15,12 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 // USA.
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <string.h>
-
-#include "Message.h"
-#include "Malloc.h"
-#include "List.h"
-#include "Tree.h"
-#include "Tools.h"
+int GmshInitialize(int argc=0, char **argv=0);
+int GmshFinalize();
+int GmshBatch();
 
 #endif
