@@ -1,7 +1,7 @@
 #ifndef _GMSH_UI_H_
 #define _GMSH_UI_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -41,8 +41,8 @@
 # define GMSH_WINDOW_BOX FL_FLAT_BOX
 #endif
 
-// recent Cygwin releases define min/max macros! 
-#if defined(__CYGWIN__)
+// some Windows versions define min/max macros! 
+#if defined(WIN32)
 #undef min
 #undef max
 #endif
