@@ -1,6 +1,6 @@
-// $Id: OS.cpp,v 1.8 2007-05-23 15:35:33 geuzaine Exp $
+// $Id: OS.cpp,v 1.13 2008-05-04 08:31:11 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -101,7 +101,7 @@ void CheckResources(){
   // can cause crashes in the recursive calls (e.g. for tet
   // classification in 3D Delaunay)
   if(r.rlim_cur < 16 * 1024 * 1024){
-    Msg(INFO, "Increasing process stack size (%d kB < 16 MB)", r.rlim_cur / 1024);
+    Msg::Info("Increasing process stack size (%d kB < 16 MB)", r.rlim_cur / 1024);
     r.rlim_cur = r.rlim_max;
     setrlimit(RLIMIT_STACK, &r);
   }
@@ -125,7 +125,7 @@ int GetProcessId()
 #endif
 }
 
-int UnlinkFile(char *filename)
+int UnlinkFile(const char *filename)
 {
 #if !defined(WIN32) || defined(__CYGWIN__)
   return unlink(filename);
@@ -134,7 +134,7 @@ int UnlinkFile(char *filename)
 #endif
 }
 
-int StatFile(char *filename)
+int StatFile(const char *filename)
 {
 #if !defined(WIN32) || defined(__CYGWIN__)
   struct stat buf;
@@ -160,23 +160,23 @@ int KillProcess(int pid)
   return 1;
 }
 
-int SystemCall(char *command)
+int SystemCall(const char *command)
 {
 #if defined(WIN32)
   STARTUPINFO suInfo;
   PROCESS_INFORMATION prInfo;
   memset(&suInfo, 0, sizeof(suInfo));
   suInfo.cb = sizeof(suInfo);
-  Msg(INFO, "Calling '%s'", command);
-  CreateProcess(NULL, command, NULL, NULL, FALSE,
+  Msg::Info("Calling '%s'", command);
+  CreateProcess(NULL, (char*)command, NULL, NULL, FALSE,
                 NORMAL_PRIORITY_CLASS, NULL, NULL, &suInfo, &prInfo);
   return 0;
 #else
   if(!system(NULL)) {
-    Msg(GERROR, "Could not find /bin/sh: aborting system call");
+    Msg::Error("Could not find /bin/sh: aborting system call");
     return 1;
   }
-  Msg(INFO, "Calling '%s'", command);
+  Msg::Info("Calling '%s'", command);
   return system(command);
 #endif
 }
