@@ -1,7 +1,7 @@
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include "ColorTable.h"
+
 #define GMSH_SET       (1<<0)
 #define GMSH_GET       (1<<1)
 #define GMSH_GUI       (1<<2)
@@ -30,197 +32,196 @@
 
 // action is a combination of GMSH_SET, GMSH_GET, GMSH_GUI
 
-#define OPT_ARGS_STR   int num, int action, char *val
+#define OPT_ARGS_STR   int num, int action, const char *val
 #define OPT_ARGS_NUM   int num, int action, double val
 #define OPT_ARGS_COL   int num, int action, unsigned int val
 
 // STRINGS
 
-char * opt_general_axes_label0(OPT_ARGS_STR);
-char * opt_general_axes_label1(OPT_ARGS_STR);
-char * opt_general_axes_label2(OPT_ARGS_STR);
-char * opt_general_axes_format0(OPT_ARGS_STR);
-char * opt_general_axes_format1(OPT_ARGS_STR);
-char * opt_general_axes_format2(OPT_ARGS_STR);
-char * opt_general_display(OPT_ARGS_STR);
-char * opt_general_filename(OPT_ARGS_STR);
-char * opt_general_default_filename(OPT_ARGS_STR);
-char * opt_general_tmp_filename(OPT_ARGS_STR);
-char * opt_general_error_filename(OPT_ARGS_STR);
-char * opt_general_session_filename(OPT_ARGS_STR);
-char * opt_general_options_filename(OPT_ARGS_STR);
-char * opt_general_editor(OPT_ARGS_STR);
-char * opt_general_web_browser(OPT_ARGS_STR);
-char * opt_general_gui_theme(OPT_ARGS_STR);
-char * opt_general_graphics_font(OPT_ARGS_STR);
-char * opt_mesh_triangle_options(OPT_ARGS_STR);
-char * opt_solver_socket_name(OPT_ARGS_STR);
-char * opt_solver_name(OPT_ARGS_STR);
-char * opt_solver_name0(OPT_ARGS_STR);
-char * opt_solver_name1(OPT_ARGS_STR);
-char * opt_solver_name2(OPT_ARGS_STR);
-char * opt_solver_name3(OPT_ARGS_STR);
-char * opt_solver_name4(OPT_ARGS_STR);
-char * opt_solver_executable(OPT_ARGS_STR);
-char * opt_solver_executable0(OPT_ARGS_STR);
-char * opt_solver_executable1(OPT_ARGS_STR);
-char * opt_solver_executable2(OPT_ARGS_STR);
-char * opt_solver_executable3(OPT_ARGS_STR);
-char * opt_solver_executable4(OPT_ARGS_STR);
-char * opt_solver_help(OPT_ARGS_STR);
-char * opt_solver_help0(OPT_ARGS_STR);
-char * opt_solver_help1(OPT_ARGS_STR);
-char * opt_solver_help2(OPT_ARGS_STR);
-char * opt_solver_help3(OPT_ARGS_STR);
-char * opt_solver_help4(OPT_ARGS_STR);
-char * opt_solver_extension(OPT_ARGS_STR);
-char * opt_solver_extension0(OPT_ARGS_STR);
-char * opt_solver_extension1(OPT_ARGS_STR);
-char * opt_solver_extension2(OPT_ARGS_STR);
-char * opt_solver_extension3(OPT_ARGS_STR);
-char * opt_solver_extension4(OPT_ARGS_STR);
-char * opt_solver_mesh_name(OPT_ARGS_STR);
-char * opt_solver_mesh_name0(OPT_ARGS_STR);
-char * opt_solver_mesh_name1(OPT_ARGS_STR);
-char * opt_solver_mesh_name2(OPT_ARGS_STR);
-char * opt_solver_mesh_name3(OPT_ARGS_STR);
-char * opt_solver_mesh_name4(OPT_ARGS_STR);
-char * opt_solver_mesh_command(OPT_ARGS_STR);
-char * opt_solver_mesh_command0(OPT_ARGS_STR);
-char * opt_solver_mesh_command1(OPT_ARGS_STR);
-char * opt_solver_mesh_command2(OPT_ARGS_STR);
-char * opt_solver_mesh_command3(OPT_ARGS_STR);
-char * opt_solver_mesh_command4(OPT_ARGS_STR);
-char * opt_solver_socket_command(OPT_ARGS_STR);
-char * opt_solver_socket_command0(OPT_ARGS_STR);
-char * opt_solver_socket_command1(OPT_ARGS_STR);
-char * opt_solver_socket_command2(OPT_ARGS_STR);
-char * opt_solver_socket_command3(OPT_ARGS_STR);
-char * opt_solver_socket_command4(OPT_ARGS_STR);
-char * opt_solver_name_command(OPT_ARGS_STR);
-char * opt_solver_name_command0(OPT_ARGS_STR);
-char * opt_solver_name_command1(OPT_ARGS_STR);
-char * opt_solver_name_command2(OPT_ARGS_STR);
-char * opt_solver_name_command3(OPT_ARGS_STR);
-char * opt_solver_name_command4(OPT_ARGS_STR);
-char * opt_solver_option_command(OPT_ARGS_STR);
-char * opt_solver_option_command0(OPT_ARGS_STR);
-char * opt_solver_option_command1(OPT_ARGS_STR);
-char * opt_solver_option_command2(OPT_ARGS_STR);
-char * opt_solver_option_command3(OPT_ARGS_STR);
-char * opt_solver_option_command4(OPT_ARGS_STR);
-char * opt_solver_first_option(OPT_ARGS_STR);
-char * opt_solver_first_option0(OPT_ARGS_STR);
-char * opt_solver_first_option1(OPT_ARGS_STR);
-char * opt_solver_first_option2(OPT_ARGS_STR);
-char * opt_solver_first_option3(OPT_ARGS_STR);
-char * opt_solver_first_option4(OPT_ARGS_STR);
-char * opt_solver_second_option(OPT_ARGS_STR);
-char * opt_solver_second_option0(OPT_ARGS_STR);
-char * opt_solver_second_option1(OPT_ARGS_STR);
-char * opt_solver_second_option2(OPT_ARGS_STR);
-char * opt_solver_second_option3(OPT_ARGS_STR);
-char * opt_solver_second_option4(OPT_ARGS_STR);
-char * opt_solver_third_option(OPT_ARGS_STR);
-char * opt_solver_third_option0(OPT_ARGS_STR);
-char * opt_solver_third_option1(OPT_ARGS_STR);
-char * opt_solver_third_option2(OPT_ARGS_STR);
-char * opt_solver_third_option3(OPT_ARGS_STR);
-char * opt_solver_third_option4(OPT_ARGS_STR);
-char * opt_solver_fourth_option(OPT_ARGS_STR);
-char * opt_solver_fourth_option0(OPT_ARGS_STR);
-char * opt_solver_fourth_option1(OPT_ARGS_STR);
-char * opt_solver_fourth_option2(OPT_ARGS_STR);
-char * opt_solver_fourth_option3(OPT_ARGS_STR);
-char * opt_solver_fourth_option4(OPT_ARGS_STR);
-char * opt_solver_fifth_option(OPT_ARGS_STR);
-char * opt_solver_fifth_option0(OPT_ARGS_STR);
-char * opt_solver_fifth_option1(OPT_ARGS_STR);
-char * opt_solver_fifth_option2(OPT_ARGS_STR);
-char * opt_solver_fifth_option3(OPT_ARGS_STR);
-char * opt_solver_fifth_option4(OPT_ARGS_STR);
-char * opt_solver_first_button(OPT_ARGS_STR);
-char * opt_solver_first_button0(OPT_ARGS_STR);
-char * opt_solver_first_button1(OPT_ARGS_STR);
-char * opt_solver_first_button2(OPT_ARGS_STR);
-char * opt_solver_first_button3(OPT_ARGS_STR);
-char * opt_solver_first_button4(OPT_ARGS_STR);
-char * opt_solver_first_button_command(OPT_ARGS_STR);
-char * opt_solver_first_button_command0(OPT_ARGS_STR);
-char * opt_solver_first_button_command1(OPT_ARGS_STR);
-char * opt_solver_first_button_command2(OPT_ARGS_STR);
-char * opt_solver_first_button_command3(OPT_ARGS_STR);
-char * opt_solver_first_button_command4(OPT_ARGS_STR);
-char * opt_solver_second_button(OPT_ARGS_STR);
-char * opt_solver_second_button0(OPT_ARGS_STR);
-char * opt_solver_second_button1(OPT_ARGS_STR);
-char * opt_solver_second_button2(OPT_ARGS_STR);
-char * opt_solver_second_button3(OPT_ARGS_STR);
-char * opt_solver_second_button4(OPT_ARGS_STR);
-char * opt_solver_second_button_command(OPT_ARGS_STR);
-char * opt_solver_second_button_command0(OPT_ARGS_STR);
-char * opt_solver_second_button_command1(OPT_ARGS_STR);
-char * opt_solver_second_button_command2(OPT_ARGS_STR);
-char * opt_solver_second_button_command3(OPT_ARGS_STR);
-char * opt_solver_second_button_command4(OPT_ARGS_STR);
-char * opt_solver_third_button(OPT_ARGS_STR);
-char * opt_solver_third_button0(OPT_ARGS_STR);
-char * opt_solver_third_button1(OPT_ARGS_STR);
-char * opt_solver_third_button2(OPT_ARGS_STR);
-char * opt_solver_third_button3(OPT_ARGS_STR);
-char * opt_solver_third_button4(OPT_ARGS_STR);
-char * opt_solver_third_button_command(OPT_ARGS_STR);
-char * opt_solver_third_button_command0(OPT_ARGS_STR);
-char * opt_solver_third_button_command1(OPT_ARGS_STR);
-char * opt_solver_third_button_command2(OPT_ARGS_STR);
-char * opt_solver_third_button_command3(OPT_ARGS_STR);
-char * opt_solver_third_button_command4(OPT_ARGS_STR);
-char * opt_solver_fourth_button(OPT_ARGS_STR);
-char * opt_solver_fourth_button0(OPT_ARGS_STR);
-char * opt_solver_fourth_button1(OPT_ARGS_STR);
-char * opt_solver_fourth_button2(OPT_ARGS_STR);
-char * opt_solver_fourth_button3(OPT_ARGS_STR);
-char * opt_solver_fourth_button4(OPT_ARGS_STR);
-char * opt_solver_fourth_button_command(OPT_ARGS_STR);
-char * opt_solver_fourth_button_command0(OPT_ARGS_STR);
-char * opt_solver_fourth_button_command1(OPT_ARGS_STR);
-char * opt_solver_fourth_button_command2(OPT_ARGS_STR);
-char * opt_solver_fourth_button_command3(OPT_ARGS_STR);
-char * opt_solver_fourth_button_command4(OPT_ARGS_STR);
-char * opt_solver_fifth_button(OPT_ARGS_STR);
-char * opt_solver_fifth_button0(OPT_ARGS_STR);
-char * opt_solver_fifth_button1(OPT_ARGS_STR);
-char * opt_solver_fifth_button2(OPT_ARGS_STR);
-char * opt_solver_fifth_button3(OPT_ARGS_STR);
-char * opt_solver_fifth_button4(OPT_ARGS_STR);
-char * opt_solver_fifth_button_command(OPT_ARGS_STR);
-char * opt_solver_fifth_button_command0(OPT_ARGS_STR);
-char * opt_solver_fifth_button_command1(OPT_ARGS_STR);
-char * opt_solver_fifth_button_command2(OPT_ARGS_STR);
-char * opt_solver_fifth_button_command3(OPT_ARGS_STR);
-char * opt_solver_fifth_button_command4(OPT_ARGS_STR);
-char * opt_view_name(OPT_ARGS_STR);
-char * opt_view_format(OPT_ARGS_STR);
-char * opt_view_filename(OPT_ARGS_STR);
-char * opt_view_axes_label0(OPT_ARGS_STR);
-char * opt_view_axes_label1(OPT_ARGS_STR);
-char * opt_view_axes_label2(OPT_ARGS_STR);
-char * opt_view_axes_format0(OPT_ARGS_STR);
-char * opt_view_axes_format1(OPT_ARGS_STR);
-char * opt_view_axes_format2(OPT_ARGS_STR);
-char * opt_view_gen_raise0(OPT_ARGS_STR);
-char * opt_view_gen_raise1(OPT_ARGS_STR);
-char * opt_view_gen_raise2(OPT_ARGS_STR);
-char * opt_view_stipple0(OPT_ARGS_STR);
-char * opt_view_stipple1(OPT_ARGS_STR);
-char * opt_view_stipple2(OPT_ARGS_STR);
-char * opt_view_stipple3(OPT_ARGS_STR);
-char * opt_view_stipple4(OPT_ARGS_STR);
-char * opt_view_stipple5(OPT_ARGS_STR);
-char * opt_view_stipple6(OPT_ARGS_STR);
-char * opt_view_stipple7(OPT_ARGS_STR);
-char * opt_view_stipple8(OPT_ARGS_STR);
-char * opt_view_stipple9(OPT_ARGS_STR);
+const char *opt_general_axes_label0(OPT_ARGS_STR);
+const char *opt_general_axes_label1(OPT_ARGS_STR);
+const char *opt_general_axes_label2(OPT_ARGS_STR);
+const char *opt_general_axes_format0(OPT_ARGS_STR);
+const char *opt_general_axes_format1(OPT_ARGS_STR);
+const char *opt_general_axes_format2(OPT_ARGS_STR);
+const char *opt_general_display(OPT_ARGS_STR);
+const char *opt_general_filename(OPT_ARGS_STR);
+const char *opt_general_default_filename(OPT_ARGS_STR);
+const char *opt_general_tmp_filename(OPT_ARGS_STR);
+const char *opt_general_error_filename(OPT_ARGS_STR);
+const char *opt_general_session_filename(OPT_ARGS_STR);
+const char *opt_general_options_filename(OPT_ARGS_STR);
+const char *opt_general_editor(OPT_ARGS_STR);
+const char *opt_general_web_browser(OPT_ARGS_STR);
+const char *opt_general_gui_theme(OPT_ARGS_STR);
+const char *opt_general_graphics_font(OPT_ARGS_STR);
+const char *opt_solver_socket_name(OPT_ARGS_STR);
+const char *opt_solver_name(OPT_ARGS_STR);
+const char *opt_solver_name0(OPT_ARGS_STR);
+const char *opt_solver_name1(OPT_ARGS_STR);
+const char *opt_solver_name2(OPT_ARGS_STR);
+const char *opt_solver_name3(OPT_ARGS_STR);
+const char *opt_solver_name4(OPT_ARGS_STR);
+const char *opt_solver_executable(OPT_ARGS_STR);
+const char *opt_solver_executable0(OPT_ARGS_STR);
+const char *opt_solver_executable1(OPT_ARGS_STR);
+const char *opt_solver_executable2(OPT_ARGS_STR);
+const char *opt_solver_executable3(OPT_ARGS_STR);
+const char *opt_solver_executable4(OPT_ARGS_STR);
+const char *opt_solver_help(OPT_ARGS_STR);
+const char *opt_solver_help0(OPT_ARGS_STR);
+const char *opt_solver_help1(OPT_ARGS_STR);
+const char *opt_solver_help2(OPT_ARGS_STR);
+const char *opt_solver_help3(OPT_ARGS_STR);
+const char *opt_solver_help4(OPT_ARGS_STR);
+const char *opt_solver_extension(OPT_ARGS_STR);
+const char *opt_solver_extension0(OPT_ARGS_STR);
+const char *opt_solver_extension1(OPT_ARGS_STR);
+const char *opt_solver_extension2(OPT_ARGS_STR);
+const char *opt_solver_extension3(OPT_ARGS_STR);
+const char *opt_solver_extension4(OPT_ARGS_STR);
+const char *opt_solver_mesh_name(OPT_ARGS_STR);
+const char *opt_solver_mesh_name0(OPT_ARGS_STR);
+const char *opt_solver_mesh_name1(OPT_ARGS_STR);
+const char *opt_solver_mesh_name2(OPT_ARGS_STR);
+const char *opt_solver_mesh_name3(OPT_ARGS_STR);
+const char *opt_solver_mesh_name4(OPT_ARGS_STR);
+const char *opt_solver_mesh_command(OPT_ARGS_STR);
+const char *opt_solver_mesh_command0(OPT_ARGS_STR);
+const char *opt_solver_mesh_command1(OPT_ARGS_STR);
+const char *opt_solver_mesh_command2(OPT_ARGS_STR);
+const char *opt_solver_mesh_command3(OPT_ARGS_STR);
+const char *opt_solver_mesh_command4(OPT_ARGS_STR);
+const char *opt_solver_socket_command(OPT_ARGS_STR);
+const char *opt_solver_socket_command0(OPT_ARGS_STR);
+const char *opt_solver_socket_command1(OPT_ARGS_STR);
+const char *opt_solver_socket_command2(OPT_ARGS_STR);
+const char *opt_solver_socket_command3(OPT_ARGS_STR);
+const char *opt_solver_socket_command4(OPT_ARGS_STR);
+const char *opt_solver_name_command(OPT_ARGS_STR);
+const char *opt_solver_name_command0(OPT_ARGS_STR);
+const char *opt_solver_name_command1(OPT_ARGS_STR);
+const char *opt_solver_name_command2(OPT_ARGS_STR);
+const char *opt_solver_name_command3(OPT_ARGS_STR);
+const char *opt_solver_name_command4(OPT_ARGS_STR);
+const char *opt_solver_option_command(OPT_ARGS_STR);
+const char *opt_solver_option_command0(OPT_ARGS_STR);
+const char *opt_solver_option_command1(OPT_ARGS_STR);
+const char *opt_solver_option_command2(OPT_ARGS_STR);
+const char *opt_solver_option_command3(OPT_ARGS_STR);
+const char *opt_solver_option_command4(OPT_ARGS_STR);
+const char *opt_solver_first_option(OPT_ARGS_STR);
+const char *opt_solver_first_option0(OPT_ARGS_STR);
+const char *opt_solver_first_option1(OPT_ARGS_STR);
+const char *opt_solver_first_option2(OPT_ARGS_STR);
+const char *opt_solver_first_option3(OPT_ARGS_STR);
+const char *opt_solver_first_option4(OPT_ARGS_STR);
+const char *opt_solver_second_option(OPT_ARGS_STR);
+const char *opt_solver_second_option0(OPT_ARGS_STR);
+const char *opt_solver_second_option1(OPT_ARGS_STR);
+const char *opt_solver_second_option2(OPT_ARGS_STR);
+const char *opt_solver_second_option3(OPT_ARGS_STR);
+const char *opt_solver_second_option4(OPT_ARGS_STR);
+const char *opt_solver_third_option(OPT_ARGS_STR);
+const char *opt_solver_third_option0(OPT_ARGS_STR);
+const char *opt_solver_third_option1(OPT_ARGS_STR);
+const char *opt_solver_third_option2(OPT_ARGS_STR);
+const char *opt_solver_third_option3(OPT_ARGS_STR);
+const char *opt_solver_third_option4(OPT_ARGS_STR);
+const char *opt_solver_fourth_option(OPT_ARGS_STR);
+const char *opt_solver_fourth_option0(OPT_ARGS_STR);
+const char *opt_solver_fourth_option1(OPT_ARGS_STR);
+const char *opt_solver_fourth_option2(OPT_ARGS_STR);
+const char *opt_solver_fourth_option3(OPT_ARGS_STR);
+const char *opt_solver_fourth_option4(OPT_ARGS_STR);
+const char *opt_solver_fifth_option(OPT_ARGS_STR);
+const char *opt_solver_fifth_option0(OPT_ARGS_STR);
+const char *opt_solver_fifth_option1(OPT_ARGS_STR);
+const char *opt_solver_fifth_option2(OPT_ARGS_STR);
+const char *opt_solver_fifth_option3(OPT_ARGS_STR);
+const char *opt_solver_fifth_option4(OPT_ARGS_STR);
+const char *opt_solver_first_button(OPT_ARGS_STR);
+const char *opt_solver_first_button0(OPT_ARGS_STR);
+const char *opt_solver_first_button1(OPT_ARGS_STR);
+const char *opt_solver_first_button2(OPT_ARGS_STR);
+const char *opt_solver_first_button3(OPT_ARGS_STR);
+const char *opt_solver_first_button4(OPT_ARGS_STR);
+const char *opt_solver_first_button_command(OPT_ARGS_STR);
+const char *opt_solver_first_button_command0(OPT_ARGS_STR);
+const char *opt_solver_first_button_command1(OPT_ARGS_STR);
+const char *opt_solver_first_button_command2(OPT_ARGS_STR);
+const char *opt_solver_first_button_command3(OPT_ARGS_STR);
+const char *opt_solver_first_button_command4(OPT_ARGS_STR);
+const char *opt_solver_second_button(OPT_ARGS_STR);
+const char *opt_solver_second_button0(OPT_ARGS_STR);
+const char *opt_solver_second_button1(OPT_ARGS_STR);
+const char *opt_solver_second_button2(OPT_ARGS_STR);
+const char *opt_solver_second_button3(OPT_ARGS_STR);
+const char *opt_solver_second_button4(OPT_ARGS_STR);
+const char *opt_solver_second_button_command(OPT_ARGS_STR);
+const char *opt_solver_second_button_command0(OPT_ARGS_STR);
+const char *opt_solver_second_button_command1(OPT_ARGS_STR);
+const char *opt_solver_second_button_command2(OPT_ARGS_STR);
+const char *opt_solver_second_button_command3(OPT_ARGS_STR);
+const char *opt_solver_second_button_command4(OPT_ARGS_STR);
+const char *opt_solver_third_button(OPT_ARGS_STR);
+const char *opt_solver_third_button0(OPT_ARGS_STR);
+const char *opt_solver_third_button1(OPT_ARGS_STR);
+const char *opt_solver_third_button2(OPT_ARGS_STR);
+const char *opt_solver_third_button3(OPT_ARGS_STR);
+const char *opt_solver_third_button4(OPT_ARGS_STR);
+const char *opt_solver_third_button_command(OPT_ARGS_STR);
+const char *opt_solver_third_button_command0(OPT_ARGS_STR);
+const char *opt_solver_third_button_command1(OPT_ARGS_STR);
+const char *opt_solver_third_button_command2(OPT_ARGS_STR);
+const char *opt_solver_third_button_command3(OPT_ARGS_STR);
+const char *opt_solver_third_button_command4(OPT_ARGS_STR);
+const char *opt_solver_fourth_button(OPT_ARGS_STR);
+const char *opt_solver_fourth_button0(OPT_ARGS_STR);
+const char *opt_solver_fourth_button1(OPT_ARGS_STR);
+const char *opt_solver_fourth_button2(OPT_ARGS_STR);
+const char *opt_solver_fourth_button3(OPT_ARGS_STR);
+const char *opt_solver_fourth_button4(OPT_ARGS_STR);
+const char *opt_solver_fourth_button_command(OPT_ARGS_STR);
+const char *opt_solver_fourth_button_command0(OPT_ARGS_STR);
+const char *opt_solver_fourth_button_command1(OPT_ARGS_STR);
+const char *opt_solver_fourth_button_command2(OPT_ARGS_STR);
+const char *opt_solver_fourth_button_command3(OPT_ARGS_STR);
+const char *opt_solver_fourth_button_command4(OPT_ARGS_STR);
+const char *opt_solver_fifth_button(OPT_ARGS_STR);
+const char *opt_solver_fifth_button0(OPT_ARGS_STR);
+const char *opt_solver_fifth_button1(OPT_ARGS_STR);
+const char *opt_solver_fifth_button2(OPT_ARGS_STR);
+const char *opt_solver_fifth_button3(OPT_ARGS_STR);
+const char *opt_solver_fifth_button4(OPT_ARGS_STR);
+const char *opt_solver_fifth_button_command(OPT_ARGS_STR);
+const char *opt_solver_fifth_button_command0(OPT_ARGS_STR);
+const char *opt_solver_fifth_button_command1(OPT_ARGS_STR);
+const char *opt_solver_fifth_button_command2(OPT_ARGS_STR);
+const char *opt_solver_fifth_button_command3(OPT_ARGS_STR);
+const char *opt_solver_fifth_button_command4(OPT_ARGS_STR);
+const char *opt_view_name(OPT_ARGS_STR);
+const char *opt_view_format(OPT_ARGS_STR);
+const char *opt_view_filename(OPT_ARGS_STR);
+const char *opt_view_axes_label0(OPT_ARGS_STR);
+const char *opt_view_axes_label1(OPT_ARGS_STR);
+const char *opt_view_axes_label2(OPT_ARGS_STR);
+const char *opt_view_axes_format0(OPT_ARGS_STR);
+const char *opt_view_axes_format1(OPT_ARGS_STR);
+const char *opt_view_axes_format2(OPT_ARGS_STR);
+const char *opt_view_gen_raise0(OPT_ARGS_STR);
+const char *opt_view_gen_raise1(OPT_ARGS_STR);
+const char *opt_view_gen_raise2(OPT_ARGS_STR);
+const char *opt_view_stipple0(OPT_ARGS_STR);
+const char *opt_view_stipple1(OPT_ARGS_STR);
+const char *opt_view_stipple2(OPT_ARGS_STR);
+const char *opt_view_stipple3(OPT_ARGS_STR);
+const char *opt_view_stipple4(OPT_ARGS_STR);
+const char *opt_view_stipple5(OPT_ARGS_STR);
+const char *opt_view_stipple6(OPT_ARGS_STR);
+const char *opt_view_stipple7(OPT_ARGS_STR);
+const char *opt_view_stipple8(OPT_ARGS_STR);
+const char *opt_view_stipple9(OPT_ARGS_STR);
 
 // NUMBERS
 
@@ -253,6 +254,10 @@ double opt_general_plugin_position0(OPT_ARGS_NUM);
 double opt_general_plugin_position1(OPT_ARGS_NUM);
 double opt_general_plugin_size0(OPT_ARGS_NUM);
 double opt_general_plugin_size1(OPT_ARGS_NUM);
+double opt_general_field_position0(OPT_ARGS_NUM);
+double opt_general_field_position1(OPT_ARGS_NUM);
+double opt_general_field_size0(OPT_ARGS_NUM);
+double opt_general_field_size1(OPT_ARGS_NUM);
 double opt_general_statistics_position0(OPT_ARGS_NUM);
 double opt_general_statistics_position1(OPT_ARGS_NUM);
 double opt_general_visibility_position0(OPT_ARGS_NUM);
@@ -288,11 +293,13 @@ double opt_general_color_scheme(OPT_ARGS_NUM);
 double opt_general_background_gradient(OPT_ARGS_NUM);
 double opt_general_verbosity(OPT_ARGS_NUM);
 double opt_general_nopopup(OPT_ARGS_NUM);
+double opt_general_non_modal_windows(OPT_ARGS_NUM);
 double opt_general_terminal(OPT_ARGS_NUM);
 double opt_general_tooltips(OPT_ARGS_NUM);
 double opt_general_confirm_overwrite(OPT_ARGS_NUM);
 double opt_general_orthographic(OPT_ARGS_NUM);
 double opt_general_mouse_selection(OPT_ARGS_NUM);
+double opt_general_mouse_hover_meshes(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_xmin(OPT_ARGS_NUM);
@@ -302,6 +309,7 @@ double opt_general_ymax(OPT_ARGS_NUM);
 double opt_general_zmin(OPT_ARGS_NUM);
 double opt_general_zmax(OPT_ARGS_NUM);
 double opt_general_axes(OPT_ARGS_NUM);
+double opt_general_axes_mikado(OPT_ARGS_NUM);
 double opt_general_axes_auto_position(OPT_ARGS_NUM);
 double opt_general_axes_xmin(OPT_ARGS_NUM);
 double opt_general_axes_xmax(OPT_ARGS_NUM);
@@ -407,7 +415,9 @@ double opt_geometry_point_type(OPT_ARGS_NUM);
 double opt_geometry_line_width(OPT_ARGS_NUM);
 double opt_geometry_line_sel_width(OPT_ARGS_NUM);
 double opt_geometry_line_type(OPT_ARGS_NUM);
+double opt_geometry_surface_type(OPT_ARGS_NUM);
 double opt_geometry_light(OPT_ARGS_NUM);
+double opt_geometry_light_two_side(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_edges(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_sew_faces(OPT_ARGS_NUM);
@@ -422,12 +432,19 @@ double opt_geometry_snap1(OPT_ARGS_NUM);
 double opt_geometry_snap2(OPT_ARGS_NUM);
 double opt_mesh_label_frequency(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
+double opt_mesh_optimize_netgen(OPT_ARGS_NUM);
+double opt_mesh_refine_steps(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
 double opt_mesh_tangents(OPT_ARGS_NUM);
 double opt_mesh_explode(OPT_ARGS_NUM);
 double opt_mesh_scaling_factor(OPT_ARGS_NUM);
+double opt_mesh_lc_min(OPT_ARGS_NUM);
+double opt_mesh_lc_max(OPT_ARGS_NUM);
 double opt_mesh_lc_factor(OPT_ARGS_NUM);
 double opt_mesh_lc_from_curvature(OPT_ARGS_NUM);
+double opt_mesh_lc_from_points(OPT_ARGS_NUM);
+double opt_mesh_lc_extend_from_boundary(OPT_ARGS_NUM);
+double opt_mesh_lc_integration_precision(OPT_ARGS_NUM);
 double opt_mesh_rand_factor(OPT_ARGS_NUM);
 double opt_mesh_quality_inf(OPT_ARGS_NUM);
 double opt_mesh_quality_sup(OPT_ARGS_NUM);
@@ -454,7 +471,6 @@ double opt_mesh_volumes_num(OPT_ARGS_NUM);
 double opt_mesh_point_size(OPT_ARGS_NUM);
 double opt_mesh_point_type(OPT_ARGS_NUM);
 double opt_mesh_line_width(OPT_ARGS_NUM);
-double opt_mesh_vertex_arrays(OPT_ARGS_NUM);
 double opt_mesh_reverse_all_normals(OPT_ARGS_NUM);
 double opt_mesh_smooth_normals(OPT_ARGS_NUM);
 double opt_mesh_angle_smooth_normals(OPT_ARGS_NUM);
@@ -471,13 +487,16 @@ double opt_mesh_algo2d(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_recombine_algo(OPT_ARGS_NUM);
 double opt_mesh_min_circ_points(OPT_ARGS_NUM);
-double opt_mesh_constrained_bgmesh(OPT_ARGS_NUM);
+double opt_mesh_allow_swap_edge_angle(OPT_ARGS_NUM);
+double opt_mesh_min_curv_points(OPT_ARGS_NUM);
 double opt_mesh_order(OPT_ARGS_NUM);
 double opt_mesh_c1(OPT_ARGS_NUM);
 double opt_mesh_smooth_internal_edges(OPT_ARGS_NUM);
+double opt_mesh_second_order_experimental(OPT_ARGS_NUM);
 double opt_mesh_second_order_linear(OPT_ARGS_NUM);
 double opt_mesh_second_order_incomplete(OPT_ARGS_NUM);
 double opt_mesh_dual(OPT_ARGS_NUM);
+double opt_mesh_draw_skin_only(OPT_ARGS_NUM);
 double opt_mesh_use_cut_plane(OPT_ARGS_NUM);
 double opt_mesh_cut_plane_draw_intersect(OPT_ARGS_NUM);
 double opt_mesh_cut_plane_only_volume(OPT_ARGS_NUM);
@@ -517,7 +536,6 @@ double opt_solver_merge_views1(OPT_ARGS_NUM);
 double opt_solver_merge_views2(OPT_ARGS_NUM);
 double opt_solver_merge_views3(OPT_ARGS_NUM);
 double opt_solver_merge_views4(OPT_ARGS_NUM);
-double opt_post_vertex_arrays(OPT_ARGS_NUM);
 double opt_post_horizontal_scales(OPT_ARGS_NUM);
 double opt_post_link(OPT_ARGS_NUM);
 double opt_post_smooth(OPT_ARGS_NUM);
@@ -545,6 +563,7 @@ double opt_view_offset2(OPT_ARGS_NUM);
 double opt_view_raise0(OPT_ARGS_NUM);
 double opt_view_raise1(OPT_ARGS_NUM);
 double opt_view_raise2(OPT_ARGS_NUM);
+double opt_view_normal_raise(OPT_ARGS_NUM);
 double opt_view_transform00(OPT_ARGS_NUM);
 double opt_view_transform01(OPT_ARGS_NUM);
 double opt_view_transform02(OPT_ARGS_NUM);
@@ -585,6 +604,7 @@ double opt_view_gen_raise_factor(OPT_ARGS_NUM);
 double opt_view_use_gen_raise(OPT_ARGS_NUM);
 double opt_view_type(OPT_ARGS_NUM);
 double opt_view_axes(OPT_ARGS_NUM);
+double opt_view_axes_mikado(OPT_ARGS_NUM);
 double opt_view_axes_auto_position(OPT_ARGS_NUM);
 double opt_view_axes_tics0(OPT_ARGS_NUM);
 double opt_view_axes_tics1(OPT_ARGS_NUM);
@@ -622,11 +642,13 @@ double opt_view_draw_pyramids(OPT_ARGS_NUM);
 double opt_view_draw_scalars(OPT_ARGS_NUM);
 double opt_view_draw_vectors(OPT_ARGS_NUM);
 double opt_view_draw_tensors(OPT_ARGS_NUM);
+double opt_view_draw_skin_only(OPT_ARGS_NUM);
 double opt_view_scale_type(OPT_ARGS_NUM);
 double opt_view_tensor_type(OPT_ARGS_NUM);
 double opt_view_range_type(OPT_ARGS_NUM);
 double opt_view_vector_type(OPT_ARGS_NUM);
 double opt_view_glyph_location(OPT_ARGS_NUM);
+double opt_view_center_glyphs(OPT_ARGS_NUM);
 double opt_view_point_size(OPT_ARGS_NUM);
 double opt_view_line_width(OPT_ARGS_NUM);
 double opt_view_point_type(OPT_ARGS_NUM);
@@ -643,6 +665,11 @@ double opt_print_eps_point_size_factor(OPT_ARGS_NUM);
 double opt_print_jpeg_quality(OPT_ARGS_NUM);
 double opt_print_jpeg_smoothing(OPT_ARGS_NUM);
 double opt_print_geo_labels(OPT_ARGS_NUM);
+double opt_print_pos_elementary(OPT_ARGS_NUM);
+double opt_print_pos_element(OPT_ARGS_NUM);
+double opt_print_pos_gamma(OPT_ARGS_NUM);
+double opt_print_pos_eta(OPT_ARGS_NUM);
+double opt_print_pos_rho(OPT_ARGS_NUM);
 double opt_print_gif_dither(OPT_ARGS_NUM);
 double opt_print_gif_sort(OPT_ARGS_NUM);
 double opt_print_gif_interlace(OPT_ARGS_NUM);
@@ -671,6 +698,7 @@ unsigned int opt_geometry_color_highlight1(OPT_ARGS_COL);
 unsigned int opt_geometry_color_highlight2(OPT_ARGS_COL);
 unsigned int opt_geometry_color_tangents(OPT_ARGS_COL);
 unsigned int opt_geometry_color_normals(OPT_ARGS_COL);
+unsigned int opt_geometry_color_projection(OPT_ARGS_COL);
 unsigned int opt_mesh_color_points(OPT_ARGS_COL); 
 unsigned int opt_mesh_color_points_sup(OPT_ARGS_COL); 
 unsigned int opt_mesh_color_lines(OPT_ARGS_COL); 
@@ -719,46 +747,46 @@ unsigned int opt_view_color_axes(OPT_ARGS_COL);
 // Data structures and global functions
 
 typedef struct {
-  char *str ; 
+  const char *str ; 
   int int1, int2, int3, int4 ;
 } StringX4Int;
 
 typedef struct {
   int level;
-  char *str ;
-  char * (*function)(int num, int action, char *val) ;
-  char *def ;
-  char *help ;
+  const char *str ;
+  const char * (*function)(int num, int action, const char *val) ;
+  const char *def ;
+  const char *help ;
 } StringXString ;
 
 typedef struct {
   int level;
-  char *str;
+  const char *str;
   double (*function)(int num, int action, double val) ;
   double def ;
-  char *help ;
+  const char *help ;
 } StringXNumber ;
 
 typedef struct {
   int level;
-  char *str ; 
+  const char *str ; 
   unsigned int (*function)(int num, int action, unsigned int val) ;
   // the defaults are stored in individual bytes so that we can initialize
   // them statically independently of the machine endianness. They will be
   // packed into unsigned ints at runtime
   unsigned char def1[4], def2[4], def3[4] ;
-  char *help ;
+  const char *help ;
 } StringXColor ;
 
 void Init_Options(int num);
 void Init_Options_GUI(int num);
 void ReInit_Options(int num);
-void Print_Options(int num, int level, int diff, int help, char *filename);
+void Print_Options(int num, int level, int diff, int help, const char *filename);
 void Print_OptionsDoc();
 
-StringXString * Get_StringOptionCategory(char * cat);
-StringXNumber * Get_NumberOptionCategory(char * cat);
-StringXColor * Get_ColorOptionCategory(char * cat);
+StringXString *Get_StringOptionCategory(const char *cat);
+StringXNumber *Get_NumberOptionCategory(const char *cat);
+StringXColor *Get_ColorOptionCategory(const char *cat);
 
 void Set_DefaultStringOptions(int num, StringXString s[]);
 void Set_DefaultNumberOptions(int num, StringXNumber s[]);
@@ -768,20 +796,22 @@ void Set_StringOptions_GUI(int num, StringXString s[]);
 void Set_NumberOptions_GUI(int num, StringXNumber s[]);
 void Set_ColorOptions_GUI(int num, StringXColor s[]);
 
-void * Get_StringOption(char *str, StringXString s[]);
-void * Get_NumberOption(char *str, StringXNumber s[]);
-void * Get_ColorOption(char *str, StringXColor s[]);
+void * Get_StringOption(const char *str, StringXString s[]);
+void * Get_NumberOption(const char *str, StringXNumber s[]);
+void * Get_ColorOption(const char *str, StringXColor s[]);
 
 void Print_StringOptions(int num, int level, int diff, int help,
-			 StringXString s[], char *prefix, FILE *file);
+                         StringXString s[], const char *prefix, FILE *file);
 void Print_NumberOptions(int num, int level, int diff, int help,
-			 StringXNumber s[], char *prefix, FILE *file);
+                         StringXNumber s[], const char *prefix, FILE *file);
 void Print_ColorOptions(int num, int level, int diff, int help,
-			StringXColor s[], char *prefix, FILE *file);
+                        StringXColor s[], const char *prefix, FILE *file);
 
-void Print_StringOptionsDoc(StringXString s[], char *prefix, FILE * file);
-void Print_NumberOptionsDoc(StringXNumber s[], char *prefix, FILE * file);
-void Print_ColorOptionsDoc(StringXColor s[], char *prefix, FILE * file);
+void Print_StringOptionsDoc(StringXString s[], const char *prefix, FILE * file);
+void Print_NumberOptionsDoc(StringXNumber s[], const char *prefix, FILE * file);
+void Print_ColorOptionsDoc(StringXColor s[], const char *prefix, FILE * file);
+
+GmshColorTable *Get_ColorTable(int num);
 
 extern StringXString GeneralOptions_String[] ;
 extern StringXString GeometryOptions_String[] ;
