@@ -1,7 +1,7 @@
 #ifndef _CALLBACKS_H_
 #define _CALLBACKS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
 
 #define CALLBACK_ARGS   Fl_Widget* w, void* data
 
-int SetGlobalShortcut(int event);
 void ManualPlay(int time, int step);
+int SetGlobalShortcut(int event);
 
 // Common callbacks
 
@@ -123,6 +123,7 @@ void view_save_parsed_cb(CALLBACK_ARGS);
 void view_save_stl_cb(CALLBACK_ARGS);
 void view_save_txt_cb(CALLBACK_ARGS);
 void view_save_msh_cb(CALLBACK_ARGS);
+void view_save_med_cb(CALLBACK_ARGS);
 void view_alias_cb(CALLBACK_ARGS);
 void view_alias_with_options_cb(CALLBACK_ARGS);
 void view_combine_space_all_cb(CALLBACK_ARGS);
@@ -133,6 +134,16 @@ void view_combine_time_visible_cb(CALLBACK_ARGS);
 void view_combine_time_by_name_cb(CALLBACK_ARGS);
 void view_all_visible_cb(CALLBACK_ARGS);
 void view_applybgmesh_cb(CALLBACK_ARGS);
+void view_field_cb(CALLBACK_ARGS);
+void view_field_new_cb(CALLBACK_ARGS);
+void view_field_apply_cb(CALLBACK_ARGS);
+void view_field_revert_cb(CALLBACK_ARGS);
+void view_field_cancel_cb(CALLBACK_ARGS);
+void view_field_browser_cb(CALLBACK_ARGS);
+void view_field_delete_cb(CALLBACK_ARGS);
+void view_field_set_size_btn_cb(CALLBACK_ARGS);
+void view_field_put_on_view_cb(CALLBACK_ARGS);
+void view_field_select_node_cb(CALLBACK_ARGS);
 void view_plugin_cb(CALLBACK_ARGS);
 void view_plugin_browser_cb(CALLBACK_ARGS);
 void view_plugin_run_cb(CALLBACK_ARGS);
@@ -198,68 +209,21 @@ void mod_forward_cb(CALLBACK_ARGS);
 void geometry_elementary_cb(CALLBACK_ARGS);
 void   geometry_elementary_add_cb(CALLBACK_ARGS);
 void     geometry_elementary_add_new_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_parameter_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_point_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_line_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_spline_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_bspline_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_circle_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_ellipse_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_planesurface_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_ruledsurface_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_new_volume_cb(CALLBACK_ARGS);
 void     geometry_elementary_add_translate_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_translate_point_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_translate_line_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_translate_surface_cb(CALLBACK_ARGS);
 void     geometry_elementary_add_rotate_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_rotate_point_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_rotate_line_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_rotate_surface_cb(CALLBACK_ARGS);
 void     geometry_elementary_add_scale_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_scale_point_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_scale_line_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_scale_surface_cb(CALLBACK_ARGS);
 void     geometry_elementary_add_symmetry_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_symmetry_point_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_symmetry_line_cb(CALLBACK_ARGS);
-void       geometry_elementary_add_symmetry_surface_cb(CALLBACK_ARGS);
+void   geometry_elementary_delete_cb(CALLBACK_ARGS);
 void   geometry_elementary_translate_cb(CALLBACK_ARGS);
-void     geometry_elementary_translate_point_cb(CALLBACK_ARGS);
-void     geometry_elementary_translate_line_cb(CALLBACK_ARGS);
-void     geometry_elementary_translate_surface_cb(CALLBACK_ARGS);
 void   geometry_elementary_rotate_cb(CALLBACK_ARGS);
-void     geometry_elementary_rotate_point_cb(CALLBACK_ARGS);
-void     geometry_elementary_rotate_line_cb(CALLBACK_ARGS);
-void     geometry_elementary_rotate_surface_cb(CALLBACK_ARGS);
 void   geometry_elementary_scale_cb(CALLBACK_ARGS);
-void     geometry_elementary_scale_point_cb(CALLBACK_ARGS);
-void     geometry_elementary_scale_line_cb(CALLBACK_ARGS);
-void     geometry_elementary_scale_surface_cb(CALLBACK_ARGS);
 void   geometry_elementary_symmetry_cb(CALLBACK_ARGS);
-void     geometry_elementary_symmetry_point_cb(CALLBACK_ARGS);
-void     geometry_elementary_symmetry_line_cb(CALLBACK_ARGS);
-void     geometry_elementary_symmetry_surface_cb(CALLBACK_ARGS);
 void   geometry_elementary_extrude_cb(CALLBACK_ARGS);
 void     geometry_elementary_extrude_translate_cb(CALLBACK_ARGS);
-void        geometry_elementary_extrude_translate_point_cb(CALLBACK_ARGS);
-void        geometry_elementary_extrude_translate_line_cb(CALLBACK_ARGS);
-void        geometry_elementary_extrude_translate_surface_cb(CALLBACK_ARGS);
 void     geometry_elementary_extrude_rotate_cb(CALLBACK_ARGS);
-void        geometry_elementary_extrude_rotate_point_cb(CALLBACK_ARGS);
-void        geometry_elementary_extrude_rotate_line_cb(CALLBACK_ARGS);
-void        geometry_elementary_extrude_rotate_surface_cb(CALLBACK_ARGS);
 void   geometry_elementary_coherence_cb(CALLBACK_ARGS);
-void   geometry_elementary_delete_cb(CALLBACK_ARGS);
-void     geometry_elementary_delete_point_cb(CALLBACK_ARGS);
-void     geometry_elementary_delete_line_cb(CALLBACK_ARGS);
-void     geometry_elementary_delete_surface_cb(CALLBACK_ARGS);
 void geometry_physical_cb(CALLBACK_ARGS);
 void   geometry_physical_add_cb(CALLBACK_ARGS);
-void     geometry_physical_add_point_cb (CALLBACK_ARGS);
-void     geometry_physical_add_line_cb (CALLBACK_ARGS);
-void     geometry_physical_add_surface_cb (CALLBACK_ARGS);
-void     geometry_physical_add_volume_cb (CALLBACK_ARGS);
 void geometry_edit_cb(CALLBACK_ARGS);
 void geometry_reload_cb(CALLBACK_ARGS); 
 
@@ -274,7 +238,6 @@ void mesh_define_cb(CALLBACK_ARGS);
 void mesh_1d_cb(CALLBACK_ARGS);
 void mesh_2d_cb(CALLBACK_ARGS); 
 void mesh_3d_cb(CALLBACK_ARGS); 
-void mesh_edit_cb(CALLBACK_ARGS);
 void mesh_delete_cb(CALLBACK_ARGS);
 void mesh_delete_parts_cb(CALLBACK_ARGS);
 void mesh_inspect_cb(CALLBACK_ARGS);
@@ -283,6 +246,8 @@ void mesh_update_edges_cb(CALLBACK_ARGS);
 void mesh_parameterize_cb(CALLBACK_ARGS);
 void mesh_degree_cb(CALLBACK_ARGS); 
 void mesh_optimize_cb(CALLBACK_ARGS); 
+void mesh_optimize_netgen_cb(CALLBACK_ARGS); 
+void mesh_classify_cb(CALLBACK_ARGS); 
 void mesh_define_length_cb (CALLBACK_ARGS);
 void mesh_define_recombine_cb (CALLBACK_ARGS);
 void mesh_define_transfinite_cb (CALLBACK_ARGS); 
