@@ -1,6 +1,6 @@
-// $Id: Colorbar_Window.cpp,v 1.58 2007-06-12 07:04:08 geuzaine Exp $
+// $Id: Colorbar_Window.cpp,v 1.65 2008-04-23 17:36:34 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 // Copyright (C) 1990 - 1995 Bill Hibbard, Brian Paul, Dave Santek,
 // and Andre Battaiola.
    
-#include "Gmsh.h"
 #include "GmshUI.h"
 #include "GUI.h"
 #include "ColorTable.h"
@@ -45,7 +44,7 @@ Colorbar_Window::Colorbar_Window(int x, int y, int w, int h, const char *l)
   ct = NULL;
   label = NULL;
   help_flag = 1;
-  font_height = GetFontSize();
+  font_height = GetFontSize() - 1; // use slightly smaller font
   marker_height = font_height;
   wedge_height = marker_height;
   marker_pos = 0;
@@ -223,10 +222,10 @@ void Colorbar_Window::redraw_range(int a, int b)
   // print colortable mode and help
   fl_font(FL_HELVETICA, font_height);
   fl_color(fl_contrast(FL_BLACK, color_bg));
-  int xx0 = 10, xx1 = 13 * font_height, yy0 = 10;
+  int xx0 = 6, xx1 = 11 * font_height, yy0 = 10;
   if(help_flag) {
     i = 0;
-    fl_draw("1, 2, ..., Ctrl+1, Ctrl+2, ...", xx0, yy0 + (i + 1) * font_height);
+    fl_draw("1, 2, ..., Ctrl+1, ...", xx0, yy0 + (i + 1) * font_height);
     fl_draw("Select predefined colormap", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("mouse1", xx0, yy0 + (i + 1) * font_height);
@@ -257,7 +256,7 @@ void Colorbar_Window::redraw_range(int a, int b)
     fl_draw("Invert abscissa or ordinate", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("up, down", xx0, yy0 + (i + 1) * font_height);
-    fl_draw("Modify curvature of color channels", xx1, yy0 + (i + 1) * font_height);
+    fl_draw("Modify color channel curvature", xx1, yy0 + (i + 1) * font_height);
     i++;
     fl_draw("a, Ctrl+a", xx0, yy0 + (i + 1) * font_height);
     fl_draw("Modify alpha coefficient", xx1, yy0 + (i + 1) * font_height);
@@ -320,16 +319,16 @@ void Colorbar_Window::draw()
   marker_y = label_y - marker_height - font_height;
   wedge_y = marker_y - wedge_height;
   color_bg = fl_color_cube(CTX.UNPACK_RED(CTX.color.bg) * FL_NUM_RED / 256,
-			   CTX.UNPACK_GREEN(CTX.color.bg) * FL_NUM_GREEN / 256,
-			   CTX.UNPACK_BLUE(CTX.color.bg) * FL_NUM_BLUE / 256);
+                           CTX.UNPACK_GREEN(CTX.color.bg) * FL_NUM_GREEN / 256,
+                           CTX.UNPACK_BLUE(CTX.color.bg) * FL_NUM_BLUE / 256);
   redraw_range(0, ct->size - 1);
   redraw_marker();
 }
 
 // Update
 
-void Colorbar_Window::update(char *name, double min, double max,
-                             GmshColorTable * table, int *changed)
+void Colorbar_Window::update(const char *name, double min, double max,
+                             GmshColorTable *table, bool *changed)
 {
   label = name;
   ct = table;
@@ -453,7 +452,7 @@ int Colorbar_Window::handle(int event)
     else if(Fl::test_shortcut(FL_CTRL + 'v')) {
       ColorTable_Paste(ct);
       redraw();
-      *viewchanged = 1;
+      *viewchanged = true;
     }
     else if(Fl::test_shortcut('h')) {
       help_flag = !help_flag;
@@ -545,7 +544,7 @@ int Colorbar_Window::handle(int event)
     if(compute) {
       ColorTable_Recompute(ct);
       redraw();
-      *viewchanged = 1;
+      *viewchanged = true;
       do_callback();
     }
     return 1;
@@ -659,7 +658,7 @@ int Colorbar_Window::handle(int event)
       else
         redraw_range(entry - 1, pentry + 1);
       pentry = entry;
-      *viewchanged = 1;
+      *viewchanged = true;
     }
     return 1;
   }
