@@ -1,7 +1,7 @@
 #ifndef _COLORBAR_WINDOW_H
 #define _COLORBAR_WINDOW_H
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ class Colorbar_Window : public Fl_Window {
   void redraw_marker();
 
   int font_height, marker_height, wedge_height;
-  char *label;
+  const char *label;
 
   double minval, maxval;  // min and max data values
   int wedge_y;     // top coord of color wedge
@@ -47,15 +47,14 @@ class Colorbar_Window : public Fl_Window {
   int help_flag;   // if nonzero, print help message
   int marker_pos;  // position of marker as index into table
   
-  GmshColorTable *ct; // pointer to the color table (allocated in Post_View)
-  int *viewchanged;   // pointer to changed bit in view
+  GmshColorTable *ct; // pointer to the color table (allocated in the view)
+  bool *viewchanged;   // pointer to changed bit in view
   Fl_Color color_bg;
 
 public:
-
-  Colorbar_Window(int x,int y,int w,int h,const char *l=0);
-  void update(char *name, double min, double max, GmshColorTable *ct, int *changed);
-
+  Colorbar_Window(int x, int y, int w, int h, const char *l=0);
+  void update(const char *name, double min, double max, GmshColorTable *ct,
+              bool *changed);
 };
 
 #endif
