@@ -1,7 +1,7 @@
 #ifndef _GUI_H_
 #define _GUI_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -55,16 +55,19 @@
 
 #include <vector>
 #include <string>
+#include <list>
 
 #include "Opengl_Window.h"
 #include "Colorbar_Window.h"
 #include "Popup_Button.h"
 #include "SpherePosition_Widget.h"
 
+#include "Field.h"
+
 // The dynamic contexts
 
 typedef struct{
-  char *label;
+  const char *label;
   Fl_Callback* callback;
   void *arg;
 } Context_Item;
@@ -108,6 +111,21 @@ struct PluginDialogBox
   Fl_Input *input[MAX_PLUGIN_OPTIONS];
 };
 
+class FieldDialogBox
+{
+  std::list<Fl_Widget*> inputs;
+  std::list<std::string> inputs_values;
+ public:
+  Field *current_field;
+  Fl_Group *group;
+  Fl_Menu_Button *put_on_view_btn;
+  Fl_Check_Button *set_size_btn;
+  Fl_Button *apply_btn,*revert_btn;
+  FieldDialogBox(Field *field, int x, int y, int width, int height, int fontsize);
+  void load_field(Field *field);
+  void save_values();
+};
+
 // The dialog for solvers
 
 struct SolverDialogBox
@@ -126,10 +144,16 @@ class GUI{
   int MH, fontsize;
   Fl_Scroll *m_scroll;
 
-  void add_multiline_in_browser(Fl_Browser *o, char* prefix, char *str);
+  void add_multiline_in_browser(Fl_Browser *o, const char *prefix, const char *str);
 
 public:
 
+  // graphic window
+  Fl_Window        *g_window;
+  Opengl_Window    *g_opengl_window;
+  Fl_Button        *g_status_butt[12];
+  Fl_Box           *g_status_label[2];
+
   // menu window
   Fl_Window        *m_window;
 #if defined(__APPLE__) && defined(HAVE_FLTK_1_1_5_OR_ABOVE)
@@ -145,12 +169,6 @@ public:
   std::vector<Popup_Button*>    m_popup2_butt;
   std::vector<char*>            m_pop_label;
 
-  // graphic window
-  Fl_Window        *g_window;
-  Opengl_Window    *g_opengl_window;
-  Fl_Button        *g_status_butt[12];
-  Fl_Box           *g_status_label[2];
-
   // Option window
   Fl_Window        *opt_window;
   Fl_Hold_Browser  *opt_browser;
@@ -211,6 +229,11 @@ public:
   Fl_Return_Button *view_ok;
   Fl_Menu_Button   *view_menu_butt[2];
 
+  // Field window
+  Fl_Window        *field_window;
+  Fl_Hold_Browser  *field_browser;
+  FieldDialogBox   *selected_field_dialog_box;
+
   // Plugin window
   Fl_Window        *plugin_window;
   Fl_Hold_Browser  *plugin_browser;
@@ -283,6 +306,8 @@ public:
   void create_view_options_window(int numview);
   void create_plugin_dialog_box(GMSH_Plugin *p, int x, int y, int width, int height);
   void create_plugin_window(int numview);
+  void create_field_dialog_box(Field *f, int x, int y, int width, int height);
+  void create_field_window(int numview);
   void create_visibility_window(bool redraw_only=false);
   void create_clip_window();
   void create_manip_window();
@@ -304,15 +329,16 @@ public:
   void set_size(int w, int h);
   void set_context(Context_Item menu[], int flag);
   int  get_context();
+  void update_views();
   void set_anim_buttons(int mode);
   void check_anim_buttons();
-  void set_status(char *msg, int num);
-  void add_message(char *msg);
-  void save_message(char *filename);
-  void fatal_error(char *filename);
+  void set_status(const char *msg, int num);
+  void add_message(const char *msg);
+  void save_message(const char *filename);
+  void fatal_error(const char *filename);
   void set_statistics(bool compute_quality);
   void update_view_window(int numview);
-  void set_title(char *str);
+  void set_title(const char *str);
   void add_handler();
   int  global_shortcuts(int event);
   int  arrow_shortcuts();
@@ -328,10 +354,10 @@ public:
 };
 
 // some utility font functions
-int GetFontIndex(char *fontname);
+int GetFontIndex(const char *fontname);
 int GetFontEnum(int index);
-char *GetFontName(int index);
-int GetFontAlign(char *alignstr);
+const char *GetFontName(int index);
+int GetFontAlign(const char *alignstr);
 int GetFontSize();
 
 #endif
