@@ -1,6 +1,6 @@
-// $Id: GUI_Extras.cpp,v 1.36 2007-06-22 08:07:46 geuzaine Exp $
+// $Id: GUI_Extras.cpp,v 1.47 2008-04-13 11:07:58 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,99 +19,124 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
 #include "GmshUI.h"
 #include "GmshDefines.h"
-#include "File_Picker.h"
 #include "CreateFile.h"
 #include "Options.h"
 #include "Context.h"
 #include "Draw.h"
 #include "GUI.h"
 #include "Shortcut_Window.h"
+#include "GModel.h"
 
 #include <FL/Fl_Value_Slider.H>
 #include <FL/Fl_Menu_Window.H>
+#include <FL/Fl_Select_Browser.H>
 #include <errno.h>
 
+#if defined(HAVE_NATIVE_FILE_CHOOSER)
+#include <FL/Fl_Native_File_Chooser.H>
+static Fl_Native_File_Chooser *fc = 0;
+#else
+#include "File_Picker.h"
+static File_Picker *fc = 0;
+#endif
+
 extern Context_T CTX;
 
 // File chooser
 
-static File_Picker *fc = NULL;
-
 int file_chooser(int multi, int create, const char *message,
-		 const char *filter, char *fname)
+                 const char *filter, const char *fname)
 {
   static char thefilter[1024] = "";
   static int thefilterindex = 0;
 
-  Fl_File_Chooser::show_label = "Format:";
-  Fl_File_Chooser::all_files_label = "All files (*)";
-
   if(strncmp(thefilter, filter, 1024)) {
     // reset the filter and the selection if the filter has changed
     strncpy(thefilter, filter, 1024);
     thefilterindex = 0;
   }
 
+#if defined(HAVE_NATIVE_FILE_CHOOSER)
+  if(!fc) fc = new Fl_Native_File_Chooser();
+  if(multi)
+    fc->type(Fl_Native_File_Chooser::BROWSE_MULTI_FILE);
+  else if(create)
+    fc->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
+  else
+    fc->type(Fl_Native_File_Chooser::BROWSE_FILE);
+  fc->title(message);
+  fc->filter(filter);
+  fc->filter_value(thefilterindex);
+  if(fname) fc->preset_file(fname);
+  int ret = 0;
+  switch(fc->show()) {
+  case -1: break; // error
+  case  1: break; // cancel
+  default: if(fc->filename()) ret = fc->count(); break;
+  }
+  thefilterindex = fc->filter_value();
+  // hack to clear the KEYDOWN state that remains when calling the
+  // file chooser on Mac and Windows using a keyboard shortcut
+  Fl::e_state = 0;
+  return ret;
+#else
+  Fl_File_Chooser::show_label = "Format:";
+  Fl_File_Chooser::all_files_label = "All files (*)";
   if(!fc) {
     fc = new File_Picker(getenv("PWD") ? "." : CTX.home_dir, thefilter, 
-			 Fl_File_Chooser::SINGLE, message);
+                         Fl_File_Chooser::SINGLE, message);
     fc->position(CTX.file_chooser_position[0], CTX.file_chooser_position[1]);
   }
-
   if(multi)
     fc->type(Fl_File_Chooser::MULTI);
   else if(create)
     fc->type(Fl_File_Chooser::CREATE);
   else
     fc->type(Fl_File_Chooser::SINGLE);
-
   fc->label(message);
   fc->filter(thefilter);
   fc->filter_value(thefilterindex);
-  if(fname)
-    fc->value(fname);
-
+  if(fname) fc->value(fname);
   fc->show();
-
-  while(fc->shown())
-    Fl::wait();
-
+  while(fc->shown()) Fl::wait();
   thefilterindex = fc->filter_value();
-
   if(fc->value())
     return fc->count();
   else
     return 0;
+#endif
 }
 
-char *file_chooser_get_name(int num)
+std::string file_chooser_get_name(int num)
 {
-  if(!fc)
-    return "";
-  return (char *)fc->value(num);
+  if(!fc) return "";
+#if defined(HAVE_NATIVE_FILE_CHOOSER)
+  return std::string(fc->filename(num - 1));
+#else
+  return std::string(fc->value(num));
+#endif
 }
 
 int file_chooser_get_filter()
 {
-  if(!fc)
-    return 0;
+  if(!fc) return 0;
   return fc->filter_value();
 }
 
 void file_chooser_get_position(int *x, int *y)
 {
-  if(!fc)
-    return;
+  if(!fc) return;
+#if !defined(HAVE_NATIVE_FILE_CHOOSER)
   *x = fc->x();
   *y = fc->y();
+#endif
 }
 
 // Arrow editor
 
-int arrow_editor(char *title, double &a, double &b, double &c)
+int arrow_editor(const char *title, double &a, double &b, double &c)
 {
   struct _editor{
     Fl_Window *window;
@@ -126,7 +151,7 @@ int arrow_editor(char *title, double &a, double &b, double &c)
 
   if(!editor){
     editor = new _editor;
-    editor->window = new Dialog_Window(2 * BB + 3 * WB, 4 * BH + 3 * WB);
+    editor->window = new Dialog_Window(2 * BB + 3 * WB, 4 * BH + 3 * WB, CTX.non_modal_windows);
     editor->sa = new Fl_Value_Slider(WB, WB, BB, BH, "Head radius");
     editor->sa->type(FL_HOR_SLIDER);
     editor->sa->align(FL_ALIGN_RIGHT);
@@ -154,14 +179,14 @@ int arrow_editor(char *title, double &a, double &b, double &c)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == editor->apply) {
-	a = editor->sa->value();
-	b = editor->sb->value();
-	c = editor->sc->value();
-	return 1;
+        a = editor->sa->value();
+        b = editor->sb->value();
+        c = editor->sc->value();
+        return 1;
       }
       if (o == editor->window || o == editor->cancel){
-	editor->window->hide();
-	return 0;
+        editor->window->hide();
+        return 0;
       }
     }
   }
@@ -170,7 +195,8 @@ int arrow_editor(char *title, double &a, double &b, double &c)
 
 // Perspective editor (aka z-clipping planes factor slider)
 
-static void persp_change_factor(Fl_Widget* w, void* data){
+static void persp_change_factor(Fl_Widget* w, void* data)
+{
   opt_general_clip_factor(0, GMSH_SET|GMSH_GUI, ((Fl_Slider*)w)->value());
   Draw();
 }
@@ -180,7 +206,7 @@ class Release_Slider : public Fl_Slider {
     switch (event) {
     case FL_RELEASE: 
       if(window())
-	window()->hide();
+        window()->hide();
       return 1;
     default:
       return Fl_Slider::handle(event);
@@ -202,6 +228,7 @@ int perspective_editor()
   if(!editor){
     editor = new _editor;
     editor->window = new Fl_Menu_Window(20, 100);
+    if(CTX.non_modal_windows) editor->window->set_non_modal();
     editor->sa = new Release_Slider(0, 0, 20, 100);
     editor->sa->type(FL_VERT_NICE_SLIDER);
     editor->sa->minimum(12);
@@ -213,13 +240,60 @@ int perspective_editor()
 
   editor->window->hotspot(editor->window);
   editor->sa->value(CTX.clip_factor);
+
+  if(editor->window->non_modal() && !editor->window->shown())
+    editor->window->show(); // fix ordering
   editor->window->show();
   return 0;
 }
 
+// Model chooser
+
+static void model_switch(Fl_Widget* w, void* data)
+{
+  Fl_Select_Browser *b = (Fl_Select_Browser *)w;
+  GModel::current(b->value() - 1);
+  if(w->window()) w->window()->hide();
+  CTX.mesh.changed = ENT_ALL;
+  Draw();
+}
+
+int model_chooser()
+{
+  struct _menu{
+    Fl_Menu_Window *window;
+    Fl_Select_Browser *browser;
+  };
+  static _menu *menu = NULL;
+
+  if(!menu){
+    menu = new _menu;
+    menu->window = new Fl_Menu_Window(200, 100);
+    if(CTX.non_modal_windows) menu->window->set_non_modal();
+    menu->window->border(0);
+    menu->browser = new Fl_Select_Browser(0, 0, 200, 100);
+    menu->browser->callback(model_switch);
+    menu->browser->when(FL_WHEN_RELEASE_ALWAYS);
+    menu->window->end();
+  }
+
+  menu->window->hotspot(menu->window);
+  menu->browser->clear();
+  for(unsigned int i = 0; i < GModel::list.size(); i++){
+    char tmp[256];
+    sprintf(tmp, "Model %d <<%s>>", i, GModel::list[i]->getName().c_str());
+    menu->browser->add(tmp);
+  }
+
+  if(menu->window->non_modal() && !menu->window->shown())
+    menu->window->show(); // fix ordering
+  menu->window->show();
+  return 0;
+}
+
 // Generic save bitmap dialog
 
-int generic_bitmap_dialog(char *name, char *title, int format)
+int generic_bitmap_dialog(const char *name, const char *title, int format)
 {
   struct _generic_bitmap_dialog{
     Fl_Window *window;
@@ -257,14 +331,14 @@ int generic_bitmap_dialog(char *name, char *title, int format)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_print_text(0, GMSH_SET | GMSH_GUI, (int)dialog->b->value());
-	CreateOutputFile(name, format);
-	dialog->window->hide();
-	return 1;
+        opt_print_text(0, GMSH_SET | GMSH_GUI, (int)dialog->b->value());
+        CreateOutputFile(name, format);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -273,7 +347,7 @@ int generic_bitmap_dialog(char *name, char *title, int format)
 
 // TeX dialog
 
-int latex_dialog(char *name)
+int latex_dialog(const char *name)
 {
   struct _latex_dialog{
     Fl_Window *window;
@@ -310,14 +384,14 @@ int latex_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_print_tex_as_equation(0, GMSH_SET | GMSH_GUI, (int)dialog->b->value());
-	CreateOutputFile(name, FORMAT_TEX);
-	dialog->window->hide();
-	return 1;
+        opt_print_tex_as_equation(0, GMSH_SET | GMSH_GUI, (int)dialog->b->value());
+        CreateOutputFile(name, FORMAT_TEX);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -326,7 +400,7 @@ int latex_dialog(char *name)
 
 // Save jpeg dialog
 
-int jpeg_dialog(char *name)
+int jpeg_dialog(const char *name)
 {
   struct _jpeg_dialog{
     Fl_Window *window;
@@ -378,16 +452,16 @@ int jpeg_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_print_jpeg_quality(0, GMSH_SET | GMSH_GUI, (int)dialog->s[0]->value());
-	opt_print_jpeg_smoothing(0, GMSH_SET | GMSH_GUI, (int)dialog->s[1]->value());
-	opt_print_text(0, GMSH_SET | GMSH_GUI, (int)dialog->b->value());
-	CreateOutputFile(name, FORMAT_JPEG);
-	dialog->window->hide();
-	return 1;
+        opt_print_jpeg_quality(0, GMSH_SET | GMSH_GUI, (int)dialog->s[0]->value());
+        opt_print_jpeg_smoothing(0, GMSH_SET | GMSH_GUI, (int)dialog->s[1]->value());
+        opt_print_text(0, GMSH_SET | GMSH_GUI, (int)dialog->b->value());
+        CreateOutputFile(name, FORMAT_JPEG);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -396,7 +470,7 @@ int jpeg_dialog(char *name)
 
 // Save gif dialog
 
-int gif_dialog(char *name)
+int gif_dialog(const char *name)
 {
   struct _gif_dialog{
     Fl_Window *window;
@@ -443,18 +517,18 @@ int gif_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_print_gif_dither(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value());
-	opt_print_gif_interlace(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value());
-	opt_print_gif_sort(0, GMSH_SET | GMSH_GUI, dialog->b[2]->value());
-	opt_print_gif_transparent(0, GMSH_SET | GMSH_GUI, dialog->b[3]->value());
-	opt_print_text(0, GMSH_SET | GMSH_GUI, dialog->b[4]->value());
-	CreateOutputFile(name, FORMAT_GIF);
-	dialog->window->hide();
-	return 1;
+        opt_print_gif_dither(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value());
+        opt_print_gif_interlace(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value());
+        opt_print_gif_sort(0, GMSH_SET | GMSH_GUI, dialog->b[2]->value());
+        opt_print_gif_transparent(0, GMSH_SET | GMSH_GUI, dialog->b[3]->value());
+        opt_print_text(0, GMSH_SET | GMSH_GUI, dialog->b[4]->value());
+        CreateOutputFile(name, FORMAT_GIF);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -499,7 +573,7 @@ static void activate_gl2ps_choices(int format, int quality, Fl_Check_Button *b[5
   }
 }
 
-int gl2ps_dialog(char *name, char *title, int format)
+int gl2ps_dialog(const char *name, const char *title, int format)
 {
   struct _gl2ps_dialog{
     Fl_Window *window;
@@ -566,23 +640,23 @@ int gl2ps_dialog(char *name, char *title, int format)
       if (!o) break;
 
       if (o == dialog->c){
-	activate_gl2ps_choices(format, dialog->c->value(), dialog->b);
+        activate_gl2ps_choices(format, dialog->c->value(), dialog->b);
       }
       if (o == dialog->ok) {
-	opt_print_eps_quality(0, GMSH_SET | GMSH_GUI, dialog->c->value());
-	opt_print_eps_compress(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value());
-	opt_print_eps_background(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value());
-	opt_print_eps_occlusion_culling(0, GMSH_SET | GMSH_GUI, dialog->b[2]->value());
-	opt_print_eps_best_root(0, GMSH_SET | GMSH_GUI, dialog->b[3]->value());
-	opt_print_eps_ps3shading(0, GMSH_SET | GMSH_GUI, dialog->b[4]->value());
-	opt_print_text(0, GMSH_SET | GMSH_GUI, dialog->b[5]->value());
-	CreateOutputFile(name, format);
-	dialog->window->hide();
-	return 1;
+        opt_print_eps_quality(0, GMSH_SET | GMSH_GUI, dialog->c->value());
+        opt_print_eps_compress(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value());
+        opt_print_eps_background(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value());
+        opt_print_eps_occlusion_culling(0, GMSH_SET | GMSH_GUI, dialog->b[2]->value());
+        opt_print_eps_best_root(0, GMSH_SET | GMSH_GUI, dialog->b[3]->value());
+        opt_print_eps_ps3shading(0, GMSH_SET | GMSH_GUI, dialog->b[4]->value());
+        opt_print_text(0, GMSH_SET | GMSH_GUI, dialog->b[5]->value());
+        CreateOutputFile(name, format);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -591,7 +665,7 @@ int gl2ps_dialog(char *name, char *title, int format)
 
 // Save options dialog
 
-int options_dialog(char *name)
+int options_dialog(const char *name)
 {
   struct _options_dialog{
     Fl_Window *window;
@@ -631,13 +705,13 @@ int options_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	Print_Options(0, GMSH_FULLRC, dialog->b[0]->value(), dialog->b[1]->value(), name);
-	dialog->window->hide();
-	return 1;
+        Print_Options(0, GMSH_FULLRC, dialog->b[0]->value(), dialog->b[1]->value(), name);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -646,7 +720,7 @@ int options_dialog(char *name)
 
 // geo dialog
 
-int geo_dialog(char *name)
+int geo_dialog(const char *name)
 {
   struct _geo_dialog{
     Fl_Window *window;
@@ -683,14 +757,81 @@ int geo_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_print_geo_labels(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
-	CreateOutputFile(name, FORMAT_GEO);
-	dialog->window->hide();
-	return 1;
+        opt_print_geo_labels(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
+        CreateOutputFile(name, FORMAT_GEO);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
+      }
+    }
+  }
+  return 0;
+}
+
+int pos_dialog(const char *name)
+{
+  struct _pos_dialog{
+    Fl_Window *window;
+    Fl_Check_Button *b[6];
+    Fl_Button *ok, *cancel;
+  };
+  static _pos_dialog *dialog = NULL;
+
+  const int BH = 2 * GetFontSize() + 1;
+  const int BB = 7 * GetFontSize() + 9;
+  const int WB = 7;
+
+  if(!dialog){
+    dialog = new _pos_dialog;
+    int h = 3 * WB + 7 * BH, w = 2 * BB + 3 * WB, y = WB;
+    // not a "Dialog_Window" since it is modal 
+    dialog->window = new Fl_Double_Window(w, h, "POS Options");
+    dialog->window->box(GMSH_WINDOW_BOX);
+    dialog->b[0] = new Fl_Check_Button(WB, y, 2 * BB + WB, BH, "Save all (ignore physical groups)"); y += BH;
+    dialog->b[1] = new Fl_Check_Button(WB, y, 2 * BB + WB, BH, "Print elementary tags"); y += BH;
+    dialog->b[2] = new Fl_Check_Button(WB, y, 2 * BB + WB, BH, "Print element numbers"); y += BH;
+    dialog->b[3] = new Fl_Check_Button(WB, y, 2 * BB + WB, BH, "Print Gamma quality measure"); y += BH;
+    dialog->b[4] = new Fl_Check_Button(WB, y, 2 * BB + WB, BH, "Print Eta quality measure"); y += BH;
+    dialog->b[5] = new Fl_Check_Button(WB, y, 2 * BB + WB, BH, "Print Rho quality measure"); y += BH;
+    for(int i = 0; i < 5; i++)
+      dialog->b[i]->type(FL_TOGGLE_BUTTON);
+    dialog->ok = new Fl_Return_Button(WB, y + WB, BB, BH, "OK");
+    dialog->cancel = new Fl_Button(2 * WB + BB, y + WB, BB, BH, "Cancel");
+    dialog->window->set_modal();
+    dialog->window->end();
+    dialog->window->hotspot(dialog->window);
+  }
+  
+  dialog->b[0]->value(CTX.mesh.save_all ? 1 : 0);
+  dialog->b[1]->value(CTX.print.pos_elementary ? 1 : 0);
+  dialog->b[2]->value(CTX.print.pos_element ? 1 : 0);
+  dialog->b[3]->value(CTX.print.pos_gamma ? 1 : 0);
+  dialog->b[4]->value(CTX.print.pos_eta ? 1 : 0);
+  dialog->b[5]->value(CTX.print.pos_rho ? 1 : 0);
+  dialog->window->show();
+
+  while(dialog->window->shown()){
+    Fl::wait();
+    for (;;) {
+      Fl_Widget* o = Fl::readqueue();
+      if (!o) break;
+      if (o == dialog->ok) {
+        opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value() ? 1 : 0);
+        opt_print_pos_elementary(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value() ? 1 : 0);
+        opt_print_pos_element(0, GMSH_SET | GMSH_GUI, dialog->b[2]->value() ? 1 : 0);
+        opt_print_pos_gamma(0, GMSH_SET | GMSH_GUI, dialog->b[3]->value() ? 1 : 0);
+        opt_print_pos_eta(0, GMSH_SET | GMSH_GUI, dialog->b[4]->value() ? 1 : 0);
+        opt_print_pos_rho(0, GMSH_SET | GMSH_GUI, dialog->b[5]->value() ? 1 : 0);
+        CreateOutputFile(name, FORMAT_POS);
+        dialog->window->hide();
+        return 1;
+      }
+      if (o == dialog->window || o == dialog->cancel){
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -699,7 +840,7 @@ int geo_dialog(char *name)
 
 // Generic save mesh dialog
 
-int generic_mesh_dialog(char *name, char *title, int format)
+int generic_mesh_dialog(const char *name, const char *title, int format)
 {
   struct _generic_mesh_dialog{
     Fl_Window *window;
@@ -737,14 +878,14 @@ int generic_mesh_dialog(char *name, char *title, int format)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
-	CreateOutputFile(name, format);
-	dialog->window->hide();
-	return 1;
+        opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
+        CreateOutputFile(name, format);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -753,7 +894,7 @@ int generic_mesh_dialog(char *name, char *title, int format)
 
 // Save msh dialog
 
-int msh_dialog(char *name)
+int msh_dialog(const char *name)
 {
   struct _msh_dialog{
     Fl_Window *window;
@@ -793,7 +934,7 @@ int msh_dialog(char *name)
   }
   
   dialog->c->value((CTX.mesh.msh_file_version == 1.0) ? 0 : 
-		   CTX.mesh.msh_binary ? 2 : 1);
+                   CTX.mesh.msh_binary ? 2 : 1);
   dialog->b->value(CTX.mesh.save_all ? 1 : 0);
   dialog->window->show();
 
@@ -803,18 +944,18 @@ int msh_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_mesh_msh_file_version(0, GMSH_SET | GMSH_GUI, 
-				  (dialog->c->value() == 0) ? 1. : 2.);
-	opt_mesh_msh_binary(0, GMSH_SET | GMSH_GUI, 
-			    (dialog->c->value() == 2) ? 1 : 0);
-	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
-	CreateOutputFile(name, FORMAT_MSH);
-	dialog->window->hide();
-	return 1;
+        opt_mesh_msh_file_version(0, GMSH_SET | GMSH_GUI, 
+                                  (dialog->c->value() == 0) ? 1. : 2.);
+        opt_mesh_msh_binary(0, GMSH_SET | GMSH_GUI, 
+                            (dialog->c->value() == 2) ? 1 : 0);
+        opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
+        CreateOutputFile(name, FORMAT_MSH);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -823,7 +964,7 @@ int msh_dialog(char *name)
 
 // unv mesh dialog
 
-int unv_dialog(char *name)
+int unv_dialog(const char *name)
 {
   struct _unv_dialog{
     Fl_Window *window;
@@ -863,15 +1004,15 @@ int unv_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value() ? 1 : 0);
-	opt_mesh_save_groups_of_nodes(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value() ? 1 : 0);
-	CreateOutputFile(name, FORMAT_UNV);
-	dialog->window->hide();
-	return 1;
+        opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value() ? 1 : 0);
+        opt_mesh_save_groups_of_nodes(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value() ? 1 : 0);
+        CreateOutputFile(name, FORMAT_UNV);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -880,7 +1021,7 @@ int unv_dialog(char *name)
 
 // Save bdf dialog
 
-int bdf_dialog(char *name)
+int bdf_dialog(const char *name)
 {
   struct _bdf_dialog{
     Fl_Window *window;
@@ -929,15 +1070,15 @@ int bdf_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_mesh_bdf_field_format(0, GMSH_SET | GMSH_GUI, dialog->c->value());
-	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
-	CreateOutputFile(name, FORMAT_BDF);
-	dialog->window->hide();
-	return 1;
+        opt_mesh_bdf_field_format(0, GMSH_SET | GMSH_GUI, dialog->c->value());
+        opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
+        CreateOutputFile(name, FORMAT_BDF);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
@@ -946,7 +1087,7 @@ int bdf_dialog(char *name)
 
 // Save stl dialog
 
-int stl_dialog(char *name)
+int stl_dialog(const char *name)
 {
   struct _stl_dialog{
     Fl_Window *window;
@@ -994,15 +1135,15 @@ int stl_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	opt_mesh_stl_binary(0, GMSH_SET | GMSH_GUI, dialog->c->value());
-	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
-	CreateOutputFile(name, FORMAT_STL);
-	dialog->window->hide();
-	return 1;
+        opt_mesh_stl_binary(0, GMSH_SET | GMSH_GUI, dialog->c->value());
+        opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value() ? 1 : 0);
+        CreateOutputFile(name, FORMAT_STL);
+        dialog->window->hide();
+        return 1;
       }
       if (o == dialog->window || o == dialog->cancel){
-	dialog->window->hide();
-	return 0;
+        dialog->window->hide();
+        return 0;
       }
     }
   }
