@@ -1,6 +1,6 @@
-// $Id: Opengl.cpp,v 1.73 2007-06-22 08:07:46 geuzaine Exp $
+// $Id: Opengl.cpp,v 1.80 2008-03-20 11:44:03 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,12 +19,13 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
 #include "GmshUI.h"
+#include "GmshDefines.h"
 #include "Numeric.h"
 #include "Context.h"
 #include "Draw.h"
 #include "SelectBuffer.h"
+#include "StringUtils.h"
 #include "GUI.h"
 #include "gl2ps.h"
 
@@ -45,35 +46,7 @@ void Draw(void)
   WID->redraw_opengl();
 }
 
-void SanitizeTeXString(char *in, char *out)
-{
-  // if there is a '$' or a '\' in the string, assume the author knows
-  // what he's doing:
-  if(strstr(in, "$") || strstr(in, "\\")){
-    strcpy(out, in);
-    return;
-  }
-
-  if(CTX.print.tex_as_equation) *out++ = '$';
-
-  // otherwise, escape the following special characters:
-  char bad[8] = { '%', '^', '#', '%', '&', '_', '{', '}' };
-  while(*in){
-    for(unsigned int i = 0; i < sizeof(bad); i++){
-      if(*in == bad[i]){
-	*out++ = '\\';
-	break;
-      }
-    }
-    *out++ = *in++;
-  }
-
-  if(CTX.print.tex_as_equation) *out++ = '$';
-
-  *out = '\0';
-}
-
-void Draw_String(char *s, char *font_name, int font_enum, int font_size, int align)
+void Draw_String(const char *s, const char *font_name, int font_enum, int font_size, int align)
 {
   if(CTX.printing && !CTX.print.text) return;
 
@@ -110,8 +83,7 @@ void Draw_String(char *s, char *font_name, int font_enum, int font_size, int ali
   }
   else{
     if(CTX.print.format == FORMAT_TEX){
-      char tmp[1024];
-      SanitizeTeXString(s, tmp);
+      std::string tmp = sanitizeTeXString(s, CTX.print.tex_as_equation);
       int opt;
       switch(align){
       case 1: opt = GL2PS_TEXT_B;   break; // bottom center
@@ -124,12 +96,12 @@ void Draw_String(char *s, char *font_name, int font_enum, int font_size, int ali
       case 8: opt = GL2PS_TEXT_CR;  break; // center right
       default: opt = GL2PS_TEXT_BL; break; // bottom left
       }
-      gl2psTextOpt(tmp, font_name, font_size, opt, 0.);
+      gl2psTextOpt(tmp.c_str(), font_name, font_size, opt, 0.);
     }
     else if(CTX.print.eps_quality && (CTX.print.format == FORMAT_PS ||
-				      CTX.print.format == FORMAT_EPS ||
-				      CTX.print.format == FORMAT_PDF ||
-				      CTX.print.format == FORMAT_SVG)){
+                                      CTX.print.format == FORMAT_EPS ||
+                                      CTX.print.format == FORMAT_PDF ||
+                                      CTX.print.format == FORMAT_SVG)){
       gl2psText(s, font_name, font_size);
     }
     else{
@@ -139,22 +111,22 @@ void Draw_String(char *s, char *font_name, int font_enum, int font_size, int ali
   }
 }
 
-void Draw_String(char *s)
+void Draw_String(const char *s)
 {
   Draw_String(s, CTX.gl_font, CTX.gl_font_enum, CTX.gl_fontsize, 0);
 }
 
-void Draw_String_Center(char *s)
+void Draw_String_Center(const char *s)
 {
   Draw_String(s, CTX.gl_font, CTX.gl_font_enum, CTX.gl_fontsize, 1);
 }
 
-void Draw_String_Right(char *s)
+void Draw_String_Right(const char *s)
 {
   Draw_String(s, CTX.gl_font, CTX.gl_font_enum, CTX.gl_fontsize, 2);
 }
 
-void Draw_String(char *s, double style)
+void Draw_String(const char *s, double style)
 {
   unsigned int bits = (unsigned int)style;
 
@@ -166,7 +138,7 @@ void Draw_String(char *s, double style)
     int font = (bits>>8 & 0xff);
     int align = (bits>>16 & 0xff);
     int font_enum = GetFontEnum(font);
-    char *font_name = GetFontName(font);
+    const char *font_name = GetFontName(font);
     if(!size) size = CTX.gl_fontsize;
     Draw_String(s, font_name, font_enum, size, align);
   }
@@ -195,11 +167,11 @@ void Draw_OnScreenMessages()
 // Select entity routine
 
 char SelectEntity(int type, 
-		  std::vector<GVertex*> &vertices,
-		  std::vector<GEdge*> &edges,
-		  std::vector<GFace*> &faces,
-		  std::vector<GRegion*> &regions,
-		  std::vector<MElement*> &elements)
+                  std::vector<GVertex*> &vertices,
+                  std::vector<GEdge*> &edges,
+                  std::vector<GFace*> &faces,
+                  std::vector<GRegion*> &regions,
+                  std::vector<MElement*> &elements)
 {
   if(!WID) return 'q';
 
@@ -237,27 +209,31 @@ char SelectEntity(int type,
       WID->undo_selection = 0;
       return 'u';
     }
+    if(WID->invert_selection) {
+      WID->invert_selection = 0;
+      return 'i';
+    }
     if(WID->try_selection) {
       bool add = (WID->try_selection > 0) ? true : false;
       bool multi = (abs(WID->try_selection) > 1) ? true : false;
       WID->try_selection = 0;
       if(WID->selection == ENT_NONE){ // just report the mouse click
-	WID->g_opengl_window->SelectionMode = false;
-	return 'c';
+        WID->g_opengl_window->SelectionMode = false;
+        return 'c';
       }
       else if(ProcessSelectionBuffer(WID->selection, multi, true,
-				     WID->try_selection_xywh[0],
-				     WID->try_selection_xywh[1], 
-				     WID->try_selection_xywh[2],
-				     WID->try_selection_xywh[3], 
-				     vertices, edges, faces, regions,
-				     elements)){
-	WID->selection = ENT_NONE;
-	WID->g_opengl_window->SelectionMode = false;
-	if(add)
-	  return 'l';
-	else
-	  return 'r';
+                                     WID->try_selection_xywh[0],
+                                     WID->try_selection_xywh[1], 
+                                     WID->try_selection_xywh[2],
+                                     WID->try_selection_xywh[3], 
+                                     vertices, edges, faces, regions,
+                                     elements)){
+        WID->selection = ENT_NONE;
+        WID->g_opengl_window->SelectionMode = false;
+        if(add)
+          return 'l';
+        else
+          return 'r';
       }
     }
   }
