@@ -1,7 +1,7 @@
 #ifndef _SHORTCUT_WINDOW_H
 #define _SHORTCUT_WINDOW_H
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "GmshUI.h"
-
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Double_Window.H>
 #include <FL/fl_ask.H>
@@ -41,18 +40,24 @@ class Dialog_Window : public Fl_Double_Window {
 #else
       if(Fl::test_shortcut(FL_CTRL+'w') || Fl::test_shortcut(FL_Escape)){
 #endif
-	do_callback();
-	return 1;
+        do_callback();
+        return 1;
       }
       break;
     }
     return Fl_Double_Window::handle(event);
   }
  public:
-  Dialog_Window(int x,int y,int w,int h,const char *l=0) :
-    Fl_Double_Window(x, y, w, h, l) {}
-  Dialog_Window(int w,int h,const char *l=0) :
-    Fl_Double_Window(w, h, l) {}
+  Dialog_Window(int w,int h,int nonmodal=false,const char *l=0)
+    : Fl_Double_Window(w, h, l) 
+  {
+    if(nonmodal) set_non_modal();
+  }
+  void show()
+  {
+    if(non_modal() && !shown()) Fl_Double_Window::show(); // fix ordering
+    Fl_Double_Window::show();
+  }
 };
 
 // Do the same for the main windows, but ask if we really want to quit
@@ -73,19 +78,25 @@ class Main_Window : public Fl_Window {
 #else
       if(Fl::test_shortcut(FL_CTRL+'w')){
 #endif
-	if(fl_choice("Do you really want to quit?", "Cancel", "Quit", NULL))
-	  do_callback();
-	return 1;
+        if(fl_choice("Do you really want to quit?", "Cancel", "Quit", NULL))
+          do_callback();
+        return 1;
       }
       break;
     }
     return Fl_Window::handle(event);
   }
  public:
-  Main_Window(int x,int y,int w,int h,const char *l=0) :
-    Fl_Window(x, y, w, h, l) {}
-  Main_Window(int w,int h,const char *l=0) :
-    Fl_Window(w, h, l) {}
+  Main_Window(int w,int h,bool nonmodal=false, const char *l=0) 
+    : Fl_Window(w, h, l) 
+  {
+    if(nonmodal) set_non_modal();
+  }
+  void show()
+  {
+    if(non_modal() && !shown()) Fl_Window::show(); // fix ordering
+    Fl_Window::show();
+  }
 };
 
 #endif
