@@ -1,6 +1,6 @@
-// $Id: Solvers.cpp,v 1.55 2006-12-16 15:44:28 geuzaine Exp $
+// $Id: Solvers.cpp,v 1.64 2008-05-06 21:11:47 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,12 +19,14 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
+#include <string.h>
+#include "Message.h"
 #include "Solvers.h"
 #include "GmshServer.h"
 #include "OpenFile.h"
 #include "GmshUI.h"
 #include "GUI.h"
+#include "PView.h"
 #include "Draw.h"
 #include "Context.h"
 
@@ -66,7 +68,7 @@ int WaitForData(int socket, int num, double waitint)
     else{ 
       // an error happened
       if(num >= 0)
-	SINFO[num].pid = -1;
+        SINFO[num].pid = -1;
       return 1;
     }
   }
@@ -75,7 +77,7 @@ int WaitForData(int socket, int num, double waitint)
 // This routine either launches a solver and waits for some answer (if
 // num >= 0), or simply waits for messages (if num < 0)
 
-int Solver(int num, char *args)
+int Solver(int num, const char *args)
 {
   char command[1024], sockname[1024], prog[1024], tmp[1024], tmp2[1024];
 
@@ -96,7 +98,7 @@ int Solver(int num, char *args)
   }
   else{
     if(!CTX.solver.listen){
-      Msg(INFO, "Stopped listening for solver connections");
+      Msg::Info("Stopped listening for solver connections");
       return 0;
     }
     // we don't know who will (maybe) contact us
@@ -131,36 +133,36 @@ int Solver(int num, char *args)
   if(sock < 0) {
     switch (sock) {
     case -1:
-      Msg(GERROR, "Couldn't create socket '%s'", sockname);
+      Msg::Error("Couldn't create socket '%s'", sockname);
       break;
     case -2:
-      Msg(GERROR, "Couldn't bind socket to name '%s'", sockname);
+      Msg::Error("Couldn't bind socket to name '%s'", sockname);
       break;
     case -3:
-      Msg(GERROR, "Socket listen failed on '%s'", sockname);
+      Msg::Error("Socket listen failed on '%s'", sockname);
       break;
     case -4:
-      Msg(GERROR, "Socket listen timeout on '%s'", sockname);
-      Msg(GERROR, "Is '%s' correctly installed?", prog);
+      Msg::Error("Socket listen timeout on '%s'", sockname);
+      Msg::Error("Is '%s' correctly installed?", prog);
       break;
     case -5:
-      Msg(GERROR, "Socket accept failed on '%s'", sockname);
+      Msg::Error("Socket accept failed on '%s'", sockname);
       break;
     case -6:
-      Msg(INFO, "Stopped listening for solver connections");
+      Msg::Info("Stopped listening for solver connections");
       server.StopClient();
       break;
     case -7:
-      Msg(GERROR, "Unix sockets not available on Windows without Cygwin");
-      Msg(GERROR, "Use TCP/IP sockets instead");
+      Msg::Error("Unix sockets not available on Windows without Cygwin");
+      Msg::Error("Use TCP/IP sockets instead");
       break;
     case -8:
-      Msg(GERROR, "Could not initialize Windows sockets");
+      Msg::Error("Could not initialize Windows sockets");
       break;
     }
     if(num >= 0){
       for(int i = 0; i < SINFO[num].nboptions; i++)
-	WID->solver[num].choice[i]->clear();
+        WID->solver[num].choice[i]->clear();
     }
     return 0;
   }
@@ -171,7 +173,7 @@ int Solver(int num, char *args)
     SINFO[num].pid = 0;
   }
 
-  Msg(STATUS2N, "Running '%s'", prog);
+  Msg::StatusBar(2, false, "Running '%s'", prog);
 
   while(1) {
 
@@ -189,73 +191,75 @@ int Solver(int num, char *args)
     if(server.ReceiveMessageHeader(&type, &length)){
       char *message = new char[length + 1];
       if(server.ReceiveMessageBody(length, message)){
-	switch (type) {
-	case GmshServer::CLIENT_START:
-	  if(num >= 0)
-	    SINFO[num].pid = atoi(message);
-	  break;
-	case GmshServer::CLIENT_STOP:
-	  stop = 1;
-	  if(num >= 0)
-	    SINFO[num].pid = -1;
-	  break;
-	case GmshServer::CLIENT_PROGRESS:
-	  if(num >= 0)
-	    Msg(STATUS2N, "%s %s", SINFO[num].name, message);
-	  else
-	    Msg(STATUS2N, "%s", message);
-	  break;
-	case GmshServer::CLIENT_OPTION_1:
-	  if(num >= 0)
-	    strcpy(SINFO[num].option[0][SINFO[num].nbval[0]++], message);
-	  break;
-	case GmshServer::CLIENT_OPTION_2:
-	  if(num >= 0)
-	    strcpy(SINFO[num].option[1][SINFO[num].nbval[1]++], message);
-	  break;
-	case GmshServer::CLIENT_OPTION_3:
-	  if(num >= 0)
-	    strcpy(SINFO[num].option[2][SINFO[num].nbval[2]++], message);
-	  break;
-	case GmshServer::CLIENT_OPTION_4:
-	  if(num >= 0)
-	    strcpy(SINFO[num].option[3][SINFO[num].nbval[3]++], message);
-	  break;
-	case GmshServer::CLIENT_OPTION_5:
-	  if(num >= 0)
-	    strcpy(SINFO[num].option[4][SINFO[num].nbval[4]++], message);
-	  break;
-	case GmshServer::CLIENT_MERGE_FILE:
-	  if(num < 0 || (num >= 0 && SINFO[num].merge_views)) {
-	    int n = List_Nbr(CTX.post.list);
-	    MergeFile(message);
-	    Draw();
-	    if(n != List_Nbr(CTX.post.list))
-	      WID->set_context(menu_post, 0);
-	  }
-	  break;
-	case GmshServer::CLIENT_PARSE_STRING:
-	  ParseString(message);
-	  Draw();
-	  break;
-	case GmshServer::CLIENT_INFO:
-	  Msg(SOLVER, "%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
-	  break;
-	case GmshServer::CLIENT_WARNING:
-	case GmshServer::CLIENT_ERROR:
-	  Msg(SOLVERR, "%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
-	  break;
-	default:
-	  Msg(WARNING, "Unknown type of message received from %s",
-	      num >= 0 ? SINFO[num].name : "client");
-	  Msg(SOLVER, "%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
-	  break;
-	}
-	WID->check();
+        switch (type) {
+        case GmshServer::CLIENT_START:
+          if(num >= 0)
+            SINFO[num].pid = atoi(message);
+          break;
+        case GmshServer::CLIENT_STOP:
+          stop = 1;
+          if(num >= 0)
+            SINFO[num].pid = -1;
+          break;
+        case GmshServer::CLIENT_PROGRESS:
+          if(num >= 0)
+            Msg::StatusBar(2, false, "%s %s", SINFO[num].name, message);
+          else
+            Msg::StatusBar(2, false, "%s", message);
+          break;
+        case GmshServer::CLIENT_OPTION_1:
+          if(num >= 0)
+            strcpy(SINFO[num].option[0][SINFO[num].nbval[0]++], message);
+          break;
+        case GmshServer::CLIENT_OPTION_2:
+          if(num >= 0)
+            strcpy(SINFO[num].option[1][SINFO[num].nbval[1]++], message);
+          break;
+        case GmshServer::CLIENT_OPTION_3:
+          if(num >= 0)
+            strcpy(SINFO[num].option[2][SINFO[num].nbval[2]++], message);
+          break;
+        case GmshServer::CLIENT_OPTION_4:
+          if(num >= 0)
+            strcpy(SINFO[num].option[3][SINFO[num].nbval[3]++], message);
+          break;
+        case GmshServer::CLIENT_OPTION_5:
+          if(num >= 0)
+            strcpy(SINFO[num].option[4][SINFO[num].nbval[4]++], message);
+          break;
+        case GmshServer::CLIENT_MERGE_FILE:
+          if(num < 0 || (num >= 0 && SINFO[num].merge_views)) {
+            int n = PView::list.size();
+            MergeFile(message);
+            Draw();
+            if(n != (int)PView::list.size()) 
+              WID->set_context(menu_post, 0);
+          }
+          break;
+        case GmshServer::CLIENT_PARSE_STRING:
+          ParseString(message);
+          Draw();
+          break;
+        case GmshServer::CLIENT_INFO:
+          Msg::Direct("%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
+          break;
+        case GmshServer::CLIENT_WARNING:
+          Msg::Direct(2, "%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
+          break;
+        case GmshServer::CLIENT_ERROR:
+          Msg::Direct(1, "%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
+          break;
+        default:
+          Msg::Warning("Unknown type of message received from %s",
+              num >= 0 ? SINFO[num].name : "client");
+	  Msg::Direct("%-8.8s: %s", num >= 0 ? SINFO[num].name : "Client", message);
+          break;
+        }
+        WID->check();
       }
       else{
-	Msg(WARNING, "Failed to receive message body on socket: aborting");
-	break;
+        Msg::Warning("Failed to receive message body on socket: aborting");
+        break;
       }
       delete [] message;
     }
@@ -268,22 +272,22 @@ int Solver(int num, char *args)
   if(num >= 0){
     for(int i = 0; i < SINFO[num].nboptions; i++) {
       if(SINFO[num].nbval[i]) {
-	WID->solver[num].choice[i]->clear();
-	for(int j = 0; j < SINFO[num].nbval[i]; j++)
-	  WID->solver[num].choice[i]->add(SINFO[num].option[i][j]);
-	WID->solver[num].choice[i]->value(0);
+        WID->solver[num].choice[i]->clear();
+        for(int j = 0; j < SINFO[num].nbval[i]; j++)
+          WID->solver[num].choice[i]->add(SINFO[num].option[i][j]);
+        WID->solver[num].choice[i]->value(0);
       }
     }
   }
 
   if(server.StopClient() < 0)
-    Msg(WARNING, "Impossible to unlink the socket '%s'", sockname);
+    Msg::Warning("Impossible to unlink the socket '%s'", sockname);
 
   if(num >= 0){
-    Msg(STATUS2N, "");
+    Msg::StatusBar(2, false, "");
   }
   else{
-    Msg(INFO, "Client disconnected: starting new connection");
+    Msg::Info("Client disconnected: starting new connection");
     goto new_connection;
   }
 
