@@ -1,7 +1,7 @@
 #ifndef _SOLVERS_H_
 #define _SOLVERS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,13 +31,13 @@ typedef struct{
   char option_name[MAXSOLVERS][256], option_command[256];
   char option[MAXSOLVERS][100][256];
   int  nboptions, nbval[MAXSOLVERS];
-  char *help;
+  const char *help;
   int client_server, popup_messages, merge_views;
   int pid;
 } SolverInfo ;
 
 extern SolverInfo SINFO[MAXSOLVERS] ;
 
-int Solver(int num, char *args);
+int Solver(int num, const char *args);
 
 #endif
