@@ -1,6 +1,6 @@
-// $Id: ExtrudeParams.cpp,v 1.24 2007-03-05 09:30:53 geuzaine Exp $
+// $Id: ExtrudeParams.cpp,v 1.28 2008-05-04 08:31:13 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
+#include "Message.h"
 #include "Geo.h"
 #include "ExtrudeParams.h"
 
@@ -118,7 +118,7 @@ void ExtrudeParams::Extrude(double t, double &x, double &y, double &z)
     z += n[2] * t;
     break;
   default:
-    Msg(GERROR, "Unknown extrusion type");
+    Msg::Error("Unknown extrusion type");
     break;
   }
 }
