@@ -1,6 +1,6 @@
-// $Id: GModelIO_Geo.cpp,v 1.11 2007-03-18 12:05:16 geuzaine Exp $
+// $Id: GModelIO_Geo.cpp,v 1.25 2008-06-12 09:31:36 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,126 +19,164 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <stdlib.h>
 #include "GModel.h"
 #include "Geo.h"
 #include "OpenFile.h"
-#include "Tools.h"
 #include "Numeric.h"
+#include "ListUtils.h"
 #include "Message.h"
 #include "gmshVertex.h"
 #include "gmshFace.h"
 #include "gmshEdge.h"
 #include "gmshRegion.h"
 #include "Parser.h" // for Symbol_T
+#include "Field.h"
 
-extern Mesh *THEM;
+void GModel::_createGEOInternals()
+{
+  _geo_internals = new GEO_Internals;
+}
+
+void GModel::_deleteGEOInternals()
+{
+  delete _geo_internals;
+  _geo_internals = 0;
+}
 
 int GModel::readGEO(const std::string &name)
 {
-  ParseFile((char*)name.c_str(), 1);
-  return importTHEM();
+  ParseFile(name.c_str(), 1);
+  return importGEOInternals();
 }
 
-int GModel::importTHEM()
+int GModel::importGEOInternals()
 {
-  if(Tree_Nbr(THEM->Points)) {
-    List_T *points = Tree2List(THEM->Points);
+  if(Tree_Nbr(_geo_internals->Points)) {
+    List_T *points = Tree2List(_geo_internals->Points);
     for(int i = 0; i < List_Nbr(points); i++){
       Vertex *p;
       List_Read(points, i, &p);
-      GVertex *v = vertexByTag(p->Num);
+      GVertex *v = getVertexByTag(p->Num);
       if(!v){
-	v = new gmshVertex(this, p);
-	add(v);
+        v = new gmshVertex(this, p);
+        add(v);
       }
       if(!p->Visible) v->setVisibility(0);
     }
     List_Delete(points);
   }
-  if(Tree_Nbr(THEM->Curves)) {
-    List_T *curves = Tree2List(THEM->Curves);
+  if(Tree_Nbr(_geo_internals->Curves)) {
+    List_T *curves = Tree2List(_geo_internals->Curves);
     for(int i = 0; i < List_Nbr(curves); i++){
       Curve *c;
       List_Read(curves, i, &c);
       if(c->Num >= 0 && c->beg && c->end){
-	GEdge *e = edgeByTag(c->Num);
-	if(!e){
-	  e = new gmshEdge(this, c,
-			   vertexByTag(c->beg->Num),
-			   vertexByTag(c->end->Num));
-	  add(e);
-	}
-	else
-	  e->resetMeshAttributes();
-	if(!c->Visible) e->setVisibility(0);
-	if(c->Color.type) e->setColor(c->Color.mesh);
+        GEdge *e = getEdgeByTag(c->Num);
+        if(!e){
+          e = new gmshEdge(this, c,
+                           getVertexByTag(c->beg->Num),
+                           getVertexByTag(c->end->Num));
+          add(e);
+        }
+        else
+          e->resetMeshAttributes();
+        if(!c->Visible) e->setVisibility(0);
+        if(c->Color.type) e->setColor(c->Color.mesh);
       }
     }
     List_Delete(curves);
   }
-  if(Tree_Nbr(THEM->Surfaces)) {
-    List_T *surfaces = Tree2List(THEM->Surfaces);
+  if(Tree_Nbr(_geo_internals->Surfaces)) {
+    List_T *surfaces = Tree2List(_geo_internals->Surfaces);
     for(int i = 0; i < List_Nbr(surfaces); i++){
       Surface *s;
       List_Read(surfaces, i, &s);
-      GFace *f = faceByTag(s->Num);
+      GFace *f = getFaceByTag(s->Num);
       if(!f){
-	f = new gmshFace(this, s);
-	add(f);
+        f = new gmshFace(this, s);
+        add(f);
       }
       else
-	f->resetMeshAttributes();
+        f->resetMeshAttributes();
       if(!s->Visible) f->setVisibility(0);
       if(s->Color.type) f->setColor(s->Color.mesh);
     }
     List_Delete(surfaces);
   } 
-  if(Tree_Nbr(THEM->Volumes)) {
-    List_T *volumes = Tree2List(THEM->Volumes);
+  if(Tree_Nbr(_geo_internals->Volumes)) {
+    List_T *volumes = Tree2List(_geo_internals->Volumes);
     for(int i = 0; i < List_Nbr(volumes); i++){
       Volume *v;
       List_Read(volumes, i, &v);
-      GRegion *r = regionByTag(v->Num);
+      GRegion *r = getRegionByTag(v->Num);
       if(!r){
-	r = new gmshRegion(this, v);
-	add(r);
+        r = new gmshRegion(this, v);
+        add(r);
       }
       else
-	r->resetMeshAttributes();	
+        r->resetMeshAttributes();       
       if(!v->Visible) r->setVisibility(0);
       if(v->Color.type) r->setColor(v->Color.mesh);
     }
     List_Delete(volumes);
   }
-  for(int i = 0; i < List_Nbr(THEM->PhysicalGroups); i++){
+  for(int i = 0; i < List_Nbr(_geo_internals->PhysicalGroups); i++){
     PhysicalGroup *p;
-    List_Read(THEM->PhysicalGroups, i, &p);
+    List_Read(_geo_internals->PhysicalGroups, i, &p);
     for(int j = 0; j < List_Nbr(p->Entities); j++){
       int num;
       List_Read(p->Entities, j, &num);
       GEntity *ge = 0;
       switch(p->Typ){
-      case MSH_PHYSICAL_POINT:   ge = vertexByTag(abs(num)); break;
-      case MSH_PHYSICAL_LINE:    ge = edgeByTag(abs(num)); break;
-      case MSH_PHYSICAL_SURFACE: ge = faceByTag(abs(num)); break;
-      case MSH_PHYSICAL_VOLUME:  ge = regionByTag(abs(num)); break;
+      case MSH_PHYSICAL_POINT:   ge = getVertexByTag(abs(num)); break;
+      case MSH_PHYSICAL_LINE:    ge = getEdgeByTag(abs(num)); break;
+      case MSH_PHYSICAL_SURFACE: ge = getFaceByTag(abs(num)); break;
+      case MSH_PHYSICAL_VOLUME:  ge = getRegionByTag(abs(num)); break;
       }
       int pnum = sign(num) * p->Num;
       if(ge && std::find(ge->physicals.begin(), ge->physicals.end(), pnum) == 
-	 ge->physicals.end())
-	ge->physicals.push_back(pnum);
+         ge->physicals.end())
+        ge->physicals.push_back(pnum);
     }
   }
 
-  Msg(DEBUG, "Gmsh model imported:");
-  Msg(DEBUG, "%d Vertices", vertices.size());
-  Msg(DEBUG, "%d Edges", edges.size());
-  Msg(DEBUG, "%d Faces", faces.size());
-  Msg(DEBUG, "%d Regions", regions.size());
+  Msg::Debug("Gmsh model imported:");
+  Msg::Debug("%d Vertices", vertices.size());
+  Msg::Debug("%d Edges", edges.size());
+  Msg::Debug("%d Faces", faces.size());
+  Msg::Debug("%d Regions", regions.size());
   
   return 1;
 }
 
+class writeFieldOptionGEO{
+ private :
+  FILE *geo;
+  Field *field;
+ public :
+  writeFieldOptionGEO(FILE *fp,Field *_field) { geo = fp ? fp : stdout; field=_field;}
+  void operator() (std::pair<std::string, FieldOption *> it)
+  {
+    std::string v;
+    it.second->get_text_representation(v);
+    fprintf(geo, "Field[%i].%s = %s;\n", field->id, it.first.c_str(), v.c_str());
+  }
+};
+
+class writeFieldGEO{
+ private :
+  FILE *geo;
+ public :
+  writeFieldGEO(FILE *fp) { geo = fp ? fp : stdout; }
+  void operator() (std::pair<const int, Field *> it)
+  {
+    fprintf(geo, "Field[%i] = %s;\n", it.first, it.second->get_name());
+    std::for_each(it.second->options.begin(), it.second->options.end(),
+                  writeFieldOptionGEO(geo, it.second));
+  }
+};
+
 class writeGVertexGEO {
  private :
   FILE *geo;
@@ -150,12 +188,12 @@ class writeGVertexGEO {
       Vertex *v = (Vertex*)gv->getNativePtr();
       if(!v) return;
       fprintf(geo, "Point (%d) = {%.16g, %.16g, %.16g, %.16g};\n",
-	      v->Num, v->Pos.X, v->Pos.Y, v->Pos.Z, v->lc);
+              v->Num, v->Pos.X, v->Pos.Y, v->Pos.Z, v->lc);
     }
     else{
       fprintf(geo, "Point (%d) = {%.16g, %.16g, %.16g, %.16g};\n",
-	      gv->tag(), gv->x(), gv->y(), gv->z(), 
-	      gv->prescribedMeshSizeAtVertex());
+              gv->tag(), gv->x(), gv->y(), gv->z(), 
+              gv->prescribedMeshSizeAtVertex());
     }
   }
 };
@@ -174,89 +212,89 @@ class writeGEdgeGEO {
       if(!c || c->Num < 0) return;
       switch (c->Typ) {
       case MSH_SEGM_LINE:
-	fprintf(geo, "Line (%d) = ", c->Num);
-	break;
+        fprintf(geo, "Line (%d) = ", c->Num);
+        break;
       case MSH_SEGM_CIRC:
       case MSH_SEGM_CIRC_INV:
-	fprintf(geo, "Circle (%d) = ", c->Num);
-	break;
+        fprintf(geo, "Circle (%d) = ", c->Num);
+        break;
       case MSH_SEGM_ELLI:
       case MSH_SEGM_ELLI_INV:
-	fprintf(geo, "Ellipse (%d) = ", c->Num);
-	break;
+        fprintf(geo, "Ellipse (%d) = ", c->Num);
+        break;
       case MSH_SEGM_NURBS:
-	fprintf(geo, "Nurbs (%d) = {", c->Num);
-	for(int i = 0; i < List_Nbr(c->Control_Points); i++) {
-	  Vertex *v;
-	  List_Read(c->Control_Points, i, &v);
-	  if(!i)
-	    fprintf(geo, "%d", v->Num);
-	  else
-	    fprintf(geo, ", %d", v->Num);
-	  if(i % 8 == 7 && i != List_Nbr(c->Control_Points) - 1)
-	    fprintf(geo, "\n");
-	}
-	fprintf(geo, "}\n");
-	fprintf(geo, "  Knots {");
-	for(int j = 0; j < List_Nbr(c->Control_Points) + c->degre + 1; j++) {
-	  if(!j)
-	    fprintf(geo, "%.16g", c->k[j]);
-	  else
-	    fprintf(geo, ", %.16g", c->k[j]);
-	  if(j % 5 == 4 && j != List_Nbr(c->Control_Points) + c->degre)
-	    fprintf(geo, "\n        ");
-	}
-	fprintf(geo, "}\n");
-	fprintf(geo, "  Order %d;\n", c->degre);
-	return;
+        fprintf(geo, "Nurbs (%d) = {", c->Num);
+        for(int i = 0; i < List_Nbr(c->Control_Points); i++) {
+          Vertex *v;
+          List_Read(c->Control_Points, i, &v);
+          if(!i)
+            fprintf(geo, "%d", v->Num);
+          else
+            fprintf(geo, ", %d", v->Num);
+          if(i % 8 == 7 && i != List_Nbr(c->Control_Points) - 1)
+            fprintf(geo, "\n");
+        }
+        fprintf(geo, "}\n");
+        fprintf(geo, "  Knots {");
+        for(int j = 0; j < List_Nbr(c->Control_Points) + c->degre + 1; j++) {
+          if(!j)
+            fprintf(geo, "%.16g", c->k[j]);
+          else
+            fprintf(geo, ", %.16g", c->k[j]);
+          if(j % 5 == 4 && j != List_Nbr(c->Control_Points) + c->degre)
+            fprintf(geo, "\n        ");
+        }
+        fprintf(geo, "}\n");
+        fprintf(geo, "  Order %d;\n", c->degre);
+        return;
       case MSH_SEGM_SPLN:
-	fprintf(geo, "CatmullRom (%d) = ", c->Num);
-	break;
+        fprintf(geo, "CatmullRom (%d) = ", c->Num);
+        break;
       case MSH_SEGM_BSPLN:
-	fprintf(geo, "BSpline (%d) = ", c->Num);
-	break;
+        fprintf(geo, "BSpline (%d) = ", c->Num);
+        break;
       case MSH_SEGM_BEZIER:
-	fprintf(geo, "Bezier (%d) = ", c->Num);
-	break;
+        fprintf(geo, "Bezier (%d) = ", c->Num);
+        break;
       default:
-	Msg(GERROR, "Unknown curve type %d", c->Typ);
-	return;
+        Msg::Error("Unknown curve type %d", c->Typ);
+        return;
       }
       for(int i = 0; i < List_Nbr(c->Control_Points); i++) {
-	Vertex *v;
-	List_Read(c->Control_Points, i, &v);
-	if(i)
-	  fprintf(geo, ", %d", v->Num);
-	else
-	  fprintf(geo, "{%d", v->Num);
-	if(i % 6 == 7)
-	  fprintf(geo, "\n");
+        Vertex *v;
+        List_Read(c->Control_Points, i, &v);
+        if(i)
+          fprintf(geo, ", %d", v->Num);
+        else
+          fprintf(geo, "{%d", v->Num);
+        if(i % 6 == 7)
+          fprintf(geo, "\n");
       }
       fprintf(geo, "};\n");
     }
     else{
       if(ge->getBeginVertex() && ge->getEndVertex()){
-	if(ge->geomType() == GEntity::Line){
-	  fprintf(geo, "Line (%d) = {%d, %d};\n", 
-		  ge->tag(), ge->getBeginVertex()->tag(), ge->getEndVertex()->tag());
-	}
-	else{
-	  // approximate all other curves by splines
-	  Range<double> bounds = ge->parBounds(0);
-	  double umin = bounds.low();
-	  double umax = bounds.high();
-	  fprintf(geo, "p%d = newp;\n", ge->tag());
-	  for(int i = 1; i < ge->minimumDrawSegments(); i++){
-	    double u = umin + (double)i / ge->minimumDrawSegments() * (umax - umin);
-	    GPoint p = ge->point(u);
-	    fprintf(geo, "Point (p%d + %d) = {%.16g, %.16g, %.16g, 1.e+22};\n", 
-		    ge->tag(), i, p.x(), p.y(), p.z());
-	  }
-	  fprintf(geo, "CatmullRom (%d) = {%d", ge->tag(), ge->getBeginVertex()->tag());
-	  for(int i = 1; i < ge->minimumDrawSegments(); i++)
-	    fprintf(geo, ", p%d + %d", ge->tag(), i);
-	  fprintf(geo, ", %d};\n", ge->getEndVertex()->tag());
-	}
+        if(ge->geomType() == GEntity::Line){
+          fprintf(geo, "Line (%d) = {%d, %d};\n", 
+                  ge->tag(), ge->getBeginVertex()->tag(), ge->getEndVertex()->tag());
+        }
+        else{
+          // approximate all other curves by splines
+          Range<double> bounds = ge->parBounds(0);
+          double umin = bounds.low();
+          double umax = bounds.high();
+          fprintf(geo, "p%d = newp;\n", ge->tag());
+          for(int i = 1; i < ge->minimumDrawSegments(); i++){
+            double u = umin + (double)i / ge->minimumDrawSegments() * (umax - umin);
+            GPoint p = ge->point(u);
+            fprintf(geo, "Point (p%d + %d) = {%.16g, %.16g, %.16g, 1.e+22};\n", 
+                    ge->tag(), i, p.x(), p.y(), p.z());
+          }
+          fprintf(geo, "CatmullRom (%d) = {%d", ge->tag(), ge->getBeginVertex()->tag());
+          for(int i = 1; i < ge->minimumDrawSegments(); i++)
+            fprintf(geo, ", p%d + %d", ge->tag(), i);
+          fprintf(geo, ", %d};\n", ge->getEndVertex()->tag());
+        }
       }
     }
   }
@@ -276,25 +314,25 @@ class writeGFaceGEO {
     if(edges.size() && orientations.size() == edges.size()){
       std::vector<int> num, ori;
       for(std::list<GEdge*>::iterator it = edges.begin(); it != edges.end(); it++)
-	num.push_back((*it)->tag());
+        num.push_back((*it)->tag());
       for(std::list<int>::iterator it = orientations.begin(); it != orientations.end(); it++)
-	ori.push_back((*it) > 0 ? 1 : -1);
+        ori.push_back((*it) > 0 ? 1 : -1);
       fprintf(geo, "Line Loop (%d) = ", gf->tag());
       for(unsigned int i = 0; i < num.size(); i++){
-	if(i)
-	  fprintf(geo, ", %d", num[i] * ori[i]);
-	else
-	  fprintf(geo, "{%d", num[i] * ori[i]);
+        if(i)
+          fprintf(geo, ", %d", num[i] * ori[i]);
+        else
+          fprintf(geo, "{%d", num[i] * ori[i]);
       }
       fprintf(geo, "};\n");
       if(gf->geomType() == GEntity::Plane){
-	fprintf(geo, "Plane Surface (%d) = {%d};\n", gf->tag(), gf->tag());
+        fprintf(geo, "Plane Surface (%d) = {%d};\n", gf->tag(), gf->tag());
       }
       else if(edges.size() == 3 || edges.size() == 4){
-	fprintf(geo, "Ruled Surface (%d) = {%d};\n", gf->tag(), gf->tag());
+        fprintf(geo, "Ruled Surface (%d) = {%d};\n", gf->tag(), gf->tag());
       }
       else{
-	Msg(GERROR, "Skipping surface %d in export", gf->tag());
+        Msg::Error("Skipping surface %d in export", gf->tag());
       }
     }
   }
@@ -313,10 +351,10 @@ class writeGRegionGEO {
     if(faces.size()){
       fprintf(geo, "Surface Loop (%d) = ", gr->tag());
       for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); it++) {
-	if(it != faces.begin())
-	  fprintf(geo, ", %d", (*it)->tag());
-	else
-	  fprintf(geo, "{%d", (*it)->tag());
+        if(it != faces.begin())
+          fprintf(geo, ", %d", (*it)->tag());
+        else
+          fprintf(geo, "{%d", (*it)->tag());
       }
       fprintf(geo, "};\n");
       fprintf(geo, "Volume (%d) = {%d};\n", gr->tag(), gr->tag());
@@ -332,8 +370,8 @@ class writePhysicalGroupGEO {
   std::map<int, std::string> &oldLabels, &newLabels;
  public :
   writePhysicalGroupGEO(FILE *fp, int i, bool labels,
-			std::map<int, std::string> &o,
-			std::map<int, std::string> &n)
+                        std::map<int, std::string> &o,
+                        std::map<int, std::string> &n)
     : dim(i), printLabels(labels), oldLabels(o), newLabels(n)
   { 
     geo = fp ? fp : stdout; 
@@ -343,11 +381,11 @@ class writePhysicalGroupGEO {
     std::string oldName, newName;
     if(printLabels){
       if(oldLabels.count(g.first)) {
-	oldName = oldLabels[g.first];
-	fprintf(geo, "%s = %d;\n", oldName.c_str(), g.first);
+        oldName = oldLabels[g.first];
+        fprintf(geo, "%s = %d;\n", oldName.c_str(), g.first);
       }
       else if(newLabels.count(g.first)) {
-	newName = newLabels[g.first];
+        newName = newLabels[g.first];
       }
     }
 
@@ -398,7 +436,11 @@ int GModel::writeGEO(const std::string &name, bool printLabels)
   getPhysicalGroups(groups);
   for(int i = 0; i < 4; i++)
     std::for_each(groups[i].begin(), groups[i].end(), 
-		  writePhysicalGroupGEO(fp, i, printLabels, labels, physicalNames));
+                  writePhysicalGroupGEO(fp, i, printLabels, labels, physicalNames));
+  
+  std::for_each(getFields()->begin(), getFields()->end(), writeFieldGEO(fp));
+  if(getFields()->background_field > 0)
+    fprintf(fp, "Background Field = %i;\n", getFields()->background_field);
 
   if(fp) fclose(fp);
   return 1;
