@@ -1,7 +1,7 @@
 #ifndef _GEO_H_
 #define _GEO_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
 #include <math.h>
 #include "GmshDefines.h"
 #include "gmshSurface.h"
-#include "List.h"
-#include "Tree.h"
+#include "ListUtils.h"
+#include "TreeUtils.h"
 #include "SPoint2.h"
 #include "ExtrudeParams.h"
 
@@ -102,8 +102,8 @@ class Vertex {
   Vertex operator % (Vertex & autre) // cross product
   {
     return Vertex(Pos.Y * autre.Pos.Z - Pos.Z * autre.Pos.Y,
-		  -(Pos.X * autre.Pos.Z - Pos.Z * autre.Pos.X),
-		  Pos.X * autre.Pos.Y - Pos.Y * autre.Pos.X, lc, w);
+                  -(Pos.X * autre.Pos.Z - Pos.Z * autre.Pos.X),
+                  Pos.X * autre.Pos.Y - Pos.Y * autre.Pos.X, lc, w);
   }
 };
 
@@ -125,8 +125,9 @@ typedef struct{
   int Typ;
   char Visible;
   int Method;
-  int ipar[4];
-  double dpar[4];
+  int nbPointsTransfinite;
+  int typeTransfinite;
+  double coeffTransfinite;
   double l;
   double mat[4][4];
   Vertex *beg, *end;
@@ -154,12 +155,12 @@ typedef struct{
   int Recombine;
   int Recombine_Dir; // -1 is left, +1 is right, 0 is alternated
   double RecombineAngle;
-  int ipar[5];
+  int TransfiniteSmoothing;
   List_T *Generatrices;
   List_T *EmbeddedCurves;
   List_T *EmbeddedPoints;
-  List_T *Control_Points;
   List_T *TrsfPoints;
+  List_T *RuledSurfaceOptions;
   double plan[3][3];
   double a, b, c, d;
   ExtrudeParams *Extrude;
@@ -182,7 +183,6 @@ typedef struct {
   int Typ;
   char Visible;
   int Method;
-  int ipar[8];
   ExtrudeParams *Extrude;
   List_T *TrsfPoints;
   List_T *Surfaces;
@@ -198,7 +198,7 @@ typedef struct{
   List_T *Entities;
 }PhysicalGroup;
 
-class Mesh{
+class GEO_Internals{
  private:
   void alloc_all();
   void free_all();
@@ -212,8 +212,8 @@ class Mesh{
   List_T *PhysicalGroups;
   int MaxPointNum, MaxLineNum, MaxLineLoopNum, MaxSurfaceNum;
   int MaxSurfaceLoopNum, MaxVolumeNum, MaxPhysicalNum;
-  Mesh(){ alloc_all(); }
-  ~Mesh(){ free_all(); }
+  GEO_Internals(){ alloc_all(); }
+  ~GEO_Internals(){ free_all(); }
   void destroy(){ free_all(); alloc_all(); }
 };
 
@@ -234,7 +234,7 @@ int comparePhysicalGroup(const void *a, const void *b);
 Vertex *Create_Vertex(int Num, double X, double Y, double Z, double lc, double u);
 Vertex *Create_Vertex(int Num, double u, double v, gmshSurface *s, double lc);
 Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
-		    List_T * Knots, int p1, int p2, double u1, double u2);
+                    List_T * Knots, int p1, int p2, double u1, double u2);
 Curve *CreateReversedCurve(Curve *c);
 Surface *Create_Surface(int Num, int Typ);
 Volume *Create_Volume(int Num, int Typ);
@@ -259,6 +259,7 @@ int NEWLINELOOP(void);
 int NEWSURFACE(void);
 int NEWSURFACELOOP(void);
 int NEWVOLUME(void);
+int NEWFIELD(void);
 int NEWPHYSICAL(void);
 int NEWREG(void);
 
@@ -270,32 +271,29 @@ SurfaceLoop *FindSurfaceLoop(int inum);
 Volume *FindVolume(int inum);
 PhysicalGroup *FindPhysicalGroup(int inum, int type);
 
-void TranslateShapes(double X,double Y,double Z,
-                     List_T *ListShapes);
-void DilatShapes(double X,double Y,double Z, double A,
-                 List_T *ListShapes);
+void TranslateShapes(double X,double Y,double Z, List_T *shapes);
+void DilatShapes(double X,double Y,double Z, double A, List_T *shapes);
 void RotateShapes(double Ax,double Ay,double Az,
-		  double Px,double Py, double Pz,
-		  double alpha, List_T *ListShapes);
-void SymmetryShapes(double A,double B,double C,
-		    double D, List_T *ListShapes);
+                  double Px,double Py, double Pz, double alpha, List_T *shapes);
+void SymmetryShapes(double A,double B,double C, double D, List_T *shapes);
+void BoundaryShapes(List_T *shapes, List_T *shapesBoundary);
 void CopyShape(int Type, int Num, int *New);
 void DeleteShape(int Type, int Num);
 void ColorShape(int Type, int Num, unsigned int Color);
 void VisibilityShape(int Type, int Num, int Mode);
 void VisibilityShape(char *str, int Type, int Mode);
 void ExtrudeShape(int extrude_type, int shape_type, int shape_num,
-		  double T0, double T1, double T2,
-		  double A0, double A1, double A2,
-		  double X0, double X1, double X2, double alpha,
-		  ExtrudeParams *e,
-		  List_T *out);
+                  double T0, double T1, double T2,
+                  double A0, double A1, double A2,
+                  double X0, double X1, double X2, double alpha,
+                  ExtrudeParams *e,
+                  List_T *out);
 void ExtrudeShapes(int extrude_type, List_T *in,
-		   double T0, double T1, double T2,
-		   double A0, double A1, double A2,
-		   double X0, double X1, double X2, double alpha,
-		   ExtrudeParams *e,
-		   List_T *out);
+                   double T0, double T1, double T2,
+                   double A0, double A1, double A2,
+                   double X0, double X1, double X2, double alpha,
+                   ExtrudeParams *e,
+                   List_T *out);
 
 void ProtudeXYZ(double &x, double &y, double &z, ExtrudeParams *e);
 
@@ -304,6 +302,8 @@ void ReplaceAllDuplicates();
 bool ProjectPointOnCurve(Curve *c, Vertex *v, Vertex *RES, Vertex *DER);
 bool ProjectPointOnSurface(Surface *s, Vertex &p, double u[2]);
 
+bool IntersectCurvesWithSurface(List_T *curve_ids, int surface_id, List_T *shapes);
+
 int recognize_seg(int typ, List_T *liste, int *seg);
 int recognize_loop(List_T *liste, int *loop);
 int recognize_surfloop(List_T *liste, int *loop);
