@@ -1,7 +1,7 @@
 #ifndef _MVERTEX_H_
 #define _MVERTEX_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,18 +21,37 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include <stdio.h>
-#include <algorithm>
+#include <set>
 #include "SPoint3.h"
 
 class GEntity;
+class GFace;
+class MVertex;
+
+class MVertexLessThanLexicographic{
+ public:
+  static double tolerance;
+  bool operator()(const MVertex *v1, const MVertex *v2) const;
+};
 
 // A mesh vertex.
 class MVertex{
  private:
+  // the maximum vertex id number in the mesh
   static int _globalNum;
+  // the id number of the vertex (this number is unique and is
+  // guaranteed never to change once a mesh has been generated)
   int _num;
+  // a vertex index, used for example when saving a mesh (this index
+  // is not necessarily unique, can change after mesh renumbering,
+  // ...). By convention, vertices with negative indices are not
+  // saved
+  int _index;
+  // a visibility and polynomial order flags
   char _visible, _order;
+  // the cartesian coordinates of the vertex
   double _x, _y, _z;
+  // the geometrical entity the vertex is associated with
   GEntity *_ge;
 
  public :
@@ -46,10 +65,12 @@ class MVertex{
     else{
       _num = ++_globalNum;
     }
+    _index = num;
   }
   virtual ~MVertex(){}
 
-  // reset the global node number
+  // get/reset the global node number
+  static int getGlobalNumber(){ return _globalNum; }
   static void resetGlobalNumber(){ _globalNum = 0; }
 
   // get/set the visibility flag
@@ -77,8 +98,12 @@ class MVertex{
   inline int getNum() const { return _num; }
   inline void setNum(int num) { _num = num; }
 
+  // get/set the index
+  inline int getIndex() const { return _index; }
+  inline void setIndex(int index) { _index = index; }
+
   // get/set ith parameter
-  virtual bool getParameter(int i, double &par){ return false; }
+  virtual bool getParameter(int i, double &par) const{ return false; }
   virtual bool setParameter(int i, double par){ return false; }
 
   // measure distance to another vertex
@@ -90,30 +115,43 @@ class MVertex{
     return sqrt(dx * dx + dy * dy + dz * dz);
   }
 
-  // Get the data associated with this vertex
-  virtual void *getData(){ return 0; }
+  // linear coordinate search for the vertex in a set
+  std::set<MVertex*, MVertexLessThanLexicographic>::iterator 
+  linearSearch(std::set<MVertex*, MVertexLessThanLexicographic> &pos);
 
   // IO routines
   void writeMSH(FILE *fp, bool binary=false, double scalingFactor=1.0);
   void writeMSH(FILE *fp, double version, bool binary, int num, 
-		int elementary, int physical);
+                int elementary, int physical);
   void writeVRML(FILE *fp, double scalingFactor=1.0);
   void writeUNV(FILE *fp, double scalingFactor=1.0);
+  void writeVTK(FILE *fp, bool binary=false, double scalingFactor=1.0);
   void writeMESH(FILE *fp, double scalingFactor=1.0);
   void writeBDF(FILE *fp, int format=0, double scalingFactor=1.0);
 };
 
 class MEdgeVertex : public MVertex{
  protected:
-  double _u;
+  double _u, _lc;
  public :
-  MEdgeVertex(double x, double y, double z, GEntity *ge, double u) 
-    : MVertex(x, y, z, ge), _u(u)
+ MEdgeVertex(double x, double y, double z, GEntity *ge, double u, double lc = -1.0) 
+   : MVertex(x, y, z, ge), _u(u), _lc(lc)
   {
   }
   virtual ~MEdgeVertex(){}
-  virtual bool getParameter(int i, double &par){ par = _u; return true; }
-  virtual bool setParameter(int i, double par){ _u = par; return true; }
+  virtual bool getParameter(int i, double &par) const 
+  { 
+    if(i) throw; 
+    par = _u; 
+    return true; 
+  }
+  virtual bool setParameter(int i, double par)
+  { 
+    if(i) throw; 
+    _u = par; 
+    return true; 
+  }
+  double getLc() const { return _lc; }
 };
 
 class MFaceVertex : public MVertex{
@@ -125,36 +163,21 @@ class MFaceVertex : public MVertex{
   {
   }
   virtual ~MFaceVertex(){}
-  virtual bool getParameter(int i, double &par){ par = (i ? _v : _u); return true; }
-  virtual bool setParameter(int i, double par){ if(!i) _u = par; else _v = par; return true; }
-};
-
-template<class T>
-class MDataFaceVertex : public MFaceVertex{
- private:
-  T _data;
- public :
-  MDataFaceVertex(double x, double y, double z, GEntity *ge, double u, double v, T data) 
-    : MFaceVertex(x, y, z, ge, u, v), _data(data)
-  {
+  virtual bool getParameter(int i, double &par) const 
+  { 
+    par = (i ? _v : _u);
+    return true; 
   }
-  virtual ~MDataFaceVertex(){}
-  virtual bool getData(T &data){ data = _data; return true; }
-  virtual void *getData(){ return (void*)&_data; }
-};
-
-class MVertexLessThanLexicographic{
- public:
-  static double tolerance;
-  bool operator()(const MVertex *v1, const MVertex *v2) const
+  virtual bool setParameter(int i, double par)
   {
-    if(v1->x() - v2->x() >  tolerance) return true;
-    if(v1->x() - v2->x() < -tolerance) return false;
-    if(v1->y() - v2->y() >  tolerance) return true;
-    if(v1->y() - v2->y() < -tolerance) return false;
-    if(v1->z() - v2->z() >  tolerance) return true;
-    return false;
+    if(!i) 
+      _u = par; 
+    else 
+      _v = par; 
+    return true; 
   }
 };
 
+void parametricCoordinates(const MVertex *ver, const GFace *gf, double &u, double &v);
+
 #endif
