@@ -1,7 +1,7 @@
 #ifndef _FIND_LINKS_H_
 #define _FIND_LINKS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "List.h"
+#include "ListUtils.h"
 
 int allEdgesLinked(int ed, List_T *edges);
 int allFacesLinked(int fac, List_T *faces);
