@@ -1,6 +1,6 @@
-// $Id: gmshFace.cpp,v 1.39 2007-02-26 08:25:38 geuzaine Exp $
+// $Id: gmshFace.cpp,v 1.63 2008-06-20 12:15:44 remacle Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,31 +19,38 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <stdlib.h>
 #include "GModel.h"
-#include "gmshVertex.h"
-#include "gmshEdge.h"
 #include "gmshFace.h"
 #include "Geo.h"
 #include "GeoInterpolation.h"
 #include "Numeric.h"
 #include "Message.h"
+#include "Context.h"
 
-extern Mesh *THEM;
+extern Context_T CTX;
 
 gmshFace::gmshFace(GModel *m, Surface *face)
   : GFace(m, face->Num), s(face)
 {
-  for(int i = 0 ; i < List_Nbr(s->Generatrices); i++){
+  resetMeshAttributes();
+
+  for(int i = 0; i < List_Nbr(s->Generatrices); i++){
     Curve *c;
     List_Read(s->Generatrices, i, &c);
-    GEdge *e = m->edgeByTag(abs(c->Num));
+    GEdge *e = m->getEdgeByTag(abs(c->Num));
+
     if(e){
       l_edges.push_back(e);
       e->addFace(this);
       l_dirs.push_back((c->Num > 0) ? 1 : -1);
+      if (List_Nbr(s->Generatrices) == 2){
+        e->meshAttributes.minimumMeshSegments = 
+          std::max(e->meshAttributes.minimumMeshSegments, 2);
+      }
     }
     else
-      Msg(GERROR, "Unknown curve %d", c->Num);
+      Msg::Error("Unknown curve %d", c->Num);
   }
 
   // always compute and store the mean plane for plane surfaces
@@ -52,7 +59,7 @@ gmshFace::gmshFace(GModel *m, Surface *face)
     computeMeanPlane();
     for(int i = 0; i < 3; i++)
       for(int j = 0; j < 3; j++)
-	s->plan[i][j] = meanPlane.plan[i][j];
+        s->plan[i][j] = meanPlane.plan[i][j];
     s->a = meanPlane.a;
     s->b = meanPlane.b;
     s->c = meanPlane.c;
@@ -60,37 +67,43 @@ gmshFace::gmshFace(GModel *m, Surface *face)
   }
   
   if(s->EmbeddedCurves){
-    for(int i = 0 ; i < List_Nbr(s->EmbeddedCurves); i++){
+    for(int i = 0; i < List_Nbr(s->EmbeddedCurves); i++){
       Curve *c;
       List_Read(s->EmbeddedCurves, i, &c);
-      GEdge *e = m->edgeByTag(abs(c->Num));
+      GEdge *e = m->getEdgeByTag(abs(c->Num));
       if(e)
-	embedded_edges.push_back(e);
+        embedded_edges.push_back(e);
       else
-	Msg(GERROR, "Unknown curve %d", c->Num);
+        Msg::Error("Unknown curve %d", c->Num);
     }
   }
   if(s->EmbeddedPoints){
-    for(int i = 0 ; i < List_Nbr(s->EmbeddedPoints); i++){
+    for(int i = 0; i < List_Nbr(s->EmbeddedPoints); i++){
       Vertex *v;
       List_Read(s->EmbeddedPoints, i, &v);
-      GVertex *gv = m->vertexByTag(v->Num);
+      GVertex *gv = m->getVertexByTag(v->Num);
       if(gv)
-	embedded_vertices.push_back(gv);
+        embedded_vertices.push_back(gv);
       else
-	Msg(GERROR, "Unknown point %d", v->Num);
+        Msg::Error("Unknown point %d", v->Num);
     }
   }
-  //  face->print_info();
-
-  resetMeshAttributes();
+  isSphere = iSRuledSurfaceASphere(s, center, radius);
 }
 
-gmshFace::gmshFace(GModel *m, int num)
-  : GFace(m, num)
+double gmshFace::getMetricEigenvalue(const SPoint2 &pt)
 {
-  s = Create_Surface(num, MSH_SURF_DISCRETE);
-  Tree_Add(THEM->Surfaces, &s);
+  if(!s->geometry) return 1;
+  return s->geometry->getMetricEigenvalue(pt);
+}
+
+void gmshFace::setModelEdges(std::list<GEdge*> &ed)
+{
+  for (std::list<GEdge*>::iterator it = ed.begin(); it != ed.end() ; ++it){
+    l_edges.push_back(*it);
+    (*it)->addFace(this);
+    l_dirs.push_back(1);
+  }
 }
 
 void gmshFace::resetMeshAttributes()
@@ -101,15 +114,16 @@ void gmshFace::resetMeshAttributes()
   meshAttributes.extrude = s->Extrude;
   if(meshAttributes.Method == TRANSFINI){
     meshAttributes.transfiniteArrangement = s->Recombine_Dir;
+    meshAttributes.transfiniteSmoothing = s->TransfiniteSmoothing;
     meshAttributes.corners.clear();
     for(int i = 0; i < List_Nbr(s->TrsfPoints); i++){
       Vertex *corn;
       List_Read(s->TrsfPoints, i, &corn);
-      GVertex *gv = model()->vertexByTag(corn->Num);
+      GVertex *gv = model()->getVertexByTag(corn->Num);
       if(gv)
-	meshAttributes.corners.push_back(gv);
+        meshAttributes.corners.push_back(gv);
       else
-	Msg(GERROR, "Unknown vertex %d in transfinite attributes", corn->Num);
+        Msg::Error("Unknown vertex %d in transfinite attributes", corn->Num);
     }
   }
 }
@@ -146,13 +160,13 @@ SVector3 gmshFace::normal(const SPoint2 &param) const
       List_Read(s->Generatrices, i, &c);
       int N = (c->Typ == MSH_SEGM_LINE) ? 1 : 3;
       for(int j = 0; j < N; j++) {
-	double u1 = (double)j / (double)N;
-	double u2 = (double)(j + 1) / (double)N;
-	Vertex p1 = InterpolateCurve(c, u1, 0);
-	Vertex p2 = InterpolateCurve(c, u2, 0);
-	double v1[3] = {p1.Pos.X, p1.Pos.Y, p1.Pos.Z};
-	double v2[3] = {p2.Pos.X, p2.Pos.Y, p2.Pos.Z};
-	angle += angle_plan(v, v1, v2, n);
+        double u1 = (double)j / (double)N;
+        double u2 = (double)(j + 1) / (double)N;
+        Vertex p1 = InterpolateCurve(c, u1, 0);
+        Vertex p2 = InterpolateCurve(c, u2, 0);
+        double v1[3] = {p1.Pos.X, p1.Pos.Y, p1.Pos.Z};
+        double v2[3] = {p2.Pos.X, p2.Pos.Y, p2.Pos.Z};
+        angle += angle_plan(v, v1, v2, n);
       }
     }
     if(angle > 0)
@@ -166,8 +180,8 @@ Pair<SVector3,SVector3> gmshFace::firstDer(const SPoint2 &param) const
 {
   Vertex vu = InterpolateSurface(s, param[0], param[1], 1, 1);
   Vertex vv = InterpolateSurface(s, param[0], param[1], 1, 2);
-  return Pair<SVector3,SVector3>(SVector3(vu.Pos.X, vu.Pos.Y, vu.Pos.Z),
-				 SVector3(vv.Pos.X, vv.Pos.Y, vv.Pos.Z));
+  return Pair<SVector3, SVector3>(SVector3(vu.Pos.X, vu.Pos.Y, vu.Pos.Z),
+                                  SVector3(vv.Pos.X, vv.Pos.Y, vv.Pos.Z));
 }
 
 GPoint gmshFace::point(double par1, double par2) const
@@ -177,8 +191,8 @@ GPoint gmshFace::point(double par1, double par2) const
     double x, y, z, VX[3], VY[3];
     getMeanPlaneData(VX, VY, x, y, z);
     return GPoint(x + VX[0] * par1 + VY[0] * par2,
-		  y + VX[1] * par1 + VY[1] * par2,
-		  z + VX[2] * par1 + VY[2] * par2, this, pp);
+                  y + VX[1] * par1 + VY[1] * par2,
+                  z + VX[2] * par1 + VY[2] * par2, this, pp);
   }
   else{
     Vertex v = InterpolateSurface(s, par1, par2, 0, 0);
@@ -186,28 +200,19 @@ GPoint gmshFace::point(double par1, double par2) const
   }
 }
 
-GPoint gmshFace::closestPoint(const SPoint3 & qp) const
+GPoint gmshFace::closestPoint(const SPoint3 & qp, const double initialGuess[2]) const
 {
   Vertex v;
-  double u[2];
+  double u[2] = {initialGuess[0],initialGuess[1]};
   v.Pos.X = qp.x();
   v.Pos.Y = qp.y();
   v.Pos.Z = qp.z();
-  ProjectPointOnSurface(s, v, u);
+  bool result = ProjectPointOnSurface(s, v, u);
+  if (!result)
+    printf("Project Point on surface %d \n",result);
   return GPoint(v.Pos.X, v.Pos.Y, v.Pos.Z, this, u);
 }
 
-int gmshFace::containsParam(const SPoint2 &pt) const
-{
-  Range<double> uu = parBounds(0);
-  Range<double> vv = parBounds(1);
-  if((pt.x() >= uu.low() && pt.x() <= uu.high()) && 
-     (pt.y() >= vv.low() && pt.y() <= vv.high()))
-    return 1;
-  else 
-    return 0;
-}
-
 SPoint2 gmshFace::parFromPoint(const SPoint3 &qp) const
 {
   if(s->Typ == MSH_SURF_PLAN){
@@ -226,12 +231,40 @@ SPoint2 gmshFace::parFromPoint(const SPoint3 &qp) const
 GEntity::GeomType gmshFace::geomType() const
 {
   switch(s->Typ){
-  case MSH_SURF_PLAN: return Plane;
+  case MSH_SURF_PLAN: 
+    if(s->geometry) 
+      return ParametricSurface;
+    else
+      return Plane;
   case MSH_SURF_REGL:
-  case MSH_SURF_TRIC: return RuledSurface;
-  case MSH_SURF_DISCRETE: return DiscreteSurface;
-  case MSH_SURF_BND_LAYER: return BoundaryLayerSurface;
-  default: return Unknown;
+  case MSH_SURF_TRIC: 
+    return RuledSurface;
+  case MSH_SURF_DISCRETE: 
+    return DiscreteSurface;
+  case MSH_SURF_BND_LAYER:
+    return BoundaryLayerSurface;
+  default: 
+    return Unknown;
+  }
+}
+
+// by default we assume that straight lines are geodesics
+SPoint2 gmshFace::geodesic(const SPoint2 &pt1 , const SPoint2 &pt2 , double t)
+{
+  if(isSphere && CTX.mesh.second_order_experimental){
+    // FIXME: this is buggy -- remove the CTX option once we do it in
+    // a robust manner
+    GPoint gp1 = point(pt1.x(), pt1.y());
+    GPoint gp2 = point(pt2.x(), pt2.y());
+    SPoint2 guess = pt1 + (pt2 - pt1) * t;
+    GPoint gp = closestPoint(SPoint3(gp1.x() + t * (gp2.x() - gp1.x()),
+				     gp1.y() + t * (gp2.y() - gp1.y()),
+				     gp1.z() + t * (gp2.z() - gp1.z())),
+			     (double*)guess);
+    return SPoint2(gp.u(), gp.v());
+  }
+  else{
+    return pt1 + (pt2 - pt1) * t;
   }
 }
 
@@ -249,13 +282,13 @@ int gmshFace::containsPoint(const SPoint3 &pt) const
       List_Read(s->Generatrices, i, &c);
       int N = (c->Typ == MSH_SEGM_LINE) ? 1 : 10;
       for(int j = 0; j < N; j++) {
-	double u1 = (double)j / (double)N;
-	double u2 = (double)(j + 1) / (double)N;
-	Vertex p1 = InterpolateCurve(c, u1, 0);
-	Vertex p2 = InterpolateCurve(c, u2, 0);
-	double v1[3] = {p1.Pos.X, p1.Pos.Y, p1.Pos.Z};
-	double v2[3] = {p2.Pos.X, p2.Pos.Y, p2.Pos.Z};
-	angle += angle_plan(v, v1, v2, n);
+        double u1 = (double)j / (double)N;
+        double u2 = (double)(j + 1) / (double)N;
+        Vertex p1 = InterpolateCurve(c, u1, 0);
+        Vertex p2 = InterpolateCurve(c, u2, 0);
+        double v1[3] = {p1.Pos.X, p1.Pos.Y, p1.Pos.Z};
+        double v2[3] = {p2.Pos.X, p2.Pos.Y, p2.Pos.Z};
+        angle += angle_plan(v, v1, v2, n);
       }
     }
     // we're inside if angle equals 2 * pi
