@@ -1,6 +1,6 @@
-// $Id: Draw.cpp,v 1.109 2006-11-27 22:22:14 geuzaine Exp $
+// $Id: Draw.cpp,v 1.119 2008-05-04 08:31:14 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,38 +19,35 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
 #include "GmshUI.h"
 #include "GmshDefines.h"
+#include "Message.h"
 #include "Draw.h"
 #include "Context.h"
 #include "Numeric.h"
 #include "GModel.h"
+#include "PView.h"
+#include "PViewOptions.h"
 
 extern Context_T CTX;
-extern GModel *GMODEL;
 
 int NeedPolygonOffset()
 {
-  if(GMODEL->getMeshStatus() == 2 &&
+  GModel *m = GModel::current();
+  if(m->getMeshStatus() == 2 &&
      (CTX.mesh.surfaces_edges || CTX.geom.lines || CTX.geom.surfaces))
     return 1;
-  if(GMODEL->getMeshStatus() == 3 && 
+  if(m->getMeshStatus() == 3 && 
      (CTX.mesh.surfaces_edges || CTX.mesh.volumes_edges))
     return 1;
-  for(int i = 0; i < List_Nbr(CTX.post.list); i++){
-    Post_View *v = *(Post_View**)List_Pointer(CTX.post.list, i);
-    if(v->Visible){
-      if(v->ShowElement)
-	return 1;
-      if((v->NbST || v->NbSQ) && (v->Axes || v->IntervalsType == DRAW_POST_ISO))
-	return 1;
-    }
+  for(unsigned int i = 0; i < PView::list.size(); i++){
+    PViewOptions *opt = PView::list[i]->getOptions();
+    if(opt->Visible && opt->ShowElement) return 1;
   }
   return 0;
 }
 
-void Draw3d(void)
+void Draw3d()
 {
   // We should only enable the polygon offset when there is a mix of
   // lines and polygons to be drawn; enabling it all the time can lead
@@ -79,7 +76,7 @@ void Draw3d(void)
   Draw_Post();
 }
 
-void Draw2d(void)
+void Draw2d()
 {
   glDisable(GL_DEPTH_TEST);
   for(int i = 0; i < 6; i++)
@@ -93,16 +90,17 @@ void Draw2d(void)
   glTranslated(0., 0., CTX.clip_factor > 1. ? 1./CTX.clip_factor : CTX.clip_factor);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
+
   Draw_Graph2D();
   Draw_Text2D();
   Draw_OnScreenMessages();
-  if(CTX.post.draw)
-    Draw_Scales();
+  if(CTX.post.draw) Draw_Scales();
+
   if(CTX.small_axes)
     Draw_SmallAxes();
 }
 
-void DrawPlugin(void (*draw)(void))
+void DrawPlugin(void (*draw)())
 {
   CTX.post.plugin_draw_function = draw;
   int old = CTX.draw_bbox;
@@ -120,7 +118,7 @@ void DrawPlugin(void (*draw)(void))
   CTX.mesh.draw = 1;
 }
 
-void ClearOpengl(void)
+void ClearOpengl()
 {
   glClearColor(CTX.UNPACK_RED(CTX.color.bg) / 255.,
                CTX.UNPACK_GREEN(CTX.color.bg) / 255.,
@@ -243,8 +241,8 @@ void InitProjection(int xpick, int ypick, int wpick, int hpick)
       glVertex3d(cx + r, cy, 0.);
       int ntheta = 36;
       for(int i = 1; i < ntheta + 1; i ++){
-	double theta = i * 2 * M_PI / (double)ntheta;
-	glVertex3d(cx + r * cos(theta), cy + r * sin(theta), 0.);	
+        double theta = i * 2 * M_PI / (double)ntheta;
+        glVertex3d(cx + r * cos(theta), cy + r * sin(theta), 0.);       
       }
       glEnd();
     }
@@ -252,7 +250,7 @@ void InitProjection(int xpick, int ypick, int wpick, int hpick)
   }
 }
 
-void InitRenderModel(void)
+void InitRenderModel()
 {
   GLfloat r, g, b;
 
@@ -264,9 +262,9 @@ void InitRenderModel(void)
   for(int i = 0; i < 6; i++) {
     if(CTX.light[i]) {
       GLfloat position[4] = {(GLfloat)CTX.light_position[i][0],
-			     (GLfloat)CTX.light_position[i][1],
-			     (GLfloat)CTX.light_position[i][2],
-			     (GLfloat)CTX.light_position[i][3]};
+                             (GLfloat)CTX.light_position[i][1],
+                             (GLfloat)CTX.light_position[i][2],
+                             (GLfloat)CTX.light_position[i][3]};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_POSITION, position);
 
       r = CTX.UNPACK_RED(CTX.color.ambient_light[i])/255.;
@@ -320,7 +318,7 @@ void InitRenderModel(void)
   glDisable(GL_LIGHTING);
 }
 
-void InitPosition(void)
+void InitPosition()
 {
   glScaled(CTX.s[0], CTX.s[1], CTX.s[2]);
   glTranslated(CTX.t[0], CTX.t[1], CTX.t[2]);
@@ -329,8 +327,8 @@ void InitPosition(void)
     glTranslated(CTX.cg[0], CTX.cg[1], CTX.cg[2]);
   else
     glTranslated(CTX.rotation_center[0],
-		 CTX.rotation_center[1],
-		 CTX.rotation_center[2]);
+                 CTX.rotation_center[1],
+                 CTX.rotation_center[2]);
   
   CTX.buildRotationMatrix();
   glMultMatrixd(CTX.rot);
@@ -339,8 +337,8 @@ void InitPosition(void)
     glTranslated(-CTX.cg[0], -CTX.cg[1], -CTX.cg[2]);
   else
     glTranslated(-CTX.rotation_center[0],
-		 -CTX.rotation_center[1],
-		 -CTX.rotation_center[2]);
+                 -CTX.rotation_center[1],
+                 -CTX.rotation_center[2]);
 
   // store the projection and modelview matrices at this precise
   // moment (so that we can use them at any later time, even if the
@@ -369,9 +367,9 @@ void Unproject(double x, double y, double p[3], double d[3])
   // the matrices since unproject can be called in or after Draw2D
   
   if(!gluUnProject(x, y, 0.0, CTX.model, CTX.proj, viewport, &x0, &y0, &z0))
-    Msg(WARNING, "unproject1 failed");
+    Msg::Warning("unproject1 failed");
   if(!gluUnProject(x, y, 1.0, CTX.model, CTX.proj, viewport, &x1, &y1, &z1))
-    Msg(WARNING, "unproject2 failed");
+    Msg::Warning("unproject2 failed");
   
   p[0] = x0;
   p[1] = y0;
