@@ -1,6 +1,6 @@
-// $Id: Entity.cpp,v 1.72 2007-06-22 12:08:16 remacle Exp $
+// $Id: Entity.cpp,v 1.84 2008-04-17 18:21:11 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
+#include <string.h>
 #include "GmshUI.h"
 #include "Numeric.h"
 #include "Draw.h"
@@ -43,47 +43,63 @@ void Draw_Point(int type, double size, double *x, double *y, double *z,
 void Draw_Sphere(double size, double x, double y, double z, int light)
 {
   if(light) glEnable(GL_LIGHTING);
+
+  static int first = 1;
   static GLUquadricObj *qua;
-  static int first = 1, listnum;
-  double s = size * CTX.pixel_equiv_x / CTX.s[0]; // size is in pixels
-  if(first) {
+
+  if(first){
     first = 0;
     qua = gluNewQuadric();
-    listnum = glGenLists(1);
-    glNewList(listnum, GL_COMPILE);
-    gluSphere(qua, 1, CTX.quadric_subdivisions, CTX.quadric_subdivisions);
-    glEndList();
   }
 
   glPushMatrix();
   glTranslated(x, y, z);
+  double s = size * CTX.pixel_equiv_x / CTX.s[0]; // size is in pixels
   glScaled(s, s, s);
-  glCallList(listnum);
+  gluSphere(qua, 1, CTX.quadric_subdivisions, CTX.quadric_subdivisions);
   glPopMatrix();
   glDisable(GL_LIGHTING);
 }
 
 
-void Draw_Disk(double size, double rint, double x, double y, double z, int light)
+void Draw_TapCylinder(double width, double val1, double val2, 
+                      double ValMin, double ValMax, 
+                      double *x, double *y, double *z, int light)
 {
   if(light) glEnable(GL_LIGHTING);
+
+  static int first = 1;
   static GLUquadricObj *qua;
-  static int first = 1, listnum;
-  double s = size;
-  if(first) {
+
+  if(first){
     first = 0;
     qua = gluNewQuadric();
-    listnum = glGenLists(1);
-    glNewList(listnum, GL_COMPILE);
-    gluDisk(qua, rint, 1, 2*CTX.quadric_subdivisions, 2*CTX.quadric_subdivisions);
-    glEndList();
   }
 
+  double dx = x[1] - x[0];
+  double dy = y[1] - y[0];
+  double dz = z[1] - z[0];
+  double length = sqrt(dx*dx + dy*dy + dz*dz);
+  double radius1 = width * (val1-ValMin)/(ValMax-ValMin) * CTX.pixel_equiv_x /CTX.s[0];
+  double radius2 = width * (val2-ValMin)/(ValMax-ValMin) * CTX.pixel_equiv_x /CTX.s[0];
+  double zdir[3] = {0., 0., 1.};
+  double vdir[3] = {dx/length, dy/length, dz/length};
+  double axis[3], cosphi, phi;
+  prodve(zdir, vdir, axis);
+  prosca(zdir, vdir, &cosphi);
+  if(!norme(axis)){
+    axis[0] = 0.;
+    axis[1] = 1.;
+    axis[2] = 0.;
+  }
+  phi = 180. * myacos(cosphi) / M_PI;
+
   glPushMatrix();
-  glTranslated(x, y, z);
-  glScaled(s, s, s);
-  glCallList(listnum);
+  glTranslated(x[0], y[0], z[0]);
+  glRotated(phi, axis[0], axis[1], axis[2]);
+  gluCylinder(qua, radius1, radius2, length, CTX.quadric_subdivisions, 1);
   glPopMatrix();
+
   glDisable(GL_LIGHTING);
 }
 
@@ -138,30 +154,11 @@ void Draw_Line(int type, double width, double *x, double *y, double *z,
   }
 }
 
-void Draw_Triangle_Overlay(double r, double g, double b,
-			   double *v1, double *v2, double *v3)
-{
-  SetOpenglContext();
-  InitProjection();
-  InitPosition();
-  glDisable(GL_DEPTH_TEST);
-  glDrawBuffer(GL_FRONT);
-  glColor3f(r, g, b);
-  glBegin(GL_LINE_LOOP);
-  glVertex3dv(v1);
-  glVertex3dv(v2);
-  glVertex3dv(v3);
-  glEnd();
-  glFlush();
-  glDrawBuffer(GL_BACK);
-  glEnable(GL_DEPTH_TEST);
-}
-
 void Draw_SimpleVector(int arrow, int fill,
-		       double relHeadRadius, double relStemLength, double relStemRadius,
-		       double x, double y, double z,
-		       double dx, double dy, double dz, 
-		       double d, int light)
+                       double relHeadRadius, double relStemLength, double relStemRadius,
+                       double x, double y, double z,
+                       double dx, double dy, double dz, 
+                       double d, int light)
 {
   double n[3], t[3], u[3];
 
@@ -212,23 +209,23 @@ void Draw_SimpleVector(int arrow, int fill,
       if(light) glNormal3dv(u);
       glVertex3d(x + dx, y + dy, z + dz);
       glVertex3d(x + f2 * dx + b * (t[0]), y + f2 * dy + b * (t[1]),
-		 z + f2 * dz + b * (t[2]));
+                 z + f2 * dz + b * (t[2]));
       glVertex3d(x + f1 * dx, y + f1 * dy, z + f1 * dz);
       
       glVertex3d(x + f1 * dx, y + f1 * dy, z + f1 * dz);
       glVertex3d(x + f2 * dx + b * (-t[0]), y + f2 * dy + b * (-t[1]),
-		 z + f2 * dz + b * (-t[2]));
+                 z + f2 * dz + b * (-t[2]));
       glVertex3d(x + dx, y + dy, z + dz);
       
       if(light) glNormal3dv(t);
       glVertex3d(x + dx, y + dy, z + dz);
       glVertex3d(x + f2 * dx + b * (-u[0]), y + f2 * dy + b * (-u[1]),
-		 z + f2 * dz + b * (-u[2]));
+                 z + f2 * dz + b * (-u[2]));
       glVertex3d(x + f1 * dx, y + f1 * dy, z + f1 * dz);
 
       glVertex3d(x + f1 * dx, y + f1 * dy, z + f1 * dz);      
       glVertex3d(x + f2 * dx + b * (u[0]), y + f2 * dy + b * (u[1]),
-		 z + f2 * dz + b * (u[2]));
+                 z + f2 * dz + b * (u[2]));
       glVertex3d(x + dx, y + dy, z + dz);
       glEnd();
       glDisable(GL_LIGHTING);
@@ -238,16 +235,16 @@ void Draw_SimpleVector(int arrow, int fill,
       glVertex3d(x, y, z);
       glVertex3d(x + dx, y + dy, z + dz);
       glVertex3d(x + f2 * dx + b * (t[0]), y + f2 * dy + b * (t[1]),
-		 z + f2 * dz + b * (t[2]));
+                 z + f2 * dz + b * (t[2]));
       glVertex3d(x + f1 * dx, y + f1 * dy, z + f1 * dz);
       glVertex3d(x + f2 * dx + b * (-t[0]), y + f2 * dy + b * (-t[1]),
-		 z + f2 * dz + b * (-t[2]));
+                 z + f2 * dz + b * (-t[2]));
       glVertex3d(x + dx, y + dy, z + dz);
       glVertex3d(x + f2 * dx + b * (-u[0]), y + f2 * dy + b * (-u[1]),
-		 z + f2 * dz + b * (-u[2]));
+                 z + f2 * dz + b * (-u[2]));
       glVertex3d(x + f1 * dx, y + f1 * dy, z + f1 * dz);
       glVertex3d(x + f2 * dx + b * (u[0]), y + f2 * dy + b * (u[1]),
-		 z + f2 * dz + b * (u[2]));
+                 z + f2 * dz + b * (u[2]));
       glVertex3d(x + dx, y + dy, z + dz);
       glEnd();
     }
@@ -264,36 +261,36 @@ void Draw_SimpleVector(int arrow, int fill,
       if(light && fill) glEnable(GL_LIGHTING);
       glBegin(GL_TRIANGLES);
       if(light){
-	normal3points(tm[0], tm[1], tm[2], um[0], um[1], um[2],
-		      top[0], top[1], top[2], nn);
-	glNormal3dv(nn);
+        normal3points(tm[0], tm[1], tm[2], um[0], um[1], um[2],
+                      top[0], top[1], top[2], nn);
+        glNormal3dv(nn);
       }
       glVertex3d(tm[0], tm[1], tm[2]);
       glVertex3d(um[0], um[1], um[2]);
       glVertex3d(top[0], top[1], top[2]);
 
       if(light){
-	normal3points(um[0], um[1], um[2], tp[0], tp[1], tp[2],
-		      top[0], top[1], top[2], nn);
-	glNormal3dv(nn);
+        normal3points(um[0], um[1], um[2], tp[0], tp[1], tp[2],
+                      top[0], top[1], top[2], nn);
+        glNormal3dv(nn);
       }
       glVertex3d(um[0], um[1], um[2]);
       glVertex3d(tp[0], tp[1], tp[2]);
       glVertex3d(top[0], top[1], top[2]);
 
       if(light){
-	normal3points(tp[0], tp[1], tp[2], up[0], up[1], up[2],
-		      top[0], top[1], top[2], nn);
-	glNormal3dv(nn);
+        normal3points(tp[0], tp[1], tp[2], up[0], up[1], up[2],
+                      top[0], top[1], top[2], nn);
+        glNormal3dv(nn);
       }
       glVertex3d(tp[0], tp[1], tp[2]);
       glVertex3d(up[0], up[1], up[2]);
       glVertex3d(top[0], top[1], top[2]);
 
       if(light){
-	normal3points(up[0], up[1], up[2], tm[0], tm[1], tm[2],
-		      top[0], top[1], top[2], nn);
-	glNormal3dv(nn);
+        normal3points(up[0], up[1], up[2], tm[0], tm[1], tm[2],
+                      top[0], top[1], top[2], nn);
+        glNormal3dv(nn);
       }
       glVertex3d(up[0], up[1], up[2]);
       glVertex3d(tm[0], tm[1], tm[2]);
@@ -327,10 +324,9 @@ void Draw_SimpleVector(int arrow, int fill,
 
 }
 
-
 void Draw_3DArrow(double relHeadRadius, double relStemLength, double relStemRadius,
-		  double x, double y, double z, double dx, double dy, double dz,
-		  double length, int light)
+                  double x, double y, double z, double dx, double dy, double dz,
+                  double length, int light)
 {
   if(light) glEnable(GL_LIGHTING);
 
@@ -381,37 +377,35 @@ void Draw_3DArrow(double relHeadRadius, double relStemLength, double relStemRadi
 }
 
 void Draw_Vector(int Type, int Fill,
-		 double relHeadRadius, double relStemLength, double relStemRadius,
+                 double relHeadRadius, double relStemLength, double relStemRadius,
                  double x, double y, double z, double dx, double dy, double dz,
                  int light)
 {
   double length = sqrt(dx * dx + dy * dy + dz * dz);
 
-  if(length == 0.0)
-    return;
+  if(length == 0.0) return;
 
   switch(Type){
-  case DRAW_POST_SEGMENT:
+  case 1:
     glBegin(GL_LINES);
     glVertex3d(x, y, z);
     glVertex3d(x + dx, y + dy, z + dz);
     glEnd();
     break;
-  case DRAW_POST_ARROW:
+  case 2:
     Draw_SimpleVector(1, Fill, relHeadRadius, relStemLength, relStemRadius,
-		      x, y, z, dx, dy, dz, length, light);
+                      x, y, z, dx, dy, dz, length, light);
     break;
-  case DRAW_POST_PYRAMID:
+  case 3:
     Draw_SimpleVector(0, Fill, relHeadRadius, relStemLength, relStemRadius,
-		      x, y, z, dx, dy, dz, length, light);
+                      x, y, z, dx, dy, dz, length, light);
     break;
-  case DRAW_POST_ARROW3D:
+  case 4:
   default:
     Draw_3DArrow(relHeadRadius, relStemLength, relStemRadius,
-		 x, y, z, dx, dy, dz, length, light);
+                 x, y, z, dx, dy, dz, length, light);
     break;
   }
-
 }
 
 class point{
@@ -428,35 +422,35 @@ private:
     double _a, _b, _c, _d;
 public:
     plane(double a, double b, double c, double d) :
-	_a(a), _b(b), _c(c), _d(d) {;};
+        _a(a), _b(b), _c(c), _d(d) {;};
     double val(point &p){
-	return _a*p.x + _b*p.y + _c*p.z + _d;
+        return _a*p.x + _b*p.y + _c*p.z + _d;
     };
     point intersect(point &p1, point &p2){
-	double v1 = val(p1), v2 = val(p2);
-	if(fabs(v1) < 1.e-12){
-	    if(fabs(v2) < 1.e-12)
-		return point();
-	    else
-		return point(p1.x, p1.y, p1.z);
-	}
-	else if(fabs(v2) < 1.e-12){
-	    return point(p2.x, p2.y, p2.z);
-	}
-	else if(v1 * v2 < 0.){
-	    double coef = - v1 / (v2 - v1);
-	    return point(coef * (p2.x - p1.x) + p1.x,
-			 coef * (p2.y - p1.y) + p1.y,
-			 coef * (p2.z - p1.z) + p1.z);
-	}
-	else
-	    return point();
+        double v1 = val(p1), v2 = val(p2);
+        if(fabs(v1) < 1.e-12){
+            if(fabs(v2) < 1.e-12)
+                return point();
+            else
+                return point(p1.x, p1.y, p1.z);
+        }
+        else if(fabs(v2) < 1.e-12){
+            return point(p2.x, p2.y, p2.z);
+        }
+        else if(v1 * v2 < 0.){
+            double coef = - v1 / (v2 - v1);
+            return point(coef * (p2.x - p1.x) + p1.x,
+                         coef * (p2.y - p1.y) + p1.y,
+                         coef * (p2.z - p1.z) + p1.z);
+        }
+        else
+            return point();
     };
 };
 
 void Draw_Box(double xmin, double ymin, double zmin,
-	      double xmax, double ymax, double zmax,
-	      bool labels)
+              double xmax, double ymax, double zmax,
+              bool labels)
 {
   glBegin(GL_LINE_LOOP);
   glVertex3d(xmin, ymin, zmin);
@@ -484,22 +478,22 @@ void Draw_Box(double xmin, double ymin, double zmin,
     char label[256];
     double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
     glRasterPos3d(xmin + offset / CTX.s[0], 
-		  ymin + offset / CTX.s[1], 
-		  zmin + offset / CTX.s[2]);
+                  ymin + offset / CTX.s[1], 
+                  zmin + offset / CTX.s[2]);
     sprintf(label, "(%g,%g,%g)", xmin, ymin, zmin);
     Draw_String(label);
     glRasterPos3d(xmax + offset / CTX.s[0], 
-		  ymax + offset / CTX.s[1], 
-		  zmax + offset / CTX.s[2]);
+                  ymax + offset / CTX.s[1], 
+                  zmax + offset / CTX.s[2]);
     sprintf(label, "(%g,%g,%g)", xmax, ymax, zmax);
     Draw_String(label);
   }
 }
 
 void Draw_PlaneInBoundingBox(double xmin, double ymin, double zmin,
-			     double xmax, double ymax, double zmax,
-			     double a, double b, double c, double d,
-			     int shade)
+                             double xmax, double ymax, double zmax,
+                             double a, double b, double c, double d,
+                             int shade)
 {
 
   plane pl(a, b, c, d);
@@ -548,25 +542,25 @@ void Draw_PlaneInBoundingBox(double xmin, double ymin, double zmin,
     point p[4];
     for(int j = 0; j < 4; j++){
       if(edge[face[i][j]].valid == true)
-	p[nb++] = edge[face[i][j]];
+        p[nb++] = edge[face[i][j]];
     }
     if(nb > 1){
       for(int j = 1; j < nb; j++){
-	double xx[2] = {p[j].x, p[j-1].x};
-	double yy[2] = {p[j].y, p[j-1].y};
-	double zz[2] = {p[j].z, p[j-1].z};
-	Draw_Cylinder(CTX.line_width, xx, yy, zz, 1);
+        double xx[2] = {p[j].x, p[j-1].x};
+        double yy[2] = {p[j].y, p[j-1].y};
+        double zz[2] = {p[j].z, p[j-1].z};
+        Draw_Cylinder(CTX.line_width, xx, yy, zz, 1);
       }
       for(int j = 0; j < nb; j++){
-	Draw_3DArrow(CTX.arrow_rel_head_radius, 
-		     CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		     p[j].x, p[j].y, p[j].z, n[0], n[1], n[2], length, 1);
-	if(shade){
-	  p_shade[n_shade].x = p[j].x;
-	  p_shade[n_shade].y = p[j].y;
-	  p_shade[n_shade].z = p[j].z;
-	  n_shade++;
-	}
+        Draw_3DArrow(CTX.arrow_rel_head_radius, 
+                     CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
+                     p[j].x, p[j].y, p[j].z, n[0], n[1], n[2], length, 1);
+        if(shade){
+          p_shade[n_shade].x = p[j].x;
+          p_shade[n_shade].y = p[j].y;
+          p_shade[n_shade].z = p[j].z;
+          n_shade++;
+        }
       }
     }
   }
@@ -581,7 +575,7 @@ void Draw_PlaneInBoundingBox(double xmin, double ymin, double zmin,
     glNormal3d(n[0], n[1], n[2]);
     for(int j = 0; j < n_shade; j++){
       glVertex3d(p_shade[j].x, p_shade[j].y, p_shade[j].z);
-    }	
+    }   
     glEnd();
     glDisable(GL_LIGHTING);
     glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, twoside);
@@ -625,7 +619,7 @@ void Draw_SmallAxes()
 }
 
 int Draw_Tics(int comp, int n, char *format, char *label,
-	      double p1[3], double p2[3], double perp[3])
+              double p1[3], double p2[3], double perp[3], int mikado)
 {
   // draws n tic marks (in direction perp) and labels along the line p1->p2
 
@@ -668,15 +662,15 @@ int Draw_Tics(int comp, int n, char *format, char *label,
     glVertex3d(q[0], q[1], q[2]);
     glEnd();
 
-    if(i < n-1){
+    if(i < n-1 && !mikado){
       for(int j = 1; j < 10; j++){
-	double dd = d + j * step/10.;
-	double pp[3] = { p1[0]+t[0]*dd, p1[1]+t[1]*dd, p1[2]+t[2]*dd };
-	double qq[3] = { pp[0]+perp[0]*w2, pp[1]+perp[1]*w2, pp[2]+perp[2]*w2 };
-	glBegin(GL_LINES);
-	glVertex3d(pp[0], pp[1], pp[2]);
-	glVertex3d(qq[0], qq[1], qq[2]);
-	glEnd();
+        double dd = d + j * step/10.;
+        double pp[3] = { p1[0]+t[0]*dd, p1[1]+t[1]*dd, p1[2]+t[2]*dd };
+        double qq[3] = { pp[0]+perp[0]*w2, pp[1]+perp[1]*w2, pp[2]+perp[2]*w2 };
+        glBegin(GL_LINES);
+        glVertex3d(pp[0], pp[1], pp[2]);
+        glVertex3d(qq[0], qq[1], qq[2]);
+        glEnd();
       }
     }
 
@@ -720,20 +714,20 @@ void Draw_GridStipple(int n1, int n2, double p1[3], double p2[3], double p3[3])
   for(int i = 1; i < n1-1; i++){
     double d = (double)i/(double)(n1-1) * l1;
     glVertex3d(p1[0] + t1[0]*d,
-	       p1[1] + t1[1]*d,
-	       p1[2] + t1[2]*d);
+               p1[1] + t1[1]*d,
+               p1[2] + t1[2]*d);
     glVertex3d(p1[0] + t1[0]*d + t2[0]*l2, 
-	       p1[1] + t1[1]*d + t2[1]*l2, 
-	       p1[2] + t1[2]*d + t2[2]*l2);
+               p1[1] + t1[1]*d + t2[1]*l2, 
+               p1[2] + t1[2]*d + t2[2]*l2);
   }
   for(int i = 1; i < n2-1; i++){
     double d = (double)i/(double)(n2-1) * l2;
     glVertex3d(p1[0] + t2[0]*d, 
-	       p1[1] + t2[1]*d, 
-	       p1[2] + t2[2]*d);
+               p1[1] + t2[1]*d, 
+               p1[2] + t2[2]*d);
     glVertex3d(p1[0] + t2[0]*d + t1[0]*l1, 
-	       p1[1] + t2[1]*d + t1[1]*l1, 
-	       p1[2] + t2[2]*d + t1[2]*l1);
+               p1[1] + t2[1]*d + t1[1]*l1, 
+               p1[2] + t2[2]*d + t1[2]*l1);
   }
 
   glEnd();
@@ -742,7 +736,45 @@ void Draw_GridStipple(int n1, int n2, double p1[3], double p2[3], double p3[3])
 }
 
 void Draw_Axes(int mode, int tics[3], char format[3][256], char label[3][256], 
-	       double bb[6])
+               SBoundingBox3d &bb, int mikado)
+{
+  double bbox[6] = {bb.min().x(), bb.max().x(),
+                    bb.min().y(), bb.max().y(),
+                    bb.min().z(), bb.max().z()};
+  Draw_Axes(mode, tics, format, label, bbox, mikado);
+}
+
+void Draw_Axe(double xmin, double ymin, double zmin,
+              double xmax, double ymax, double zmax, int nticks, int mikado)
+{
+  if(mikado){
+    nticks = (nticks - 1) * mikado;
+    if(nticks < 1) nticks = 1;
+    double dd[3] = {(xmax - xmin) / nticks, 
+                    (ymax - ymin) / nticks, 
+                    (zmax - zmin) / nticks};
+    double axe_color[4];
+    glGetDoublev(GL_CURRENT_COLOR, axe_color);
+    for(int i = 1; i <= nticks; i++){
+      if(i % 2) glColor4dv(axe_color);
+      else glColor3f(1, 1, 1);
+      double cx[2] ={xmin + (i - 1) * dd[0], xmin + i * dd[0]};
+      double cy[2] ={ymin + (i - 1) * dd[1], ymin + i * dd[1]};
+      double cz[2] ={zmin + (i - 1) * dd[2], zmin + i * dd[2]};
+      Draw_Cylinder(3.5, cx, cy, cz, 1);
+    }
+    glColor4dv(axe_color);
+  }
+  else{
+    glBegin(GL_LINES);
+    glVertex3d(xmin, ymin, zmin); 
+    glVertex3d(xmax, ymax, zmax);
+    glEnd();
+  }
+}
+
+void Draw_Axes(int mode, int tics[3], char format[3][256], char label[3][256], 
+               double bb[6], int mikado)
 {
   // mode 0: nothing
   //      1: axes
@@ -760,9 +792,6 @@ void Draw_Axes(int mode, int tics[3], char format[3][256], char label[3][256],
   double orig[3] = {xmin, ymin, zmin};
 
   if(mode == 5){ // draw ruler from xyz_min to xyz_max
-    glBegin(GL_LINES);
-    glVertex3d(xmin, ymin, zmin); glVertex3d(xmax, ymax, zmax);
-    glEnd();
     double end[3] = {xmax, ymax, zmax};
     double dir[3] = {xmax-xmin, ymax-ymin, zmax-zmin};
     double perp[3];
@@ -773,42 +802,44 @@ void Draw_Axes(int mode, int tics[3], char format[3][256], char label[3][256],
     else{
       perp[0] = 0.; perp[1] = dir[2]; perp[2] = -dir[1];
     }
-    Draw_Tics(-1, tics[0], format[0], label[0], orig, end, perp);
+    Draw_Tics(-1, tics[0], format[0], label[0], orig, end, perp, mikado);
+    Draw_Axe(xmin, ymin, zmin, xmax, ymax, zmax, tics[0], mikado);
     return;
   }
-  
-  glBegin(GL_LINES);
-  // 3 axes
-  glVertex3d(xmin, ymin, zmin); glVertex3d(xmax, ymin, zmin);
-  glVertex3d(xmin, ymin, zmin); glVertex3d(xmin, ymax, zmin);
-  glVertex3d(xmin, ymin, zmin); glVertex3d(xmin, ymin, zmax);
-  // open box
-  if(mode > 1){
-    glVertex3d(xmin, ymax, zmin); glVertex3d(xmax, ymax, zmin);
-    glVertex3d(xmax, ymin, zmin); glVertex3d(xmax, ymax, zmin);
-    glVertex3d(xmin, ymin, zmax); glVertex3d(xmax, ymin, zmax);
-    glVertex3d(xmax, ymin, zmin); glVertex3d(xmax, ymin, zmax);
-    glVertex3d(xmin, ymin, zmax); glVertex3d(xmin, ymax, zmax);
-    glVertex3d(xmin, ymax, zmin); glVertex3d(xmin, ymax, zmax);
-  }
-  // closed box
-  if(mode == 2 || mode == 3){
-    glVertex3d(xmax, ymax, zmax); glVertex3d(xmin, ymax, zmax);
-    glVertex3d(xmax, ymax, zmax); glVertex3d(xmax, ymin, zmax);
-    glVertex3d(xmax, ymax, zmax); glVertex3d(xmax, ymax, zmin);
-  }
-  glEnd();
-
   double xx[3] = {xmax, ymin, zmin};
   double yy[3] = {xmin, ymax, zmin};
   double zz[3] = {xmin, ymin, zmax};
   double dxm[3] = {0., (ymin != ymax) ? -1. : 0., (zmin != zmax) ? -1. : 0.};
   double dym[3] = {(xmin != xmax) ? -1. : 0., 0., (zmin != zmax) ? -1. : 0.};
   double dzm[3] = {(xmin != xmax) ? -1. : 0., (ymin != ymax) ? -1. : 0., 0.};
+  
+  int nx = (xmin != xmax) ? 
+    Draw_Tics(0, tics[0], format[0], label[0], orig, xx, dxm, mikado) : 0;
+  int ny = (ymin != ymax) ? 
+    Draw_Tics(1, tics[1], format[1], label[1], orig, yy, dym, mikado) : 0;
+  int nz = (zmin != zmax) ? 
+    Draw_Tics(2, tics[2], format[2], label[2], orig, zz, dzm, mikado) : 0;
 
-  int nx = (xmin != xmax) ? Draw_Tics(0, tics[0], format[0], label[0], orig, xx, dxm) : 0;
-  int ny = (ymin != ymax) ? Draw_Tics(1, tics[1], format[1], label[1], orig, yy, dym) : 0;
-  int nz = (zmin != zmax) ? Draw_Tics(2, tics[2], format[2], label[2], orig, zz, dzm) : 0;
+  Draw_Axe(xmin, ymin, zmin, xmax, ymin, zmin, nx, mikado);
+  Draw_Axe(xmin, ymin, zmin, xmin, ymax, zmin, ny, mikado);
+  Draw_Axe(xmin, ymin, zmin, xmin, ymin, zmax, nz, mikado);
+
+  // open box
+  if(mode > 1){
+    Draw_Axe(xmin, ymax, zmin, xmax, ymax, zmin, nx, mikado);
+    Draw_Axe(xmax, ymin, zmin, xmax, ymax, zmin, ny, mikado);
+    Draw_Axe(xmax, ymin, zmin, xmax, ymin, zmax, nz, mikado);
+    Draw_Axe(xmin, ymin, zmax, xmax, ymin, zmax, nx, mikado);
+    Draw_Axe(xmin, ymin, zmax, xmin, ymax, zmax, ny, mikado);
+    Draw_Axe(xmin, ymax, zmin, xmin, ymax, zmax, nz, mikado);
+  }
+
+  // closed box
+  if(mode == 2 || mode == 3){
+    Draw_Axe(xmin, ymax, zmax, xmax, ymax, zmax, nx, mikado);
+    Draw_Axe(xmax, ymin, zmax, xmax, ymax, zmax, ny, mikado);
+    Draw_Axe(xmax, ymax, zmin, xmax, ymax, zmax, nz, mikado);
+  }
   
   if(mode > 2){
     Draw_GridStipple(nx, ny, orig, xx, yy);
