@@ -1,7 +1,7 @@
 #ifndef _ISO_H_
 #define _ISO_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,25 +20,21 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Views.h"
+int IsoLine(double *X, double *Y, double *Z, double *Val, double V,
+            double *Xp, double *Yp, double *Zp);
 
-void CutTriangle(double *X, double *Y, double *Z, double *Val,
-		 double V1, double V2, double *Xp, double *Yp, double *Zp, 
-		 int *nb, double *value);
+int IsoTriangle(double *X, double *Y, double *Z, double *Val, double V, 
+                double *Xp, double *Yp, double *Zp);
 
-void CutLine(double *X, double *Y, double *Z, double *Val,
-	     double V1, double V2, double *Xp, double *Yp, double *Zp,
-	     int *nb, double *value);
+int IsoSimplex(double *X, double *Y, double *Z, double *Val, double V,
+               double *Xp, double *Yp, double *Zp, double n[3]);
 
-void IsoLine(Post_View *View, double *X, double *Y, double *Z, 
-	     double *Val, double V);
+int CutLine(double *x, double *y, double *z, double *v,
+            double min, double max, 
+            double *xp, double *yp, double *zp, double *vp);
 
-void IsoTriangle(Post_View *View, 
-		 double *X, double *Y, double *Z, double *Val, double V,
-		 unsigned int color);
+int CutTriangle(double *x, double *y, double *z, double *v,
+                double min, double max, 
+                double *xp, double *yp, double *zp, double *vp);
 
-void IsoSimplex(Post_View *View, int preproNormals, 
-		double *X, double *Y, double *Z, double *Val, double V,
-		unsigned int color);
-  
 #endif
