@@ -1,6 +1,6 @@
-// $Id: Mesh.cpp,v 1.198 2007-03-11 20:18:58 geuzaine Exp $
+// $Id: Mesh.cpp,v 1.222 2008-06-12 11:52:00 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,16 +19,20 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Gmsh.h"
+#include <math.h>
+#include "Message.h"
 #include "GmshUI.h"
 #include "GModel.h"
+#include "MElement.h"
 #include "Draw.h"
 #include "Context.h"
-#include "MRep.h"
 #include "OS.h"
 #include "gl2ps.h"
+#include "VertexArray.h"
+#include "SmoothData.h"
+#include "PView.h"
+#include "PViewData.h"
 
-extern GModel *GMODEL;
 extern Context_T CTX;
 
 // General helper routines
@@ -74,14 +78,15 @@ static unsigned int getColorByElement(MElement *ele)
   else if(CTX.mesh.color_carousel == 3){ // by partition
     return CTX.color.mesh.carousel[abs(ele->getPartition() % 20)];
   }
-  else{ // by elementary or physical entity
-    // this is not perfect (e.g. a triangle can have no vertices
-    // categorized on a surface), but it's the best we can do "fast"
-    // since we don't store the associated entity in the element
+  else{ 
+    // by elementary or physical entity (this is not perfect (since
+    // e.g. a triangle can have no vertices categorized on a surface),
+    // but it's the best we can do "fast" since we don't store the
+    // associated entity in the element
     for(int i = 0; i < ele->getNumVertices(); i++){
       GEntity *e = ele->getVertex(i)->onWhat();
       if(e && (e->dim() == ele->getDim()))
-	return getColorByEntity(e);
+        return getColorByEntity(e);
     }
   }
   return CTX.color.fg;
@@ -136,7 +141,7 @@ template<class T>
 static bool areSomeElementsCurved(std::vector<T*> &elements)
 {
   for(unsigned int i = 0; i < elements.size(); i++)
-    if(elements[i]->getPolynomialOrder() > 1 ) return true;
+    if(elements[i]->getPolynomialOrder() > 1) return true;
   return false;
 }
 
@@ -152,7 +157,7 @@ static int getLabelStep(int total)
 
 template<class T>
 static void drawElementLabels(GEntity *e, std::vector<T*> &elements,
-			      int forceColor=0, unsigned int color=0)
+                              int forceColor=0, unsigned int color=0)
 {
   unsigned col = forceColor ? color : getColorByEntity(e);
   glColor4ubv((GLubyte *) & col);
@@ -166,18 +171,18 @@ static void drawElementLabels(GEntity *e, std::vector<T*> &elements,
       SPoint3 pc = ele->barycenter();
       char str[256];
       if(CTX.mesh.label_type == 4)
-	sprintf(str, "(%g,%g,%g)", pc.x(), pc.y(), pc.z());
+        sprintf(str, "(%g,%g,%g)", pc.x(), pc.y(), pc.z());
       else if(CTX.mesh.label_type == 3)
-	sprintf(str, "%d", ele->getPartition());
+        sprintf(str, "%d", ele->getPartition());
       else if(CTX.mesh.label_type == 2){
-	int np = e->physicals.size();
-	int p = np ? e->physicals[np - 1] : 0;
-	sprintf(str, "%d", p);
+        int np = e->physicals.size();
+        int p = np ? e->physicals[np - 1] : 0;
+        sprintf(str, "%d", p);
       }
       else if(CTX.mesh.label_type == 1)
-	sprintf(str, "%d", e->tag());
+        sprintf(str, "%d", e->tag());
       else
-	sprintf(str, "%d", ele->getNum());
+        sprintf(str, "%d", ele->getNum());
       glRasterPos3d(pc.x(), pc.y(), pc.z());
       Draw_String(str);
     }
@@ -191,13 +196,13 @@ static void drawNormals(std::vector<T*> &elements)
   for(unsigned int i = 0; i < elements.size(); i++){
     MElement *ele = elements[i];
     if(!isElementVisible(ele)) continue;
-    SVector3 n = ele->getFaceRep(0).normal();
+    SVector3 n = ele->getFace(0).normal();
     for(int j = 0; j < 3; j++)
       n[j] *= CTX.mesh.normals * CTX.pixel_equiv_x / CTX.s[j];
     SPoint3 pc = ele->barycenter();
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
-		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		pc.x(), pc.y(), pc.z(), n[0], n[1], n[2], CTX.mesh.light);
+                CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
+                pc.x(), pc.y(), pc.z(), n[0], n[1], n[2], CTX.mesh.light);
   }
 }
 
@@ -208,18 +213,20 @@ static void drawTangents(std::vector<T*> &elements)
   for(unsigned int i = 0; i < elements.size(); i++){
     MElement *ele = elements[i];
     if(!isElementVisible(ele)) continue;
-    SVector3 t = ele->getEdgeRep(0).tangent();
+    SVector3 t = ele->getEdge(0).tangent();
     for(int j = 0; j < 3; j++)
       t[j] *= CTX.mesh.tangents * CTX.pixel_equiv_x / CTX.s[j];
     SPoint3 pc = ele->barycenter();
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
-		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
-		pc.x(), pc.y(), pc.z(), t[0], t[1], t[2], CTX.mesh.light);
+                CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
+                pc.x(), pc.y(), pc.z(), t[0], t[1], t[2], CTX.mesh.light);
   }
 }
 
 static void drawVertexLabel(GEntity *e, MVertex *v, int partition=-1)
 {
+  if(!v->getVisibility()) return;
+
   int np = e->physicals.size();
   int physical = np ? e->physicals[np - 1] : 0;
   char str[256];
@@ -241,7 +248,7 @@ static void drawVertexLabel(GEntity *e, MVertex *v, int partition=-1)
   if(v->getPolynomialOrder() > 1)
     glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
   else
-    glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);	
+    glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);   
   glRasterPos3d(v->x(), v->y(), v->z());
   Draw_String(str);
 }
@@ -251,25 +258,25 @@ static void drawVerticesPerEntity(GEntity *e)
   if(CTX.mesh.points) {
     if(CTX.mesh.point_type) {
       for(unsigned int i = 0; i < e->mesh_vertices.size(); i++){
-	MVertex *v = e->mesh_vertices[i];
-	if(!v->getVisibility()) continue;
-	if(v->getPolynomialOrder() > 1)
-	  glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
-	else
-	  glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);	
-	Draw_Sphere(CTX.mesh.point_size, v->x(), v->y(), v->z(), CTX.mesh.light);
+        MVertex *v = e->mesh_vertices[i];
+        if(!v->getVisibility()) continue;
+        if(v->getPolynomialOrder() > 1)
+          glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
+        else
+          glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);     
+        Draw_Sphere(CTX.mesh.point_size, v->x(), v->y(), v->z(), CTX.mesh.light);
       }
     }
     else{
       glBegin(GL_POINTS);
       for(unsigned int i = 0; i < e->mesh_vertices.size(); i++){
-	MVertex *v = e->mesh_vertices[i];
-	if(!v->getVisibility()) continue;
-	if(v->getPolynomialOrder() > 1)
-	  glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
-	else
-	  glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);	
-	glVertex3d(v->x(), v->y(), v->z());
+        MVertex *v = e->mesh_vertices[i];
+        if(!v->getVisibility()) continue;
+        if(v->getPolynomialOrder() > 1)
+          glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
+        else
+          glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);     
+        glVertex3d(v->x(), v->y(), v->z());
       }
       glEnd();
     }
@@ -289,21 +296,21 @@ static void drawVerticesPerElement(GEntity *e, std::vector<T*> &elements)
     for(int j = 0; j < ele->getNumVertices(); j++){
       MVertex *v = ele->getVertex(j);
       if(isElementVisible(ele) && v->getVisibility()){
-	if(CTX.mesh.points) {
-	  if(v->getPolynomialOrder() > 1)
-	    glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
-	  else
-	    glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);	
-	  if(CTX.mesh.point_type)
-	    Draw_Sphere(CTX.mesh.point_size, v->x(), v->y(), v->z(), CTX.mesh.light);
-	  else{
-	    glBegin(GL_POINTS);
-	    glVertex3d(v->x(), v->y(), v->z());
-	    glEnd();
-	  }
-	}
-	if(CTX.mesh.points_num)
-	  drawVertexLabel(e, v);
+        if(CTX.mesh.points) {
+          if(v->getPolynomialOrder() > 1)
+            glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_sup);
+          else
+            glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);   
+          if(CTX.mesh.point_type)
+            Draw_Sphere(CTX.mesh.point_size, v->x(), v->y(), v->z(), CTX.mesh.light);
+          else{
+            glBegin(GL_POINTS);
+            glVertex3d(v->x(), v->y(), v->z());
+            glEnd();
+          }
+        }
+        if(CTX.mesh.points_num)
+          drawVertexLabel(e, v);
       }
     }
   }
@@ -323,25 +330,25 @@ static void drawBarycentricDual(std::vector<T*> &elements)
     SPoint3 pc = ele->barycenter();
     if(ele->getDim() == 2){
       for(int j = 0; j < ele->getNumEdges(); j++){
-	MEdge e = ele->getEdge(j);
-	SPoint3 p = e.barycenter();
-	glVertex3d(pc.x(), pc.y(), pc.z());
-	glVertex3d(p.x(), p.y(), p.z());
+        MEdge e = ele->getEdge(j);
+        SPoint3 p = e.barycenter();
+        glVertex3d(pc.x(), pc.y(), pc.z());
+        glVertex3d(p.x(), p.y(), p.z());
       }
     }
     else if(ele->getDim() == 3){
       for(int j = 0; j < ele->getNumFaces(); j++){
-	MFace f = ele->getFace(j);
-	SPoint3 p = f.barycenter();
-	glVertex3d(pc.x(), pc.y(), pc.z());
-	glVertex3d(p.x(), p.y(), p.z());
-	for(int k = 0; k < f.getNumVertices(); k++){
-	  MEdge e(f.getVertex(k), (k == f.getNumVertices() - 1) ? 
-		  f.getVertex(0) : f.getVertex(k + 1));
-	  SPoint3 pe = e.barycenter();
-	  glVertex3d(p.x(), p.y(), p.z());
-	  glVertex3d(pe.x(), pe.y(), pe.z());
-	}
+        MFace f = ele->getFace(j);
+        SPoint3 p = f.barycenter();
+        glVertex3d(pc.x(), pc.y(), pc.z());
+        glVertex3d(p.x(), p.y(), p.z());
+        for(int k = 0; k < f.getNumVertices(); k++){
+          MEdge e(f.getVertex(k), (k == f.getNumVertices() - 1) ? 
+                  f.getVertex(0) : f.getVertex(k + 1));
+          SPoint3 pe = e.barycenter();
+          glVertex3d(p.x(), p.y(), p.z());
+          glVertex3d(pe.x(), pe.y(), pe.z());
+        }
       }
     }
   }
@@ -357,19 +364,19 @@ static void addSmoothNormals(GEntity *e, std::vector<T*> &elements)
 {
   for(unsigned int i = 0; i < elements.size(); i++){
     MElement *ele = elements[i];
+    SPoint3 pc(0., 0., 0.);
+    if(CTX.mesh.explode != 1.) pc = ele->barycenter();
     for(int j = 0; j < ele->getNumFacesRep(); j++){
-      MFace fr = ele->getFaceRep(j);
-      SVector3 n = fr.normal();
-      SPoint3 pc;
-      if(CTX.mesh.explode != 1.) pc = ele->barycenter();
-      for(int k = 0; k < fr.getNumVertices(); k++){
-	MVertex *v = fr.getVertex(k);
-	SPoint3 p(v->x(), v->y(), v->z());
-	if(CTX.mesh.explode != 1.){
-	  for(int l = 0; l < 3; l++)
-	    p[l] = pc[l] + CTX.mesh.explode * (p[l] - pc[l]);
-	}
-	e->model()->normals->add(p[0], p[1], p[2], n[0], n[1], n[2]);
+      double x[3], y[3], z[3];
+      SVector3 n[3];
+      ele->getFaceRep(j, x, y, z, n);
+      for(int k = 0; k < 3; k++){
+        if(CTX.mesh.explode != 1.){
+          x[k] = pc[0] + CTX.mesh.explode * (x[k] - pc[0]);
+          y[k] = pc[1] + CTX.mesh.explode * (y[k] - pc[1]);
+          z[k] = pc[2] + CTX.mesh.explode * (z[k] - pc[2]);
+        }
+        e->model()->normals->add(x[k], y[k], z[k], n[k][0], n[k][1], n[k][2]);
       }
     }
   }
@@ -377,97 +384,69 @@ static void addSmoothNormals(GEntity *e, std::vector<T*> &elements)
 
 // Routines for filling and drawing the vertex arrays
 
-static void addEdgesInArrays(GEntity *e)
-{
-  MRep *m = e->meshRep;
-  for(MRep::eriter it = m->firstEdgeRep(); it != m->lastEdgeRep(); ++it){
-    MVertex *v[2] = {it->first.first, it->first.second};
-    MElement *ele = it->second;
-    SVector3 n = ele->getFaceRep(0).normal();
-    unsigned int color = getColorByElement(ele);
-    for(int i = 0; i < 2; i++){
-      if(e->dim() == 2 && CTX.mesh.smooth_normals)
-	e->model()->normals->get(v[i]->x(), v[i]->y(), v[i]->z(), n[0], n[1], n[2]);
-      m->va_lines->add(v[i]->x(), v[i]->y(), v[i]->z(), n[0], n[1], n[2], color, ele);
-    }
-  }
-}
-
-static void addFacesInArrays(GEntity *e)
-{
-  MRep *m = e->meshRep;
-  for(MRep::friter it = m->firstFaceRep(); it != m->lastFaceRep(); ++it){
-    MFace f = it->first;
-    MElement *ele = it->second;
-    SVector3 n = f.normal();
-    unsigned int color = getColorByElement(ele);
-    for(int i = 0; i < f.getNumVertices(); i++){
-      MVertex *v = f.getVertex(i);
-      if(CTX.mesh.smooth_normals)
-	e->model()->normals->get(v->x(), v->y(), v->z(), n[0], n[1], n[2]);
-      if(f.getNumVertices() == 3)
-	m->va_triangles->add(v->x(), v->y(), v->z(), n[0], n[1], n[2], color, ele);
-      else if(f.getNumVertices() == 4)
-	m->va_quads->add(v->x(), v->y(), v->z(), n[0], n[1], n[2], color, ele);
-    }
-  }
-}
-
 template<class T>
-static void addElementsInArrays(GEntity *e, std::vector<T*> &elements)
+static void addElementsInArrays(GEntity *e, std::vector<T*> &elements,
+                                bool edges, bool faces)
 {
-  MRep *m = e->meshRep;
   for(unsigned int i = 0; i < elements.size(); i++){
     MElement *ele = elements[i];
-    if(!isElementVisible(ele)) continue;
-    if(CTX.mesh.use_cut_plane && CTX.mesh.cut_plane_draw_intersect){
+
+    if(!isElementVisible(ele) || ele->getDim() < 1) continue;
+    
+    if(CTX.mesh.use_cut_plane && CTX.mesh.cut_plane_draw_intersect)
       if(e->dim() == 3 && intersectCutPlane(ele)) continue;
-    }
-    unsigned int color = getColorByElement(ele);
-    SPoint3 pc;
+    
+    unsigned int c = getColorByElement(ele);
+    unsigned int col[4] = {c, c, c, c};
+
+    SPoint3 pc(0., 0., 0.);
     if(CTX.mesh.explode != 1.) pc = ele->barycenter();
-    if(ele->getNumFacesRep()){
-      for(int j = 0; j < ele->getNumFacesRep(); j++){
-	MFace fr = ele->getFaceRep(j);
-	SVector3 n = fr.normal();
-	int numverts = fr.getNumVertices();
-	for(int k = 0; k < numverts; k++){
-	  MVertex *v = fr.getVertex(k);
-	  SPoint3 p(v->x(), v->y(), v->z());
-	  if(CTX.mesh.explode != 1.){
-	    for(int l = 0; l < 3; l++)
-	      p[l] = pc[l] + CTX.mesh.explode * (p[l] - pc[l]);
-	  }
-	  if(e->dim() == 2 && CTX.mesh.smooth_normals)
-	    e->model()->normals->get(p[0], p[1], p[2], n[0], n[1], n[2]);
-	  if(numverts == 3)
-	    m->va_triangles->add(p[0], p[1], p[2], n[0], n[1], n[2], color, ele);
-	  else if(numverts == 4)
-	    m->va_quads->add(p[0], p[1], p[2], n[0], n[1], n[2], color, ele);
-	}
+
+    if(edges){
+      bool unique = e->dim() > 1 && !CTX.pick_elements;
+      for(int j = 0; j < ele->getNumEdgesRep(); j++){
+        double x[2], y[2], z[2];
+        SVector3 n[2];
+        ele->getEdgeRep(j, x, y, z, n);
+        if(CTX.mesh.explode != 1.){
+          for(int k = 0; k < 2; k++){
+            x[k] = pc[0] + CTX.mesh.explode * (x[k] - pc[0]);
+            y[k] = pc[1] + CTX.mesh.explode * (y[k] - pc[1]);
+            z[k] = pc[2] + CTX.mesh.explode * (z[k] - pc[2]);
+          }
+        }
+        if(e->dim() == 2 && CTX.mesh.smooth_normals)
+          for(int k = 0; k < 2; k++)
+            e->model()->normals->get(x[k], y[k], z[k], n[k][0], n[k][1], n[k][2]);
+        e->va_lines->add(x, y, z, n, col, ele, unique);
       }
     }
-    if(!ele->getNumFacesRep() || ele->getPolynomialOrder() > 1){
-      SPoint3 pc;
-      if(CTX.mesh.explode != 1.) pc = ele->barycenter();
-      for(int j = 0; j < ele->getNumEdgesRep(); j++){
-	MEdge er = ele->getEdgeRep(j);
-	for(int k = 0; k < er.getNumVertices(); k++){
-	  MVertex *v = er.getVertex(k);
-	  SPoint3 p(v->x(), v->y(), v->z());
-	  if(CTX.mesh.explode != 1.){
-	    for(int l = 0; l < 3; l++)
-	      p[l] = pc[l] + CTX.mesh.explode * (p[l] - pc[l]);
-	  }
-	  m->va_lines->add(p[0], p[1], p[2], color, ele);
-	}
+
+    if(faces){
+      bool unique = e->dim() > 2 && !CTX.pick_elements;
+      bool skin = e->dim() > 2 && CTX.mesh.draw_skin_only;
+      for(int j = 0; j < ele->getNumFacesRep(); j++){
+        double x[3], y[3], z[3];
+        SVector3 n[3];
+        ele->getFaceRep(j, x, y, z, n);
+        if(CTX.mesh.explode != 1.){
+          for(int k = 0; k < 3; k++){
+            x[k] = pc[0] + CTX.mesh.explode * (x[k] - pc[0]);
+            y[k] = pc[1] + CTX.mesh.explode * (y[k] - pc[1]);
+            z[k] = pc[2] + CTX.mesh.explode * (z[k] - pc[2]);
+          }
+        }
+        if(e->dim() == 2 && CTX.mesh.smooth_normals)
+          for(int k = 0; k < 3; k++)
+            e->model()->normals->get(x[k], y[k], z[k], n[k][0], n[k][1], n[k][2]);
+        e->va_triangles->add(x, y, z, n, col, ele, unique, skin);
       }
     }
   }
 }
 
 static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalArray, 
-		       int forceColor=0, unsigned int color=0, bool drawOutline=false)
+                       int forceColor=0, unsigned int color=0)
 {
   if(!va) return;
 
@@ -475,15 +454,15 @@ static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalAr
   // draw each one separately
   if(CTX.render_mode == GMSH_SELECT && CTX.pick_elements) {
     if(va->getNumElementPointers() == va->getNumVertices()){
-      for(int i = 0; i < va->getNumVertices(); i += va->getType()){
-	glPushName(va->getType());
-	glPushName(i);
-	glBegin(type);
-	for(int j = 0; j < va->getType(); j++)
-	  glVertex3fv(va->getVertexArray(3 * (i + j)));
-	glEnd();
-	glPopName();
-	glPopName();
+      for(int i = 0; i < va->getNumVertices(); i += va->getNumVerticesPerElement()){
+        glPushName(va->getNumVerticesPerElement());
+        glPushName(i);
+        glBegin(type);
+        for(int j = 0; j < va->getNumVerticesPerElement(); j++)
+          glVertex3fv(va->getVertexArray(3 * (i + j)));
+        glEnd();
+        glPopName();
+        glPopName();
       }
       return;
     }
@@ -506,11 +485,9 @@ static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalAr
     glDisableClientState(GL_COLOR_ARRAY);
     glColor4ubv((GLubyte *) & color);
   }
-  else if(CTX.pick_elements){
-    glEnableClientState(GL_COLOR_ARRAY);
-  }
-  else if(!e->getSelection() && (CTX.mesh.color_carousel == 0 || 
-				 CTX.mesh.color_carousel == 3)){
+  else if(CTX.pick_elements || 
+          (!e->getSelection() && (CTX.mesh.color_carousel == 0 || 
+                                  CTX.mesh.color_carousel == 3))){
     glEnableClientState(GL_COLOR_ARRAY);
   }
   else{
@@ -519,15 +496,11 @@ static void drawArrays(GEntity *e, VertexArray *va, GLint type, bool useNormalAr
     glColor4ubv((GLubyte *) & color);
   }
   
-  if(va->getType() > 2 && !drawOutline && CTX.polygon_offset)
+  if(va->getNumVerticesPerElement() > 2 && CTX.polygon_offset)
     glEnable(GL_POLYGON_OFFSET_FILL);
   
-  if(drawOutline) 
-    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-  
   glDrawArrays(type, 0, va->getNumVertices());
   
-  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
   glDisable(GL_POLYGON_OFFSET_FILL);
   glDisable(GL_LIGHTING);
   
@@ -562,20 +535,28 @@ class drawMeshGVertex {
 // GEdge drawing routines
 
 class initMeshGEdge {
- public :
+ private:
+  int _estimateNumLines(GEdge *e)
+  {
+    int num = 0;
+    if(CTX.mesh.lines){
+      num += e->lines.size();
+      if(areSomeElementsCurved(e->lines)) num *= 2;
+    }
+    return num + 100;
+  }
+ public:
   void operator () (GEdge *e)
   {
     if(!e->getVisibility()) return;
 
-    if(!e->meshRep) e->meshRep = new MRepEdge(e);
-
-    MRep *m = e->meshRep;
-    m->resetArrays();
-    m->allElementsVisible = areAllElementsVisible(e->lines);
+    e->deleteVertexArrays();
+    e->setAllElementsVisible(CTX.mesh.lines && areAllElementsVisible(e->lines));
 
     if(CTX.mesh.lines){
-      m->va_lines = new VertexArray(2, e->lines.size());
-      addElementsInArrays(e, e->lines);
+      e->va_lines = new VertexArray(2, _estimateNumLines(e));
+      addElementsInArrays(e, e->lines, CTX.mesh.lines, false);
+      e->va_lines->finalize();
     }
   }
 };
@@ -586,26 +567,22 @@ class drawMeshGEdge {
   {  
     if(!e->getVisibility()) return;
     
-    MRep *m = e->meshRep;
-
-    if(!m) return;
-
     if(CTX.render_mode == GMSH_SELECT) {
       glPushName(1);
       glPushName(e->tag());
     }
 
     if(CTX.mesh.lines)
-      drawArrays(e, m->va_lines, GL_LINES, false);
+      drawArrays(e, e->va_lines, GL_LINES, false);
 
     if(CTX.mesh.lines_num)
       drawElementLabels(e, e->lines);
 
     if(CTX.mesh.points || CTX.mesh.points_num){
-      if(m->allElementsVisible)
-	drawVerticesPerEntity(e);
+      if(e->getAllElementsVisible())
+        drawVerticesPerEntity(e);
       else
-	drawVerticesPerElement(e, e->lines);
+        drawVerticesPerElement(e, e->lines);
     }
 
     if(CTX.mesh.tangents)
@@ -621,7 +598,7 @@ class drawMeshGEdge {
 // GFace drawing routines
 
 class initSmoothNormalsGFace {
- public :
+ public:
   void operator () (GFace *f)
   {
     addSmoothNormals(f, f->triangles);
@@ -630,50 +607,48 @@ class initSmoothNormalsGFace {
 };
 
 class initMeshGFace {
- public :
+ private:
+  bool _curved;
+  int _estimateNumLines(GFace *f)
+  {
+    int num = 0;
+    if(CTX.mesh.surfaces_edges){
+      num += (3 * f->triangles.size() + 4 * f->quadrangles.size()) / 2;
+      if(CTX.mesh.explode != 1.) num *= 2;
+      if(_curved) num *= 2;
+    }
+    return num + 100;
+  }
+  int _estimateNumTriangles(GFace *f)
+  {
+    int num = 0;
+    if(CTX.mesh.surfaces_faces){
+      num += (f->triangles.size() + 2 * f->quadrangles.size());
+      if(_curved) num *= 4;
+    }
+    return num + 100;
+  }
+ public:
   void operator () (GFace *f)
   {
     if(!f->getVisibility()) return;
 
-    if(!f->meshRep) f->meshRep = new MRepFace(f);
+    f->deleteVertexArrays();
+    f->setAllElementsVisible
+      (CTX.mesh.triangles && areAllElementsVisible(f->triangles) && 
+       CTX.mesh.quadrangles && areAllElementsVisible(f->quadrangles));
 
-    MRep *m = f->meshRep;
-    m->resetArrays();
-    m->allElementsVisible = 
-      CTX.mesh.triangles && areAllElementsVisible(f->triangles) && 
-      CTX.mesh.quadrangles && areAllElementsVisible(f->quadrangles);
-
-    bool curvedElements = 
-      areSomeElementsCurved(f->triangles) || 
-      areSomeElementsCurved(f->quadrangles);
-
-    bool useEdges = CTX.mesh.surfaces_edges ? true : false;
-    if(CTX.mesh.surfaces_faces || CTX.mesh.explode != 1. || 
-       CTX.pick_elements || !m->allElementsVisible)
-      useEdges = false; // cannot use edges in these cases
-
-    // Further optimizations are possible when useEdges is true:
-    // 1) store the unique vertices in the vertex array and use
-    //    glDrawElements() instead of glDrawArrays(). Question:
-    //    which normal do you choose for each vertex, and so how 
-    //    can you achieve accurate "flat shading" rendering?
-    // 2) use tc to stripe the triangles and draw strips instead of 
-    //    individual triangles
-    
-    if(useEdges){
-      Msg(DEBUG, "Using edges to draw surface %d", f->tag());
-      m->generateEdgeRep();
-      m->va_lines = new VertexArray(2, m->getNumEdgeRep());
-      addEdgesInArrays(f);
-    }
-    else if(CTX.mesh.surfaces_edges || CTX.mesh.surfaces_faces){
-      if(curvedElements) // cannot simply draw polygon outlines!
-	m->va_lines = new VertexArray(2, 6 * f->triangles.size() 
-				      + 8 * f->quadrangles.size());
-      m->va_triangles = new VertexArray(3, f->triangles.size());
-      m->va_quads = new VertexArray(4, f->quadrangles.size());
-      if(CTX.mesh.triangles) addElementsInArrays(f, f->triangles);
-      if(CTX.mesh.quadrangles) addElementsInArrays(f, f->quadrangles);
+    bool edg = CTX.mesh.surfaces_edges;
+    bool fac = CTX.mesh.surfaces_faces;
+    if(edg || fac){
+      _curved = (areSomeElementsCurved(f->triangles) ||
+                 areSomeElementsCurved(f->quadrangles));
+      f->va_lines = new VertexArray(2, _estimateNumLines(f));
+      f->va_triangles = new VertexArray(3, _estimateNumTriangles(f));
+      if(CTX.mesh.triangles) addElementsInArrays(f, f->triangles, edg, fac);
+      if(CTX.mesh.quadrangles) addElementsInArrays(f, f->quadrangles, edg, fac);
+      f->va_lines->finalize();
+      f->va_triangles->finalize();
     }
   }
 };
@@ -684,46 +659,28 @@ class drawMeshGFace {
   {  
     if(!f->getVisibility()) return;
 
-    MRep *m = f->meshRep;
-
-    if(!m) return;
-
     if(CTX.render_mode == GMSH_SELECT) {
       glPushName(2);
       glPushName(f->tag());
     }
 
-    if(CTX.mesh.surfaces_edges){
-      if(m->va_lines && m->va_lines->getNumVertices()){
-	drawArrays(f, m->va_lines, GL_LINES, CTX.mesh.light && CTX.mesh.light_lines, 
-		   CTX.mesh.surfaces_faces, CTX.color.mesh.line);
-      }
-      else{
-	drawArrays(f, m->va_triangles, GL_TRIANGLES, CTX.mesh.light && CTX.mesh.light_lines,
-		   CTX.mesh.surfaces_faces, CTX.color.mesh.line, true);
-	drawArrays(f, m->va_quads, GL_QUADS, CTX.mesh.light && CTX.mesh.light_lines, 
-		   CTX.mesh.surfaces_faces, CTX.color.mesh.line, true);
-      }
-    }
-    
-    if(CTX.mesh.surfaces_faces){
-      drawArrays(f, m->va_triangles, GL_TRIANGLES, CTX.mesh.light);
-      drawArrays(f, m->va_quads, GL_QUADS, CTX.mesh.light);
-    }
+    drawArrays(f, f->va_lines, GL_LINES, CTX.mesh.light && CTX.mesh.light_lines, 
+               CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+    drawArrays(f, f->va_triangles, GL_TRIANGLES, CTX.mesh.light);
 
     if(CTX.mesh.surfaces_num) {
       if(CTX.mesh.triangles)
-	drawElementLabels(f, f->triangles, CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+        drawElementLabels(f, f->triangles, CTX.mesh.surfaces_faces, CTX.color.mesh.line);
       if(CTX.mesh.quadrangles)
-	drawElementLabels(f, f->quadrangles, CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+        drawElementLabels(f, f->quadrangles, CTX.mesh.surfaces_faces, CTX.color.mesh.line);
     }
 
     if(CTX.mesh.points || CTX.mesh.points_num){
-      if(m->allElementsVisible)
-	drawVerticesPerEntity(f);
+      if(f->getAllElementsVisible())
+        drawVerticesPerEntity(f);
       else{
-	if(CTX.mesh.triangles) drawVerticesPerElement(f, f->triangles);
-	if(CTX.mesh.quadrangles) drawVerticesPerElement(f, f->quadrangles);
+        if(CTX.mesh.triangles) drawVerticesPerElement(f, f->triangles);
+        if(CTX.mesh.quadrangles) drawVerticesPerElement(f, f->quadrangles);
       }
     }
 
@@ -747,126 +704,104 @@ class drawMeshGFace {
 // GRegion drawing routines
 
 class initMeshGRegion {
- public :
+ private:
+  bool _curved;
+  int _estimateNumLines(GRegion *r)
+  {
+    int num = 0;
+    if(CTX.mesh.volumes_edges){
+      // suppose edge shared by 4 elements on averge (pessmistic)
+      num += (12 * r->tetrahedra.size() + 24 * r->hexahedra.size() +
+              18 * r->prisms.size() + 16 * r->pyramids.size()) / 4;
+      if(CTX.mesh.use_cut_plane && CTX.mesh.cut_plane_draw_intersect)
+        num = (int)sqrt((double)num);
+      if(CTX.mesh.explode != 1.) num *= 4;
+      if(_curved) num *= 2;
+    }
+    return num + 100;
+  }
+  int _estimateNumTriangles(GRegion *r)
+  {
+    int num = 0;
+    if(CTX.mesh.volumes_faces){
+      num += (4 * r->tetrahedra.size() + 12 * r->hexahedra.size() +
+              8 * r->prisms.size() + 6 * r->pyramids.size()) / 2;
+      if(CTX.mesh.use_cut_plane && CTX.mesh.cut_plane_draw_intersect)
+        num = (int)sqrt((double)num);
+      if(CTX.mesh.explode != 1.) num *= 2;
+      if(_curved) num *= 4;
+    }
+    return num + 100;
+  }
+ public:
   void operator () (GRegion *r)
   {  
     if(!r->getVisibility()) return;
 
-    if(!r->meshRep) r->meshRep = new MRepRegion(r);
+    r->deleteVertexArrays();
+    r->setAllElementsVisible
+      (CTX.mesh.tetrahedra && areAllElementsVisible(r->tetrahedra) &&
+       CTX.mesh.hexahedra && areAllElementsVisible(r->hexahedra) &&
+       CTX.mesh.prisms && areAllElementsVisible(r->prisms) &&
+       CTX.mesh.pyramids && areAllElementsVisible(r->pyramids));
 
-    MRep *m = r->meshRep;
-    m->resetArrays();
-    m->allElementsVisible = 
-      CTX.mesh.tetrahedra && areAllElementsVisible(r->tetrahedra) &&
-      CTX.mesh.hexahedra && areAllElementsVisible(r->hexahedra) &&
-      CTX.mesh.prisms && areAllElementsVisible(r->prisms) &&
-      CTX.mesh.pyramids && areAllElementsVisible(r->pyramids);
-
-    bool useEdges = CTX.mesh.volumes_edges ? true : false;
-    if(CTX.mesh.volumes_faces || CTX.mesh.explode != 1. || 
-       CTX.pick_elements || !m->allElementsVisible)
-      useEdges = false; // cannot use edges in these cases
-
-    bool curvedElements = 
-      areSomeElementsCurved(r->tetrahedra) || 
-      areSomeElementsCurved(r->hexahedra) ||
-      areSomeElementsCurved(r->prisms) ||
-      areSomeElementsCurved(r->pyramids);
-
-    bool useSkin = CTX.mesh.volumes_faces ? true : false;
-    if(CTX.mesh.explode != 1. || !m->allElementsVisible)
-      useSkin = false;
-    useSkin = false; // FIXME: to do
-    
-    if(useSkin){ 
-      Msg(DEBUG, "Using boundary faces to draw volume %d", r->tag());
-      m->generateBoundaryFaceRep(); 
-      m->va_triangles = new VertexArray(3, m->getNumFaceRep());
-      m->va_quads = new VertexArray(4, m->getNumFaceRep());
-      addFacesInArrays(r);
-    }
-    else if(useEdges){
-      Msg(DEBUG, "Using edges to draw volume %d", r->tag());
-      m->generateEdgeRep();
-      m->va_lines = new VertexArray(2, m->getNumEdgeRep());
-      addEdgesInArrays(r);
-    }
-    else if(CTX.mesh.volumes_edges || CTX.mesh.volumes_faces){
-      if(curvedElements) // cannot simply draw polygon outlines!
-	m->va_lines = new VertexArray(2, 12 * r->tetrahedra.size() +
-				      24 * r->hexahedra.size() +
-				      18 * r->prisms.size() +
-				      16 * r->pyramids.size());
-      m->va_triangles = new VertexArray(3, 4 * r->tetrahedra.size() +
-					2 * r->prisms.size() + 
-					4 * r->pyramids.size());
-      m->va_quads = new VertexArray(4, 6 * r->hexahedra.size() +
-				    3 * r->prisms.size() +
-				    r->pyramids.size());
-      if(CTX.mesh.tetrahedra) addElementsInArrays(r, r->tetrahedra);
-      if(CTX.mesh.hexahedra) addElementsInArrays(r, r->hexahedra);
-      if(CTX.mesh.prisms) addElementsInArrays(r, r->prisms);
-      if(CTX.mesh.pyramids) addElementsInArrays(r, r->pyramids);
+    bool edg = CTX.mesh.volumes_edges;
+    bool fac = CTX.mesh.volumes_faces;
+    if(edg || fac){
+      _curved = (areSomeElementsCurved(r->tetrahedra) || 
+                 areSomeElementsCurved(r->hexahedra) ||
+                 areSomeElementsCurved(r->prisms) ||
+                 areSomeElementsCurved(r->pyramids));
+      r->va_lines = new VertexArray(2, _estimateNumLines(r));
+      r->va_triangles = new VertexArray(3, _estimateNumTriangles(r));
+      if(CTX.mesh.tetrahedra) addElementsInArrays(r, r->tetrahedra, edg, fac);
+      if(CTX.mesh.hexahedra) addElementsInArrays(r, r->hexahedra, edg, fac);
+      if(CTX.mesh.prisms) addElementsInArrays(r, r->prisms, edg, fac);
+      if(CTX.mesh.pyramids) addElementsInArrays(r, r->pyramids, edg, fac);
+      r->va_lines->finalize();
+      r->va_triangles->finalize();
     }
   }
 };
 
 class drawMeshGRegion {
- public :
+ public:
   void operator () (GRegion *r)
   {  
     if(!r->getVisibility()) return;
 
-    MRep *m = r->meshRep;
-
-    if(!m) return;
-
     if(CTX.render_mode == GMSH_SELECT) {
       glPushName(3);
       glPushName(r->tag());
     }
 
-    if(CTX.mesh.volumes_edges){
-      if(m->va_lines && m->va_lines->getNumVertices()){
-	drawArrays(r, m->va_lines, GL_LINES, CTX.mesh.light && CTX.mesh.light_lines, 
-		   CTX.mesh.volumes_faces, CTX.color.mesh.line);
-      }
-      else{
-	drawArrays(r, m->va_triangles, GL_TRIANGLES, CTX.mesh.light && CTX.mesh.light_lines,
-		   CTX.mesh.volumes_faces, CTX.color.mesh.line, true);
-	drawArrays(r, m->va_quads, GL_QUADS, CTX.mesh.light && CTX.mesh.light_lines, 
-		   CTX.mesh.volumes_faces, CTX.color.mesh.line, true);
-      }
-    }
-    
-    if(CTX.mesh.volumes_faces){
-      drawArrays(r, m->va_triangles, GL_TRIANGLES, CTX.mesh.light);
-      drawArrays(r, m->va_quads, GL_QUADS, CTX.mesh.light);
-    }
+    drawArrays(r, r->va_lines, GL_LINES, CTX.mesh.light && CTX.mesh.light_lines, 
+               CTX.mesh.volumes_faces, CTX.color.mesh.line);
+    drawArrays(r, r->va_triangles, GL_TRIANGLES, CTX.mesh.light);
     
     if(CTX.mesh.volumes_num) {
       if(CTX.mesh.tetrahedra) 
-	drawElementLabels(r, r->tetrahedra, CTX.mesh.volumes_faces || 
-			  CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+        drawElementLabels(r, r->tetrahedra, CTX.mesh.volumes_faces || 
+                          CTX.mesh.surfaces_faces, CTX.color.mesh.line);
       if(CTX.mesh.hexahedra) 
-	drawElementLabels(r, r->hexahedra, CTX.mesh.volumes_faces || 
-			  CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+        drawElementLabels(r, r->hexahedra, CTX.mesh.volumes_faces || 
+                          CTX.mesh.surfaces_faces, CTX.color.mesh.line);
       if(CTX.mesh.prisms) 
-	drawElementLabels(r, r->prisms, CTX.mesh.volumes_faces || 
-			  CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+        drawElementLabels(r, r->prisms, CTX.mesh.volumes_faces || 
+                          CTX.mesh.surfaces_faces, CTX.color.mesh.line);
       if(CTX.mesh.pyramids) 
-	drawElementLabels(r, r->pyramids, CTX.mesh.volumes_faces ||
-			  CTX.mesh.surfaces_faces, CTX.color.mesh.line);
+        drawElementLabels(r, r->pyramids, CTX.mesh.volumes_faces ||
+                          CTX.mesh.surfaces_faces, CTX.color.mesh.line);
     }
 
     if(CTX.mesh.points || CTX.mesh.points_num){
-      if(m->allElementsVisible)
-	drawVerticesPerEntity(r);
+      if(r->getAllElementsVisible())
+        drawVerticesPerEntity(r);
       else{
-	if(CTX.mesh.tetrahedra) drawVerticesPerElement(r, r->tetrahedra);
-	if(CTX.mesh.hexahedra) drawVerticesPerElement(r, r->hexahedra);
-	if(CTX.mesh.prisms) drawVerticesPerElement(r, r->prisms);
-	if(CTX.mesh.pyramids) drawVerticesPerElement(r, r->pyramids);
+        if(CTX.mesh.tetrahedra) drawVerticesPerElement(r, r->tetrahedra);
+        if(CTX.mesh.hexahedra) drawVerticesPerElement(r, r->hexahedra);
+        if(CTX.mesh.prisms) drawVerticesPerElement(r, r->prisms);
+        if(CTX.mesh.pyramids) drawVerticesPerElement(r, r->pyramids);
       }
     }
 
@@ -889,6 +824,15 @@ class drawMeshGRegion {
 
 void Draw_Mesh()
 {
+  GModel *m = GModel::current();
+
+  // make sure to flag any model-dependent post-processing view as
+  // changed if the underlying mesh has, before resetting the changed
+  // flag
+  for(unsigned int i = 0; i < PView::list.size(); i++)
+    if(PView::list[i]->getData()->hasModel(m) && CTX.mesh.changed)
+      PView::list[i]->setChanged(true);
+  
   if(!CTX.mesh.draw) return;
   
   glPointSize(CTX.mesh.point_size);
@@ -908,33 +852,34 @@ void Draw_Mesh()
     else
       glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   
-  if(!CTX.threads_lock){
-    CTX.threads_lock = 1; 
-    int status = GMODEL->getMeshStatus();
+  static bool busy = false;
+  if(!busy){
+    busy = true;
+    int status = m->getMeshStatus();
     if(CTX.mesh.changed) {
-      Msg(DEBUG, "Mesh has changed: reinitializing drawing data", CTX.mesh.changed);
+      Msg::Debug("Mesh has changed: reinitializing drawing data", CTX.mesh.changed);
       if(status >= 1 && CTX.mesh.changed & ENT_LINE)
-	std::for_each(GMODEL->firstEdge(), GMODEL->lastEdge(), initMeshGEdge());
+        std::for_each(m->firstEdge(), m->lastEdge(), initMeshGEdge());
       if(status >= 2 && CTX.mesh.changed & ENT_SURFACE){
-	if(GMODEL->normals) delete GMODEL->normals;
-	GMODEL->normals = new smooth_normals(CTX.mesh.angle_smooth_normals);
-	if(CTX.mesh.smooth_normals)
-	  std::for_each(GMODEL->firstFace(), GMODEL->lastFace(), initSmoothNormalsGFace());
-	std::for_each(GMODEL->firstFace(), GMODEL->lastFace(), initMeshGFace());
+        if(m->normals) delete m->normals;
+        m->normals = new smooth_normals(CTX.mesh.angle_smooth_normals);
+        if(CTX.mesh.smooth_normals)
+          std::for_each(m->firstFace(), m->lastFace(), initSmoothNormalsGFace());
+        std::for_each(m->firstFace(), m->lastFace(), initMeshGFace());
       }
       if(status >= 3 && CTX.mesh.changed & ENT_VOLUME)
-	std::for_each(GMODEL->firstRegion(), GMODEL->lastRegion(), initMeshGRegion());
+        std::for_each(m->firstRegion(), m->lastRegion(), initMeshGRegion());
     }
     if(status >= 0)
-      std::for_each(GMODEL->firstVertex(), GMODEL->lastVertex(), drawMeshGVertex());
+      std::for_each(m->firstVertex(), m->lastVertex(), drawMeshGVertex());
     if(status >= 1)
-      std::for_each(GMODEL->firstEdge(), GMODEL->lastEdge(), drawMeshGEdge());
+      std::for_each(m->firstEdge(), m->lastEdge(), drawMeshGEdge());
     if(status >= 2)
-      std::for_each(GMODEL->firstFace(), GMODEL->lastFace(), drawMeshGFace());
+      std::for_each(m->firstFace(), m->lastFace(), drawMeshGFace());
     if(status >= 3)
-      std::for_each(GMODEL->firstRegion(), GMODEL->lastRegion(), drawMeshGRegion());
+      std::for_each(m->firstRegion(), m->lastRegion(), drawMeshGRegion());
     CTX.mesh.changed = 0;
-    CTX.threads_lock = 0;
+    busy = false;
   }
 
   for(int i = 0; i < 6; i++)
