@@ -1,7 +1,7 @@
 #ifndef _READ_IMG_
 #define _READ_IMG_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-int read_pnm(char *name);
-int read_jpeg(char *name);
-int read_png(char *name);
-int read_bmp(char *name);
+int read_pnm(const char *name);
+int read_jpeg(const char *name);
+int read_png(const char *name);
+int read_bmp(const char *name);
 
 #endif
