@@ -1,6 +1,6 @@
-// $Id: gl2png.cpp,v 1.7 2006-12-04 03:27:31 geuzaine Exp $
+// $Id: gl2png.cpp,v 1.10 2008-05-04 08:31:15 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 void create_png(FILE *file, PixelBuffer *buffer, int quality)
 {
-  Msg(GERROR, "This version of Gmsh was compiled without PNG support");
+  Msg::Error("This version of Gmsh was compiled without PNG support");
 }
 
 #else
@@ -40,14 +40,14 @@ void create_png(FILE *file, PixelBuffer *buffer, int quality)
 {
   if((buffer->GetFormat() != GL_RGB && buffer->GetFormat() != GL_RGBA) ||
      buffer->GetType() != GL_UNSIGNED_BYTE){
-    Msg(GERROR, "PNG only implemented for GL_RGB/GL_RGBA and GL_UNSIGNED_BYTE");
+    Msg::Error("PNG only implemented for GL_RGB/GL_RGBA and GL_UNSIGNED_BYTE");
     return;
   }
 
   png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   
   if(png_ptr == NULL) {
-    Msg(GERROR, "Could not create PNG write struct");
+    Msg::Error("Could not create PNG write struct");
     return;
   }
   
@@ -55,13 +55,13 @@ void create_png(FILE *file, PixelBuffer *buffer, int quality)
 
   if(info_ptr == NULL) {
     png_destroy_write_struct(&png_ptr, NULL);
-    Msg(GERROR, "Could not create PNG info struct");
+    Msg::Error("Could not create PNG info struct");
     return;
   }
   
   if(setjmp(png_jmpbuf(png_ptr))) {
     png_destroy_write_struct(&png_ptr, &info_ptr);
-    Msg(GERROR, "Could not setjmp in PNG");
+    Msg::Error("Could not setjmp in PNG");
     return;
   }
   
@@ -74,8 +74,8 @@ void create_png(FILE *file, PixelBuffer *buffer, int quality)
   // Z_DEFAULT_COMPRESSION, Z_BEST_SPEED, Z_BEST_COMPRESSION, Z_NO_COMPRESSION
   png_set_compression_level(png_ptr, Z_DEFAULT_COMPRESSION);
   png_set_IHDR(png_ptr, info_ptr, width, height, 8, 
-	       (numcomp == 3) ? PNG_COLOR_TYPE_RGB : PNG_COLOR_TYPE_RGBA,
-	       PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
+               (numcomp == 3) ? PNG_COLOR_TYPE_RGB : PNG_COLOR_TYPE_RGBA,
+               PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
   time_t now;
   time(&now);
   png_text text_ptr[10];  
