@@ -1,6 +1,6 @@
-// $Id: gl2ppm.cpp,v 1.15 2006-12-04 03:27:31 geuzaine Exp $
+// $Id: gl2ppm.cpp,v 1.17 2008-05-04 08:31:15 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 void create_ppm(FILE *outfile, PixelBuffer *buffer)
 {
   if(buffer->GetFormat() != GL_RGB || buffer->GetType() != GL_UNSIGNED_BYTE){
-    Msg(GERROR, "PPM only implemented for GL_RGB and GL_UNSIGNED_BYTE");
+    Msg::Error("PPM only implemented for GL_RGB and GL_UNSIGNED_BYTE");
     return;
   }
 
