@@ -1,6 +1,6 @@
-// $Id: gl2yuv.cpp,v 1.13 2006-12-04 03:27:31 geuzaine Exp $
+// $Id: gl2yuv.cpp,v 1.16 2008-06-07 17:20:47 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,13 +45,13 @@
  * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  */
 
-
+#include "MallocUtils.h"
 #include "gl2yuv.h"
 
 void create_yuv(FILE * outfile, PixelBuffer *buffer)
 {
   if(buffer->GetFormat() != GL_RGB || buffer->GetType() != GL_UNSIGNED_BYTE){
-    Msg(GERROR, "YUV only implemented for GL_RGB and GL_UNSIGNED_BYTE");
+    Msg::Error("YUV only implemented for GL_RGB and GL_UNSIGNED_BYTE");
     return;
   }
 
