@@ -1,7 +1,7 @@
 #ifndef _BACKGROUND_MESH_H_
 #define _BACKGROUND_MESH_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,10 +21,8 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 class GEntity;
-class Field;
 double BGM_MeshSize(GEntity *ge, double U, double V, double X, double Y, double Z);
-bool BGMExists();
-void BGMAddField(Field *field);
-void BGMReset();
+bool Extend1dMeshIn2dSurfaces();
+bool Extend2dMeshIn3dVolumes();
 
 #endif
