@@ -1,7 +1,7 @@
 #ifndef _GENERATOR_H_
 #define _GENERATOR_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,12 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+class GModel;
+
 void GetStatistics(double stat[50], double quality[3][100]=0);
-void GenerateMesh(int dimension);
-void OptimizeMesh();
+void AdaptMesh(GModel *m);
+void GenerateMesh(GModel *m, int dimension);
+void OptimizeMesh(GModel *m);
+void OptimizeMeshNetgen(GModel *m);
 
 #endif
