@@ -1,6 +1,6 @@
-// $Id: meshGRegion.cpp,v 1.31 2007-05-24 17:34:04 geuzaine Exp $
+// $Id: meshGRegion.cpp,v 1.54 2008-06-20 12:15:44 remacle Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <stdlib.h>
+#include <vector>
 #include "meshGRegion.h"
 #include "meshGRegionDelaunayInsertion.h"
 #include "GModel.h"
@@ -26,10 +28,11 @@
 #include "GFace.h"
 #include "GEdge.h"
 #include "gmshRegion.h"
-#include "MRep.h"
 #include "BDS.h"
 #include "Message.h"
-#include <vector>
+#include "Context.h"
+
+extern Context_T CTX;
 
 void getAllBoundingVertices(GRegion *gr, std::set<MVertex*> &allBoundingVertices)
 {
@@ -41,8 +44,8 @@ void getAllBoundingVertices(GRegion *gr, std::set<MVertex*> &allBoundingVertices
     for(unsigned int i = 0; i < gf->triangles.size(); i++){
       MTriangle *t = gf->triangles[i];
       for(int k = 0; k < 3; k++)
-	if(allBoundingVertices.find(t->getVertex(k)) ==  allBoundingVertices.end())
-	  allBoundingVertices.insert(t->getVertex(k));
+        if(allBoundingVertices.find(t->getVertex(k)) ==  allBoundingVertices.end())
+          allBoundingVertices.insert(t->getVertex(k));
     }
     ++it;
   }
@@ -94,39 +97,43 @@ void buildTetgenStructure(GRegion *gr, tetgenio &in, std::vector<MVertex*> &numb
     for(unsigned int i = 0; i < gf->triangles.size(); i++){
       MTriangle *t = gf->triangles[i];
       tetgenio::facet *f = &in.facetlist[I];
-      tetgenio::init(f);    
+      tetgenio::init(f);
       f->numberofholes = 0;
       f->numberofpolygons = 1;
       f->polygonlist = new tetgenio::polygon[f->numberofpolygons];
       tetgenio::polygon *p = &f->polygonlist[0];
-      tetgenio::init(p);    
+      tetgenio::init(p);
       p->numberofvertices = 3;
       p->vertexlist = new int[p->numberofvertices];
       p->vertexlist[0] = t->getVertex(0)->getNum();
       p->vertexlist[1] = t->getVertex(1)->getNum();
       p->vertexlist[2] = t->getVertex(2)->getNum();
-      in.facetmarkerlist[I] = gf->tag();	  
+      in.facetmarkerlist[I] = gf->tag();
       ++I;
     }
     ++it;
   }   
 }
 
-void TransferTetgenMesh(GRegion *gr, 
-			tetgenio &in, 
-			tetgenio &out, 
-			std::vector<MVertex*> &numberedV)
+void TransferTetgenMesh(GRegion *gr, tetgenio &in, tetgenio &out, 
+                        std::vector<MVertex*> &numberedV)
 {
+  // Improvement has to be done here :
+  // tetgen splits some of the existing edges of the 
+  // mesh. If those edges are classified on some
+  // model faces, new points SHOULD be classified
+  // on the model face and get the right set of parametric coordinates.
+
   for(int i = numberedV.size(); i < out.numberofpoints; i++){
     MVertex *v = new MVertex(out.pointlist[i * 3 + 0],
-			     out.pointlist[i * 3 + 1],
-			     out.pointlist[i * 3 + 2], gr); 
+                             out.pointlist[i * 3 + 1],
+                             out.pointlist[i * 3 + 2], gr); 
     gr->mesh_vertices.push_back(v);
     numberedV.push_back(v);
   }
  
-  Msg(INFO,"%d points %d edges and %d faces in the final mesh",
-      out.numberofpoints, out.numberofedges, out.numberoftrifaces);
+  Msg::Info("%d points %d edges and %d faces in the final mesh",
+	    out.numberofpoints, out.numberofedges, out.numberoftrifaces);
 
   // Tetgen modifies both surface & edge mesh, so we need to re-create
   // everything
@@ -137,39 +144,70 @@ void TransferTetgenMesh(GRegion *gr,
     for(unsigned int i = 0; i < gf->triangles.size(); i++)
       delete gf->triangles[i];
     gf->triangles.clear();
-    if(gf->meshRep) gf->meshRep->destroy();
+    gf->deleteVertexArrays();
     ++it;
-  }    
+  }
   
-  // TODO: re-create 1D mesh 
+  // TODO: re-create 1D mesh
   for(int i = 0; i < out.numberofedges; i++){
     MVertex *v[2];
-    v[0] = numberedV[out.edgelist[i * 2 + 0] -1];
-    v[1] = numberedV[out.edgelist[i * 2 + 1] -1];
+    v[0] = numberedV[out.edgelist[i * 2 + 0] - 1];
+    v[1] = numberedV[out.edgelist[i * 2 + 1] - 1];
   }
 
-  // re-create the triangular meshes
+  // re-create the triangular meshes FIXME: this can lead to hanging
+  // nodes for non manifold geometries (single surface connected to
+  // volume)
   for (int i = 0; i < out.numberoftrifaces; i++){
     MVertex *v[3];
     v[0] = numberedV[out.trifacelist[i * 3 + 0] - 1];
     v[1] = numberedV[out.trifacelist[i * 3 + 1] - 1];
     v[2] = numberedV[out.trifacelist[i * 3 + 2] - 1];
-    GFace *gf = gr->model()->faceByTag(out.trifacemarkerlist[i]);
-    for(int j = 0; j < 3; j++){	  
+    GFace *gf = gr->model()->getFaceByTag(out.trifacemarkerlist[i]);
+
+    double guess[2] = {0,0};
+    int Count = 0;
+    for(int j = 0; j < 3; j++){   
+      if(v[j]->onWhat()->dim() == 2){
+	v[j]->getParameter(0,guess[0]);
+	v[j]->getParameter(1,guess[1]);
+	Count++;
+      }
+    }
+    guess[0]/=Count;
+    guess[1]/=Count;
+    for(int j = 0; j < 3; j++){   
       if(v[j]->onWhat()->dim() == 3){
-	v[j]->onWhat()->mesh_vertices.erase
-	  (std::find(v[j]->onWhat()->mesh_vertices.begin(),
-		     v[j]->onWhat()->mesh_vertices.end(),
-		     v[j]));
-	MVertex *v1b = new MVertex(v[j]->x(), v[j]->y(), v[j]->z(), gf);
-	gf->mesh_vertices.push_back(v1b);
-	numberedV[out.trifacelist[i * 3 + j] - 1] = v1b;
-	delete v[j];
-	v[j] = v1b;
+        v[j]->onWhat()->mesh_vertices.erase
+          (std::find(v[j]->onWhat()->mesh_vertices.begin(),
+                     v[j]->onWhat()->mesh_vertices.end(),
+                     v[j]));
+        MVertex *v1b;
+	if (CTX.mesh.order > 1 && CTX.mesh.second_order_experimental){
+	  // PARAMETRIC COORDINATES SHOULD BE SET for the vertex !!!!!!!!!!!!!
+	  // This is not 100 % safe yet, so we reserve that operation for high order
+	  // meshes.
+	  GPoint gp = gf->closestPoint (SPoint3(v[j]->x(), v[j]->y(), v[j]->z()),guess);
+
+	  Msg::Debug("A new point has been inserted in mesh face %d by the 3D mesher",gf->tag());
+	  Msg::Debug("The point has been projected back to the surface (%g %g %g) -> (%g %g %g)",
+		     v[j]->x(), v[j]->y(), v[j]->z(),gp.x(),gp.y(),gp.z());
+
+	  // To be safe, we should ensure that this mesh motion does not lead to an invalid mesh !!!!
+	  v1b = new MFaceVertex(gp.x(),gp.y(),gp.z(),gf,gp.u(),gp.v());
+	}
+	else{
+	  v1b = new MVertex(v[j]->x(), v[j]->y(), v[j]->z(),gf);
+	}
+
+        gf->mesh_vertices.push_back(v1b);
+        numberedV[out.trifacelist[i * 3 + j] - 1] = v1b;
+        delete v[j];
+        v[j] = v1b;
       }
     }
     MTriangle *t = new  MTriangle(v[0], v[1], v[2]);
-    gf->triangles.push_back(t);	  
+    gf->triangles.push_back(t);   
   }
   for(int i = 0; i < out.numberoftetrahedra; i++){
     MVertex *v1 = numberedV[out.tetrahedronlist[i * 4 + 0] - 1];
@@ -188,11 +226,11 @@ void MeshDelaunayVolume(std::vector<GRegion*> &regions)
   if(regions.empty()) return;
 
 #if !defined(HAVE_TETGEN)
-  Msg(GERROR, "Tetgen is not compiled in this version of Gmsh");
+  Msg::Error("Tetgen is not compiled in this version of Gmsh");
 #else
 
   for(unsigned int i = 0; i < regions.size(); i++)
-    Msg(STATUS2, "Meshing volume %d (Delaunay)", regions[i]->tag());
+    Msg::StatusBar(2, true, "Meshing volume %d (Tetgen+Delaunay)", regions[i]->tag());
 
   // put all the faces in the same model
   GRegion *gr = regions[0];
@@ -208,14 +246,41 @@ void MeshDelaunayVolume(std::vector<GRegion*> &regions)
     allFaces.push_back(*it);
   gr->set(allFaces);
 
-  // mesh with tetgen, possibly changing the mesh on boundaries
-  tetgenio in, out;
-  std::vector<MVertex*> numberedV;
-  char opts[128];
-  buildTetgenStructure(gr, in, numberedV);
-  sprintf(opts, "pe%c", (CTX.verbosity < 3) ? 'Q': (CTX.verbosity > 6)? 'V': '\0');
-  tetrahedralize(opts, &in, &out);
-  TransferTetgenMesh(gr, in, out, numberedV);
+  // mesh with tetgen, possibly changing the mesh on boundaries (leave
+  // this in block, so in/out are destroyed before we refine the mesh)
+  {
+    tetgenio in, out;
+    std::vector<MVertex*> numberedV;
+    char opts[128];
+    buildTetgenStructure(gr, in, numberedV);
+    sprintf(opts, "pe%c", 
+	    (Msg::GetVerbosity() < 3) ? 'Q': (Msg::GetVerbosity() > 6)? 'V': '\0');
+    try{
+      tetrahedralize(opts, &in, &out);
+    }
+    catch (int error){
+      Msg::Error("Self intersecting surface mesh, computing intersections "
+		 "(this could take a while)");
+      sprintf(opts, "dV");
+      try{
+        tetrahedralize(opts, &in, &out);
+        Msg::Info("%d faces self-intersect", out.numberoftrifaces);
+        for (int i = 0; i < out.numberoftrifaces; i++){
+          Msg::Info("face (%d %d %d) on model face %d",
+              numberedV[out.trifacelist[i * 3 + 0] - 1]->getNum(),
+              numberedV[out.trifacelist[i * 3 + 1] - 1]->getNum(),
+              numberedV[out.trifacelist[i * 3 + 2] - 1]->getNum(),
+              out.trifacemarkerlist[i]);
+        }
+      }
+      catch (int error2){
+        Msg::Error("Surface mesh is wrong, cannot do the 3D mesh");
+      }
+      gr->set(faces);
+      return;
+    }
+    TransferTetgenMesh(gr, in, out, numberedV);
+  }
 
   // sort triangles in all model faces in order to be able to search in vectors
   std::list<GFace*>::iterator itf =  allFaces.begin();
@@ -229,7 +294,8 @@ void MeshDelaunayVolume(std::vector<GRegion*> &regions)
   gr->set(faces);
 
   // now do insertion of points
-  insertVerticesInRegion(gr); 
+  insertVerticesInRegion(gr);
+  Msg::Info("Gmsh 3D Delaunay has generated %d tets", gr->tetrahedra.size());
 #endif
 }
 
@@ -242,7 +308,7 @@ namespace nglib {
 using namespace nglib;
 
 Ng_Mesh *buildNetgenStructure(GRegion *gr, bool importVolumeMesh,
-			      std::vector<MVertex*> &numberedV)
+                              std::vector<MVertex*> &numberedV)
 {
   NgAddOn_Init();
   Ng_Mesh *ngmesh = Ng_NewMesh();
@@ -277,13 +343,13 @@ Ng_Mesh *buildNetgenStructure(GRegion *gr, bool importVolumeMesh,
   std::list<GFace*> faces = gr->faces();
   std::list<GFace*>::iterator it = faces.begin();
   while(it != faces.end()){
-    GFace *gf = (*it);      
+    GFace *gf = (*it);
     for(unsigned int i = 0; i< gf->triangles.size(); i++){
       MTriangle *t = gf->triangles[i];
       int tmp[3];
       tmp[0] = t->getVertex(0)->getNum();
       tmp[1] = t->getVertex(1)->getNum();
-      tmp[2] = t->getVertex(2)->getNum();	  
+      tmp[2] = t->getVertex(2)->getNum();
       Ng_AddSurfaceElement(ngmesh, NG_TRIG, tmp);
     }
     ++it;
@@ -297,8 +363,8 @@ Ng_Mesh *buildNetgenStructure(GRegion *gr, bool importVolumeMesh,
       int tmp[4];
       tmp[0] = t->getVertex(0)->getNum();
       tmp[1] = t->getVertex(1)->getNum();
-      tmp[2] = t->getVertex(2)->getNum();	  
-      tmp[3] = t->getVertex(3)->getNum();	  
+      tmp[2] = t->getVertex(2)->getNum();
+      tmp[3] = t->getVertex(3)->getNum();
       Ng_AddVolumeElement(ngmesh, NG_TET, tmp);
     }
   }
@@ -307,7 +373,7 @@ Ng_Mesh *buildNetgenStructure(GRegion *gr, bool importVolumeMesh,
 }
 
 void TransferVolumeMesh(GRegion *gr, Ng_Mesh *ngmesh, 
-			std::vector<MVertex*> &numberedV)
+                        std::vector<MVertex*> &numberedV)
 {
   // Gets total number of vertices of Netgen's mesh
   int nbv = Ng_GetNP(ngmesh);  
@@ -332,9 +398,9 @@ void TransferVolumeMesh(GRegion *gr, Ng_Mesh *ngmesh,
     int tmp[4];
     Ng_GetVolumeElement(ngmesh, i + 1, tmp);
     MTetrahedron *t = new MTetrahedron(numberedV[tmp[0] - 1],
-				       numberedV[tmp[1] - 1],
-				       numberedV[tmp[2] - 1],
-				       numberedV[tmp[3] - 1]);
+                                       numberedV[tmp[1] - 1],
+                                       numberedV[tmp[2] - 1],
+                                       numberedV[tmp[3] - 1]);
     gr->tetrahedra.push_back(t);
   }  
 }
@@ -361,11 +427,11 @@ void deMeshGRegion::operator() (GRegion *gr)
   for(unsigned int i = 0; i < gr->pyramids.size(); i++)
     delete gr->pyramids[i];
   gr->pyramids.clear();
-  if(gr->meshRep) gr->meshRep->destroy();
+  gr->deleteVertexArrays();
 }
 
 int intersect_line_triangle(double X[3], double Y[3], double Z[3] , 
-			    double P[3], double N[3] )
+                            double P[3], double N[3] )
 {
   double mat[3][3], det;
   double b[3], res[3];
@@ -387,11 +453,9 @@ int intersect_line_triangle(double X[3], double Y[3], double Z[3] ,
   b[1] = P[1] - Y[0];
   b[2] = P[2] - Z[0];
 
-  if(!sys3x3_with_tol(mat, b, res, &det))       
+  if(!sys3x3_with_tol(mat, b, res, &det))
     return 0;
 
-  //  Msg(INFO,"going there %g %g %g",res[0],res[1],res[2]);
-
   if(res[0] >= eps_prec && res[0] <= 1.0 - eps_prec && 
      res[1] >= eps_prec && res[1] <= 1.0 - eps_prec && 
      1 - res[0] - res[1] >= eps_prec && 1 - res[0] - res[1] <= 1.0 - eps_prec){
@@ -399,8 +463,8 @@ int intersect_line_triangle(double X[3], double Y[3], double Z[3] ,
     return (res[2] > 0) ? 1 : 0;
   }
   else if(res[0] < -eps_prec || res[0] > 1.0 + eps_prec || 
-	  res[1] < -eps_prec || res[1] > 1.0 + eps_prec || 
-	  1 - res[0] - res[1] < -eps_prec || 1 - res[0] - res[1] > 1.0 + eps_prec){
+          res[1] < -eps_prec || res[1] > 1.0 + eps_prec || 
+          1 - res[0] - res[1] < -eps_prec || 1 - res[0] - res[1] > 1.0 + eps_prec){
     // the line clearly does NOT intersect the triangle
     return 0;
   }
@@ -423,9 +487,9 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
 
   double rrr[6];
   setRand(rrr);
-		   
+                   
   while(it != faces.end()){
-    GFace *gf = (*it);      
+    GFace *gf = (*it); 
     int nb_intersect = 0;
     for(unsigned int i = 0; i < gf->triangles.size(); i++){
       MTriangle *t = gf->triangles[i];
@@ -435,7 +499,7 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
       double P[3] = {(X[0]+X[1]+X[2])/3., (Y[0]+Y[1]+Y[2])/3., (Z[0]+Z[1]+Z[2])/3.};
       double v1[3] = {X[0]-X[1], Y[0]-Y[1], Z[0]-Z[1]};
       double v2[3] = {X[2]-X[1], Y[2]-Y[1], Z[2]-Z[1]};
-      double N[3] ;
+      double N[3];
       prodve(v1, v2, N);
       norme(v1);
       norme(v2);
@@ -446,23 +510,23 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
       norme(N);
       std::list<GFace*>::iterator it_b = faces.begin();
       while(it_b != faces.end()){
-	GFace *gf_b = (*it_b);
-	for(unsigned int i_b = 0; i_b < gf_b->triangles.size(); i_b++){
-	  MTriangle *t_b = gf_b->triangles[i_b];
-	  if(t_b != t){
-	    double X_b[3] = {t_b->getVertex(0)->x(), t_b->getVertex(1)->x(), 
-			     t_b->getVertex(2)->x()};
-	    double Y_b[3] = {t_b->getVertex(0)->y(), t_b->getVertex(1)->y(),
-			     t_b->getVertex(2)->y()};
-	    double Z_b[3] = {t_b->getVertex(0)->z(), t_b->getVertex(1)->z(),
-			     t_b->getVertex(2)->z()};
-	    int inters = intersect_line_triangle(X_b, Y_b, Z_b, P, N);
-	    nb_intersect += inters;
-	  }
-	}
-	++it_b;
+        GFace *gf_b = (*it_b);
+        for(unsigned int i_b = 0; i_b < gf_b->triangles.size(); i_b++){
+          MTriangle *t_b = gf_b->triangles[i_b];
+          if(t_b != t){
+            double X_b[3] = {t_b->getVertex(0)->x(), t_b->getVertex(1)->x(), 
+                             t_b->getVertex(2)->x()};
+            double Y_b[3] = {t_b->getVertex(0)->y(), t_b->getVertex(1)->y(),
+                             t_b->getVertex(2)->y()};
+            double Z_b[3] = {t_b->getVertex(0)->z(), t_b->getVertex(1)->z(),
+                             t_b->getVertex(2)->z()};
+            int inters = intersect_line_triangle(X_b, Y_b, Z_b, P, N);
+            nb_intersect += inters;
+          }
+        }
+        ++it_b;
       }
-      Msg(INFO,"Region %d Face %d, %d intersect", gr->tag(), gf->tag(), nb_intersect);
+      Msg::Info("Region %d Face %d, %d intersect", gr->tag(), gf->tag(), nb_intersect);
       if(nb_intersect >= 0) break; // negative value means intersection is not "robust"
     }
     
@@ -471,10 +535,10 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
     }
     else{
       if(nb_intersect % 2 == 1){ 
-	// odd nb of intersections: the normal points inside the region 
-	for (unsigned int i = 0; i < gf->triangles.size(); i++){
-	  gf->triangles[i]->revert();
-	}
+        // odd nb of intersections: the normal points inside the region 
+        for (unsigned int i = 0; i < gf->triangles.size(); i++){
+          gf->triangles[i]->revert();
+        }
       }
       ++it;
     }
@@ -511,7 +575,7 @@ void meshGRegion::operator() (GRegion *gr)
   // sanity check
   for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); it++){
     if((*it)->quadrangles.size()){
-      Msg(GERROR, "Cannot tetrahedralize volume with quadrangles on boundary");
+      Msg::Error("Cannot tetrahedralize volume with quadrangles on boundary");
       return;
     }
   }
@@ -521,18 +585,14 @@ void meshGRegion::operator() (GRegion *gr)
   }
   else if(CTX.mesh.algo3d == ALGO_3D_NETGEN ){
 #if !defined(HAVE_NETGEN)
-    Msg(GERROR, "Netgen is not compiled in this version of Gmsh");
+    Msg::Error("Netgen is not compiled in this version of Gmsh");
 #else
-    Msg(STATUS2, "Meshing volume %d (Netgen)", gr->tag());
+    Msg::StatusBar(2, true, "Meshing volume %d (Netgen)", gr->tag());
     // orient the triangles of with respect to this region
     meshNormalsPointOutOfTheRegion(gr);
     std::vector<MVertex*> numberedV;
     Ng_Mesh *ngmesh = buildNetgenStructure(gr, false, numberedV);
-    Ng_Meshing_Parameters mp;
-    mp.maxh = 1;
-    mp.fineness = 1;
-    mp.secondorder = 0;
-    NgAddOn_GenerateVolumeMesh(ngmesh, &mp); // does not optimize
+    NgAddOn_GenerateVolumeMesh(ngmesh, CTX.lc); // does not optimize
     TransferVolumeMesh(gr, ngmesh, numberedV);
     Ng_DeleteMesh(ngmesh);
     Ng_Exit();
@@ -540,18 +600,19 @@ void meshGRegion::operator() (GRegion *gr)
   }
 }
 
-void optimizeMeshGRegion::operator() (GRegion *gr) 
+void optimizeMeshGRegionNetgen::operator() (GRegion *gr) 
 {  
   if(gr->geomType() == GEntity::DiscreteVolume) return;
   
-  // don't optimize extruded meshes
+  // don't optimize transfinite or extruded meshes
+  if(gr->meshAttributes.Method == TRANSFINI) return;
   ExtrudeParams *ep = gr->meshAttributes.extrude;
   if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == EXTRUDED_ENTITY) return;
-  
+
 #if !defined(HAVE_NETGEN)
-  Msg(GERROR, "Netgen is not compiled in this version of Gmsh");
+  Msg::Error("Netgen is not compiled in this version of Gmsh");
 #else
-  Msg(STATUS2, "Optimizing volume %d", gr->tag());
+  Msg::StatusBar(2, true, "Optimizing volume %d", gr->tag());
   // import mesh into netgen, including volume tets
   std::vector<MVertex*> numberedV;
   Ng_Mesh *ngmesh = buildNetgenStructure(gr, true, numberedV);
@@ -559,13 +620,93 @@ void optimizeMeshGRegion::operator() (GRegion *gr)
   deMeshGRegion dem;
   dem(gr);
   // optimize mesh
-  Ng_Meshing_Parameters mp;
-  mp.maxh = 1;
-  mp.fineness = 1;
-  mp.secondorder = 0;
-  NgAddOn_OptimizeVolumeMesh(ngmesh, &mp);
+  NgAddOn_OptimizeVolumeMesh(ngmesh, CTX.lc);
   TransferVolumeMesh(gr, ngmesh, numberedV);
   Ng_DeleteMesh(ngmesh);
   Ng_Exit();
 #endif
 }
+
+void optimizeMeshGRegionGmsh::operator() (GRegion *gr) 
+{  
+  if(gr->geomType() == GEntity::DiscreteVolume) return;
+  
+  // don't optimize extruded meshes
+  if(gr->meshAttributes.Method == TRANSFINI) return;
+  ExtrudeParams *ep = gr->meshAttributes.extrude;
+  if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == EXTRUDED_ENTITY) return;
+  
+  Msg::StatusBar(2, true, "Optimizing volume %d", gr->tag());
+  gmshOptimizeMesh(gr, QMTET_2);  
+}
+
+bool buildFaceSearchStructure(GModel *model, fs_cont &search)
+{  
+  search.clear();
+
+  GModel::fiter fit = model->firstFace();
+  while(fit != model->lastFace()){    
+    for(unsigned int i = 0; i < (*fit)->triangles.size(); i++){
+      MVertex *p1 = (*fit)->triangles[i]->getVertex(0);
+      MVertex *p2 = (*fit)->triangles[i]->getVertex(1);
+      MVertex *p3 = (*fit)->triangles[i]->getVertex(2);
+      MVertex *p = std::min(p1, std::min(p2, p3));
+      search.insert(std::pair<MVertex*, std::pair<MTriangle*, GFace*> >
+                    (p, std::pair<MTriangle*, GFace*>((*fit)->triangles[i], *fit)));
+    }
+    ++fit;
+  }
+  return true;
+}
+
+bool buildEdgeSearchStructure(GModel *model, es_cont &search)
+{  
+  search.clear();
+
+  GModel::eiter eit = model->firstEdge();
+  while(eit != model->lastEdge()){    
+    for(unsigned int i = 0; i < (*eit)->lines.size(); i++){
+      MVertex *p1 = (*eit)->lines[i]->getVertex(0);
+      MVertex *p2 = (*eit)->lines[i]->getVertex(1);
+      MVertex *p = std::min(p1, p2);
+      search.insert(std::pair<MVertex*, std::pair<MLine*, GEdge*> > 
+                    (p, std::pair<MLine*, GEdge*>((*eit)->lines[i], *eit)));
+    }
+    ++eit;
+  }
+  return true;
+}
+
+GFace *findInFaceSearchStructure(MVertex *p1, MVertex *p2, MVertex *p3, 
+                                 const fs_cont &search)
+{
+  MVertex *p = std::min(p1, std::min(p2, p3));
+  
+  for(fs_cont::const_iterator it = search.lower_bound(p);
+      it != search.upper_bound(p);
+      ++it){
+    MTriangle *t = it->second.first;
+    GFace *gf= it->second.second;
+    if((t->getVertex(0) == p1 || t->getVertex(0) == p2 || t->getVertex(0) == p3) &&
+       (t->getVertex(1) == p1 || t->getVertex(1) == p2 || t->getVertex(1) == p3) &&
+       (t->getVertex(2) == p1 || t->getVertex(2) == p2 || t->getVertex(2) == p3)) 
+      return gf;
+  }
+  return 0;
+}
+
+GEdge *findInEdgeSearchStructure(MVertex *p1, MVertex *p2, const es_cont &search)
+{
+  MVertex *p = std::min(p1, p2);
+  
+  for(es_cont::const_iterator it = search.lower_bound(p);
+      it != search.upper_bound(p);
+      ++it){
+    MLine *l = it->second.first;
+    GEdge *ge = it->second.second;
+    if ((l->getVertex(0) == p1 || l->getVertex(0) == p2) &&
+        (l->getVertex(1) == p1 || l->getVertex(1) == p2)) 
+      return ge;
+  }
+  return 0;
+}
