@@ -1,6 +1,6 @@
-// $Id: meshGRegionExtruded.cpp,v 1.17 2007-05-10 22:08:06 geuzaine Exp $
+// $Id: meshGRegionExtruded.cpp,v 1.26 2008-05-06 21:11:48 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,9 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include <set>
-#include "ExtrudeParams.h"
 #include "GModel.h"
+#include "MElement.h"
+#include "ExtrudeParams.h"
 #include "meshGFace.h"
 #include "meshGRegion.h"
 #include "Context.h"
@@ -55,7 +56,7 @@ void createPriPyrTet(std::vector<MVertex*> &v, GRegion *to)
   }
   else {
     to->prisms.push_back(new MPrism(v));
-    if(j) Msg(GERROR, "Degenerated prism in extrusion of volume %d", to->tag());
+    if(j) Msg::Error("Degenerated prism in extrusion of volume %d", to->tag());
   }
 }
 
@@ -77,11 +78,11 @@ void createHexPri(std::vector<MVertex*> &v, GRegion *to)
     else if(dup[0] == 0 && dup[1] == 3)
       to->prisms.push_back(new MPrism(v[0], v[1], v[5], v[3], v[2], v[6]));
     else
-      Msg(GERROR, "Uncoherent hexahedron in extrusion of volume %d", to->tag());
+      Msg::Error("Uncoherent hexahedron in extrusion of volume %d", to->tag());
   }
   else {
     to->hexahedra.push_back(new MHexahedron(v));
-    if(j) Msg(GERROR, "Degenerated hexahedron in extrusion of volume %d", to->tag());
+    if(j) Msg::Error("Degenerated hexahedron in extrusion of volume %d", to->tag());
   }
 }
 
@@ -92,8 +93,8 @@ void createTet(MVertex *v1, MVertex *v2, MVertex *v3, MVertex *v4, GRegion *to)
 }
 
 int getExtrudedVertices(MElement *ele, ExtrudeParams *ep, int j, int k, 
-			std::set<MVertex*, MVertexLessThanLexicographic> &pos,
-			std::vector<MVertex*> &verts)
+                        std::set<MVertex*, MVertexLessThanLexicographic> &pos,
+                        std::vector<MVertex*> &verts)
 {
   std::set<MVertex*, MVertexLessThanLexicographic>::iterator itp;
   double x[8], y[8], z[8];
@@ -111,9 +112,13 @@ int getExtrudedVertices(MElement *ele, ExtrudeParams *ep, int j, int k,
   for(int p = 0; p < 2 * n; p++){
     MVertex tmp(x[p], y[p], z[p], 0, -1);
     itp = pos.find(&tmp);
+    if(itp == pos.end()){ // FIXME: workaround
+      Msg::Info("Linear search for (%.16g, %.16g, %.16g)", tmp.x(), tmp.y(), tmp.z());
+      itp = tmp.linearSearch(pos);
+    }
     if(itp == pos.end())
-      Msg(GERROR, "Could not find extruded vertex (%.16g, %.16g, %.16g)",
-	  tmp.x(), tmp.y(), tmp.z());
+      Msg::Error("Could not find extruded vertex (%.16g, %.16g, %.16g)",
+          tmp.x(), tmp.y(), tmp.z());
     else
       verts.push_back(*itp);
   }
@@ -121,7 +126,7 @@ int getExtrudedVertices(MElement *ele, ExtrudeParams *ep, int j, int k,
 }
 
 void extrudeMesh(GFace *from, GRegion *to,
-		 std::set<MVertex*, MVertexLessThanLexicographic> &pos)
+                 std::set<MVertex*, MVertexLessThanLexicographic> &pos)
 {
   ExtrudeParams *ep = to->meshAttributes.extrude;
 
@@ -130,13 +135,13 @@ void extrudeMesh(GFace *from, GRegion *to,
     MVertex *v = from->mesh_vertices[i];
     for(int j = 0; j < ep->mesh.NbLayer; j++) {
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
-	double x = v->x(), y = v->y(), z = v->z();
-	ep->Extrude(j, k + 1, x, y, z);
-	if(j != ep->mesh.NbLayer - 1 || k != ep->mesh.NbElmLayer[j] - 1){
-	  MVertex *newv = new MVertex(x, y, z, to);
-	  to->mesh_vertices.push_back(newv);
-	  pos.insert(newv);
-	}
+        double x = v->x(), y = v->y(), z = v->z();
+        ep->Extrude(j, k + 1, x, y, z);
+        if(j != ep->mesh.NbLayer - 1 || k != ep->mesh.NbElmLayer[j] - 1){
+          MVertex *newv = new MVertex(x, y, z, to);
+          to->mesh_vertices.push_back(newv);
+          pos.insert(newv);
+        }
       }
     }
   }
@@ -147,25 +152,25 @@ void extrudeMesh(GFace *from, GRegion *to,
   for(unsigned int i = 0; i < from->triangles.size(); i++){
     for(int j = 0; j < ep->mesh.NbLayer; j++) {
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
-	std::vector<MVertex*> verts;
-	if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, verts) == 6)
-	  createPriPyrTet(verts, to);
+        std::vector<MVertex*> verts;
+        if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, verts) == 6)
+          createPriPyrTet(verts, to);
       }
     }
   }
   for(unsigned int i = 0; i < from->quadrangles.size(); i++){
     for(int j = 0; j < ep->mesh.NbLayer; j++) {
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
-	std::vector<MVertex*> verts;
-	if(getExtrudedVertices(from->quadrangles[i], ep, j, k, pos, verts) == 8)
-	  createHexPri(verts, to);
+        std::vector<MVertex*> verts;
+        if(getExtrudedVertices(from->quadrangles[i], ep, j, k, pos, verts) == 8)
+          createHexPri(verts, to);
       }
     }
   }
 }
 
 void insertAllVertices(GRegion *gr, 
-		       std::set<MVertex*, MVertexLessThanLexicographic> &pos)
+                       std::set<MVertex*, MVertexLessThanLexicographic> &pos)
 {
   pos.insert(gr->mesh_vertices.begin(), gr->mesh_vertices.end());
   std::list<GFace*> faces = gr->faces();
@@ -177,140 +182,15 @@ void insertAllVertices(GRegion *gr,
     while(ite != edges.end()){
       pos.insert((*ite)->mesh_vertices.begin(), (*ite)->mesh_vertices.end());
       pos.insert((*ite)->getBeginVertex()->mesh_vertices.begin(),
-		 (*ite)->getBeginVertex()->mesh_vertices.end());
+                 (*ite)->getBeginVertex()->mesh_vertices.end());
       pos.insert((*ite)->getEndVertex()->mesh_vertices.begin(),
-		 (*ite)->getEndVertex()->mesh_vertices.end());
+                 (*ite)->getEndVertex()->mesh_vertices.end());
       ++ite;
     }
     ++itf;
   }
 }
 
-#if defined(HAVE_ANN_)
-
-#include "ANN/ANN.h"
-
-template <class T>
-void carveHole(std::vector<T*> &elements, double distance, ANNkd_tree *kdtree)
-{
-  // delete all elements that have at least one vertex closer than
-  // 'distance' from the carving surface vertices
-  ANNidxArray index = new ANNidx[1];
-  ANNdistArray dist = new ANNdist[1];
-  std::vector<T*> temp;
-  for(unsigned int i = 0; i < elements.size(); i++){
-    for(int j = 0; j < elements[i]->getNumVertices(); j++){
-      MVertex *v = elements[i]->getVertex(j);
-      double xyz[3] = {v->x(), v->y(), v->z()};
-      kdtree->annkSearch(xyz, 1, index, dist);
-      double d = sqrt(dist[0]);
-      if(d < distance){
-	delete elements[i];
-	break;
-      }
-      else if(j == elements[i]->getNumVertices() - 1){
-        temp.push_back(elements[i]);
-      }
-    }
-  }
-  elements = temp;
-  delete [] index;
-  delete [] dist;
-}
-
-#endif
-
-template <class T>
-void addFaces(std::vector<T*> &elements, std::set<MFace, Less_Face> &faces)
-{
-  for(unsigned int i = 0; i < elements.size(); i++){
-    for(int j = 0; j < elements[i]->getNumFaces(); j++){
-      MFace f = elements[i]->getFace(j);
-      std::set<MFace, Less_Face>::iterator it = faces.find(f);
-      if(it == faces.end())
-	faces.insert(f);
-      else
-	faces.erase(it);
-    }
-  }
-}
-
-void carveHole(GRegion *gr, int num, double distance, std::vector<int> &surfaces)
-{
-#if !defined(HAVE_ANN_)
-  Msg(GERROR, "Gmsh must be compiled with ANN support to carve holes in extruded meshes");
-#else
-  Msg(INFO, "Carving hole %d from surface %d at distance %g", num, surfaces[0], distance);
-  GModel *m = gr->model();
-
-  // add all points from carving surfaces into kdtree
-  int numnodes = 0;
-  for(unsigned int i = 0; i < surfaces.size(); i++){
-    GFace *gf = m->faceByTag(surfaces[i]);
-    if(!gf){
-      Msg(GERROR, "Unknown carving surface %d", surfaces[i]);
-      return;
-    }
-    numnodes += gf->mesh_vertices.size();
-  }
-
-  ANNpointArray kdnodes = annAllocPts(numnodes, 4);
-  int k = 0;
-  for(unsigned int i = 0; i < surfaces.size(); i++){
-    GFace *gf = m->faceByTag(surfaces[i]);
-    for(unsigned int j = 0; j < gf->mesh_vertices.size(); j++){
-      kdnodes[k][0] = gf->mesh_vertices[j]->x();
-      kdnodes[k][1] = gf->mesh_vertices[j]->y();
-      kdnodes[k][2] = gf->mesh_vertices[j]->z();
-      k++;
-    }
-  }
-  ANNkd_tree *kdtree = new ANNkd_tree(kdnodes, numnodes, 3);
-
-  // remove the volume elements that are within 'distance' of the
-  // carved surface
-  carveHole(gr->tetrahedra, distance, kdtree);
-  carveHole(gr->hexahedra, distance, kdtree);
-  carveHole(gr->prisms, distance, kdtree);
-  carveHole(gr->pyramids, distance, kdtree);
-
-  delete kdtree;
-  annDeallocPts(kdnodes);
-
-  // TODO: remove any interior elements left inside the carved surface
-  // (could shoot a line from each element's barycenter and count
-  // intersections o see who's inside)
-
-  // generate discrete boundary mesh of the carved hole
-  GFace *gf = m->faceByTag(num);
-  if(!gf) return;
-  std::set<MFace, Less_Face> faces;
-  std::list<GFace*> f = gr->faces();
-  for(std::list<GFace*>::iterator it = f.begin(); it != f.end(); it++){
-    addFaces((*it)->triangles, faces);
-    addFaces((*it)->quadrangles, faces);
-  }
-  addFaces(gr->tetrahedra, faces);
-  addFaces(gr->hexahedra, faces);
-  addFaces(gr->prisms, faces);
-  addFaces(gr->pyramids, faces);
-
-  std::set<MVertex*> verts;
-  for(std::set<MFace, Less_Face>::iterator it = faces.begin(); it != faces.end(); it++){
-    for(int i = 0; i < it->getNumVertices(); i++){
-      it->getVertex(i)->setEntity(gf);
-      verts.insert(it->getVertex(i));
-    }
-    if(it->getNumVertices() == 3)
-      gf->triangles.push_back(new MTriangle(it->getVertex(0), it->getVertex(1),
-					    it->getVertex(2)));
-    else if(it->getNumVertices() == 4)
-      gf->quadrangles.push_back(new MQuadrangle(it->getVertex(0), it->getVertex(1),
-						it->getVertex(2), it->getVertex(3)));
-  }
-#endif
-}
-
 void meshGRegionExtruded::operator() (GRegion *gr) 
 {  
   if(gr->geomType() == GEntity::DiscreteVolume) return;
@@ -319,8 +199,7 @@ void meshGRegionExtruded::operator() (GRegion *gr)
 
   if(!ep || !ep->mesh.ExtrudeMesh || ep->geo.Mode != EXTRUDED_ENTITY) return;
 
-  // Send a messsage to the GMSH environment
-  Msg(STATUS2, "Meshing volume %d (extruded)", gr->tag());
+  Msg::StatusBar(2, true, "Meshing volume %d (extruded)", gr->tag());
 
   // destroy the mesh if it exists
   deMeshGRegion dem;
@@ -333,9 +212,9 @@ void meshGRegionExtruded::operator() (GRegion *gr)
   insertAllVertices(gr, pos);
 
   // volume is extruded from a surface
-  GFace *from = gr->model()->faceByTag(std::abs(ep->geo.Source));
+  GFace *from = gr->model()->getFaceByTag(std::abs(ep->geo.Source));
   if(!from){
-    Msg(GERROR, "Unknown source surface %d for extrusion", ep->geo.Source);
+    Msg::Error("Unknown source surface %d for extrusion", ep->geo.Source);
     return;
   }
 
@@ -353,21 +232,21 @@ void meshGRegionExtruded::operator() (GRegion *gr)
 }
 
 int edgeExists(MVertex *v1, MVertex *v2, 
-	      std::set<std::pair<MVertex*, MVertex*> > &edges)
+              std::set<std::pair<MVertex*, MVertex*> > &edges)
 {
   std::pair<MVertex*, MVertex*> p(std::min(v1, v2), std::max(v1, v2));
   return edges.count(p);
 }
 
 void createEdge(MVertex *v1, MVertex *v2, 
-	      std::set<std::pair<MVertex*, MVertex*> > &edges)
+              std::set<std::pair<MVertex*, MVertex*> > &edges)
 {
   std::pair<MVertex*, MVertex*> p(std::min(v1, v2), std::max(v1, v2));
   edges.insert(p);
 }
 
 void deleteEdge(MVertex *v1, MVertex *v2, 
-	      std::set<std::pair<MVertex*, MVertex*> > &edges)
+              std::set<std::pair<MVertex*, MVertex*> > &edges)
 {
   std::pair<MVertex*, MVertex*> p(std::min(v1, v2), std::max(v1, v2));
   edges.erase(p);
@@ -375,17 +254,17 @@ void deleteEdge(MVertex *v1, MVertex *v2,
 
 // subdivide the 3 lateral faces of each prism
 void phase1(GRegion *gr,
-	    std::set<MVertex*, MVertexLessThanLexicographic> &pos,
-	    std::set<std::pair<MVertex*, MVertex*> > &edges)
+            std::set<MVertex*, MVertexLessThanLexicographic> &pos,
+            std::set<std::pair<MVertex*, MVertex*> > &edges)
 {
   ExtrudeParams *ep = gr->meshAttributes.extrude;
-  GFace *from = gr->model()->faceByTag(std::abs(ep->geo.Source));
+  GFace *from = gr->model()->getFaceByTag(std::abs(ep->geo.Source));
   if(!from) return;
 
   for(unsigned int i = 0; i < from->triangles.size(); i++){
     for(int j = 0; j < ep->mesh.NbLayer; j++) {
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
-	std::vector<MVertex*> v;
+        std::vector<MVertex*> v;
         if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, v) == 6){
 #if 0 // old
           if(!edgeExists(v[0], v[4], edges))
@@ -395,14 +274,14 @@ void phase1(GRegion *gr,
           if(!edgeExists(v[3], v[2], edges))
             createEdge(v[0], v[5], edges);
 #else // new from Michel Benhamou
-	  if(v[1] < v[0]) createEdge(v[1], v[3], edges);
-	  else createEdge(v[0], v[4], edges);
-	  if(v[1] < v[2]) createEdge(v[1], v[5], edges);
-	  else createEdge(v[4], v[2], edges);
-	  if(v[0] < v[2]) createEdge(v[0], v[5], edges);
-	  else createEdge(v[3], v[2], edges);
+          if(v[1] < v[0]) createEdge(v[1], v[3], edges);
+          else createEdge(v[0], v[4], edges);
+          if(v[1] < v[2]) createEdge(v[1], v[5], edges);
+          else createEdge(v[4], v[2], edges);
+          if(v[0] < v[2]) createEdge(v[0], v[5], edges);
+          else createEdge(v[3], v[2], edges);
 #endif
-	}
+        }
       }
     }
   }
@@ -410,67 +289,67 @@ void phase1(GRegion *gr,
 
 // modify lateral edges to make them "tet-compatible"
 void phase2(GRegion *gr,
-	    std::set<MVertex*, MVertexLessThanLexicographic> &pos,
-	    std::set<std::pair<MVertex*, MVertex*> > &edges,
-	    std::set<std::pair<MVertex*, MVertex*> > &edges_swap,
-	    int &swap)
+            std::set<MVertex*, MVertexLessThanLexicographic> &pos,
+            std::set<std::pair<MVertex*, MVertex*> > &edges,
+            std::set<std::pair<MVertex*, MVertex*> > &edges_swap,
+            int &swap)
 {
   ExtrudeParams *ep = gr->meshAttributes.extrude;
-  GFace *from = gr->model()->faceByTag(std::abs(ep->geo.Source));
+  GFace *from = gr->model()->getFaceByTag(std::abs(ep->geo.Source));
   if(!from) return;
 
   for(unsigned int i = 0; i < from->triangles.size(); i++){
     for(int j = 0; j < ep->mesh.NbLayer; j++) {
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
-	std::vector<MVertex*> v;
-	if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, v) == 6){
-	  if(edgeExists(v[3], v[1], edges) &&
-	     edgeExists(v[4], v[2], edges) && 
-	     edgeExists(v[0], v[5], edges)) {
-	    swap++;
-	    if(!edgeExists(v[3], v[1], edges_swap)) {
-	      deleteEdge(v[3], v[1], edges);
-	      createEdge(v[0], v[4], edges);
-	      createEdge(v[3], v[1], edges_swap);
-	      createEdge(v[0], v[4], edges_swap);
-	    }
-	    else if(!edgeExists(v[4], v[2], edges_swap)) {
-	      deleteEdge(v[4], v[2], edges);
-	      createEdge(v[1], v[5], edges);
-	      createEdge(v[4], v[2], edges_swap);
-	      createEdge(v[1], v[5], edges_swap);
-	    }
-	    else if(!edgeExists(v[0], v[5], edges_swap)) {
-	      deleteEdge(v[0], v[5], edges);
-	      createEdge(v[3], v[2], edges);
-	      createEdge(v[0], v[5], edges_swap);
-	      createEdge(v[3], v[2], edges_swap);
-	    }
-	  }
-	  else if(edgeExists(v[0], v[4], edges) &&
-		  edgeExists(v[1], v[5], edges) &&
-		  edgeExists(v[3], v[2], edges)) {
-	    swap++;
-	    if(!edgeExists(v[0], v[4], edges_swap)) {
-	      deleteEdge(v[0], v[4], edges);
-	      createEdge(v[3], v[1], edges);
-	      createEdge(v[0], v[4], edges_swap);
-	      createEdge(v[3], v[1], edges_swap);
-	    }
-	    else if(!edgeExists(v[1], v[5], edges_swap)) {
-	      deleteEdge(v[1], v[5], edges);
-	      createEdge(v[4], v[2], edges);
-	      createEdge(v[1], v[5], edges_swap);
-	      createEdge(v[4], v[2], edges_swap);
-	    }
-	    else if(!edgeExists(v[3], v[2], edges_swap)) {
-	      deleteEdge(v[3], v[2], edges);
-	      createEdge(v[0], v[5], edges);
-	      createEdge(v[3], v[2], edges_swap);
-	      createEdge(v[0], v[5], edges_swap);
-	    }
-	  }
-	}
+        std::vector<MVertex*> v;
+        if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, v) == 6){
+          if(edgeExists(v[3], v[1], edges) &&
+             edgeExists(v[4], v[2], edges) && 
+             edgeExists(v[0], v[5], edges)) {
+            swap++;
+            if(!edgeExists(v[3], v[1], edges_swap)) {
+              deleteEdge(v[3], v[1], edges);
+              createEdge(v[0], v[4], edges);
+              createEdge(v[3], v[1], edges_swap);
+              createEdge(v[0], v[4], edges_swap);
+            }
+            else if(!edgeExists(v[4], v[2], edges_swap)) {
+              deleteEdge(v[4], v[2], edges);
+              createEdge(v[1], v[5], edges);
+              createEdge(v[4], v[2], edges_swap);
+              createEdge(v[1], v[5], edges_swap);
+            }
+            else if(!edgeExists(v[0], v[5], edges_swap)) {
+              deleteEdge(v[0], v[5], edges);
+              createEdge(v[3], v[2], edges);
+              createEdge(v[0], v[5], edges_swap);
+              createEdge(v[3], v[2], edges_swap);
+            }
+          }
+          else if(edgeExists(v[0], v[4], edges) &&
+                  edgeExists(v[1], v[5], edges) &&
+                  edgeExists(v[3], v[2], edges)) {
+            swap++;
+            if(!edgeExists(v[0], v[4], edges_swap)) {
+              deleteEdge(v[0], v[4], edges);
+              createEdge(v[3], v[1], edges);
+              createEdge(v[0], v[4], edges_swap);
+              createEdge(v[3], v[1], edges_swap);
+            }
+            else if(!edgeExists(v[1], v[5], edges_swap)) {
+              deleteEdge(v[1], v[5], edges);
+              createEdge(v[4], v[2], edges);
+              createEdge(v[1], v[5], edges_swap);
+              createEdge(v[4], v[2], edges_swap);
+            }
+            else if(!edgeExists(v[3], v[2], edges_swap)) {
+              deleteEdge(v[3], v[2], edges);
+              createEdge(v[0], v[5], edges);
+              createEdge(v[3], v[2], edges_swap);
+              createEdge(v[0], v[5], edges_swap);
+            }
+          }
+        }
       }
     }
   }
@@ -478,61 +357,61 @@ void phase2(GRegion *gr,
  
 // create tets
 void phase3(GRegion *gr,
-	    std::set<MVertex*, MVertexLessThanLexicographic> &pos,
-	    std::set<std::pair<MVertex*, MVertex*> > &edges)
+            std::set<MVertex*, MVertexLessThanLexicographic> &pos,
+            std::set<std::pair<MVertex*, MVertex*> > &edges)
 {
   ExtrudeParams *ep = gr->meshAttributes.extrude;
-  GFace *from = gr->model()->faceByTag(std::abs(ep->geo.Source));
+  GFace *from = gr->model()->getFaceByTag(std::abs(ep->geo.Source));
   if(!from) return;
 
   for(unsigned int i = 0; i < from->triangles.size(); i++){
     for(int j = 0; j < ep->mesh.NbLayer; j++) {
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
-	std::vector<MVertex*> v;
-	if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, v) == 6){
-	  if(edgeExists(v[3], v[1], edges) &&
-	     edgeExists(v[4], v[2], edges) &&
-	     edgeExists(v[3], v[2], edges)) {
-	    createTet(v[0], v[1], v[2], v[3], gr);
-	    createTet(v[3], v[4], v[5], v[2], gr);
-	    createTet(v[1], v[3], v[4], v[2], gr);
-	  }
-	  else if(edgeExists(v[3], v[1], edges) &&
-		  edgeExists(v[1], v[5], edges) &&
-		  edgeExists(v[3], v[2], edges)) {
-	    createTet(v[0], v[1], v[2], v[3], gr);
-	    createTet(v[3], v[4], v[5], v[1], gr);
-	    createTet(v[3], v[1], v[5], v[2], gr);
-	  }
-	  else if(edgeExists(v[3], v[1], edges) &&
-		  edgeExists(v[1], v[5], edges) &&
-		  edgeExists(v[5], v[0], edges)) {
-	    createTet(v[0], v[1], v[2], v[5], gr);
-	    createTet(v[3], v[4], v[5], v[1], gr);
-	    createTet(v[1], v[3], v[5], v[0], gr);
-	  }
-	  else if(edgeExists(v[4], v[0], edges) &&
-		  edgeExists(v[4], v[2], edges) &&
-		  edgeExists(v[3], v[2], edges)) {
-	    createTet(v[0], v[1], v[2], v[4], gr);
-	    createTet(v[3], v[4], v[5], v[2], gr);
-	    createTet(v[0], v[3], v[4], v[2], gr);
-	  }
-	  else if(edgeExists(v[4], v[0], edges) &&
-		  edgeExists(v[4], v[2], edges) &&
-		  edgeExists(v[5], v[0], edges)) {
-	    createTet(v[0], v[1], v[2], v[4], gr);
-	    createTet(v[3], v[4], v[5], v[0], gr);
-	    createTet(v[0], v[2], v[4], v[5], gr);
-	  }
-	  else if(edgeExists(v[4], v[0], edges) &&
-		  edgeExists(v[1], v[5], edges) &&
-		  edgeExists(v[5], v[0], edges)) {
-	    createTet(v[0], v[1], v[2], v[5], gr);
-	    createTet(v[3], v[4], v[5], v[0], gr);
-	    createTet(v[0], v[1], v[4], v[5], gr);
-	  }
-	}
+        std::vector<MVertex*> v;
+        if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, v) == 6){
+          if(edgeExists(v[3], v[1], edges) &&
+             edgeExists(v[4], v[2], edges) &&
+             edgeExists(v[3], v[2], edges)) {
+            createTet(v[0], v[1], v[2], v[3], gr);
+            createTet(v[3], v[4], v[5], v[2], gr);
+            createTet(v[1], v[3], v[4], v[2], gr);
+          }
+          else if(edgeExists(v[3], v[1], edges) &&
+                  edgeExists(v[1], v[5], edges) &&
+                  edgeExists(v[3], v[2], edges)) {
+            createTet(v[0], v[1], v[2], v[3], gr);
+            createTet(v[3], v[4], v[5], v[1], gr);
+            createTet(v[3], v[1], v[5], v[2], gr);
+          }
+          else if(edgeExists(v[3], v[1], edges) &&
+                  edgeExists(v[1], v[5], edges) &&
+                  edgeExists(v[5], v[0], edges)) {
+            createTet(v[0], v[1], v[2], v[5], gr);
+            createTet(v[3], v[4], v[5], v[1], gr);
+            createTet(v[1], v[3], v[5], v[0], gr);
+          }
+          else if(edgeExists(v[4], v[0], edges) &&
+                  edgeExists(v[4], v[2], edges) &&
+                  edgeExists(v[3], v[2], edges)) {
+            createTet(v[0], v[1], v[2], v[4], gr);
+            createTet(v[3], v[4], v[5], v[2], gr);
+            createTet(v[0], v[3], v[4], v[2], gr);
+          }
+          else if(edgeExists(v[4], v[0], edges) &&
+                  edgeExists(v[4], v[2], edges) &&
+                  edgeExists(v[5], v[0], edges)) {
+            createTet(v[0], v[1], v[2], v[4], gr);
+            createTet(v[3], v[4], v[5], v[0], gr);
+            createTet(v[0], v[2], v[4], v[5], gr);
+          }
+          else if(edgeExists(v[4], v[0], edges) &&
+                  edgeExists(v[1], v[5], edges) &&
+                  edgeExists(v[5], v[0], edges)) {
+            createTet(v[0], v[1], v[2], v[5], gr);
+            createTet(v[3], v[4], v[5], v[0], gr);
+            createTet(v[0], v[1], v[4], v[5], gr);
+          }
+        }
       }
     }
   }
@@ -556,7 +435,7 @@ int SubdivideExtrudedMesh(GModel *m)
 
   if(regions.empty()) return 0;
 
-  Msg(INFO, "Subdividing extruded mesh");
+  Msg::Info("Subdividing extruded mesh");
 
   // create edges on lateral sides of "prisms"
   std::set<std::pair<MVertex*, MVertex*> > edges;
@@ -570,10 +449,10 @@ int SubdivideExtrudedMesh(GModel *m)
     swap = 0;
     for(unsigned int i = 0; i < regions.size(); i++)
       phase2(regions[i], pos, edges, edges_swap, swap);
-    Msg(INFO, "Swapping %d", swap);
+    Msg::Info("Swapping %d", swap);
     if(j && j == swap) {
-      Msg(GERROR, "Unable to subdivide extruded mesh: change surface mesh or");
-      Msg(GERROR, "recombine extrusion instead");
+      Msg::Error("Unable to subdivide extruded mesh: change surface mesh or");
+      Msg::Error("recombine extrusion instead");
       return -1;
     }
     j = swap;
@@ -605,12 +484,12 @@ int SubdivideExtrudedMesh(GModel *m)
     if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == EXTRUDED_ENTITY && 
        !ep->mesh.Recombine){
       GFace *gf = *it;
-      Msg(INFO, "Remeshing surface %d", gf->tag());
+      Msg::Info("Remeshing surface %d", gf->tag());
       for(unsigned int i = 0; i < gf->triangles.size(); i++) 
-	delete gf->triangles[i];
+        delete gf->triangles[i];
       gf->triangles.clear();
       for(unsigned int i = 0; i < gf->quadrangles.size(); i++) 
-	delete gf->quadrangles[i];
+        delete gf->quadrangles[i];
       gf->quadrangles.clear();
       MeshExtrudedSurface(gf, &edges);
     }
@@ -623,7 +502,7 @@ int SubdivideExtrudedMesh(GModel *m)
     if(ep->mesh.Holes.size()){
       std::map<int, std::pair<double, std::vector<int> > >::iterator it;
       for(it = ep->mesh.Holes.begin(); it != ep->mesh.Holes.end(); it++)
-	carveHole(gr, it->first, it->second.first, it->second.second);
+        carveHole(gr, it->first, it->second.first, it->second.second);
     }
   }
 
