@@ -1,7 +1,7 @@
 #ifndef _CREATE_FILE_H_
 #define _CREATE_FILE_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-int GuessFileFormatFromFileName(char *name);
+int GuessFileFormatFromFileName(const char *name);
 void GetDefaultFileName(int format, char *name);
-void CreateOutputFile(char *name, int format);
+void CreateOutputFile(const char *name, int format);
 
 #endif
