@@ -1,6 +1,6 @@
-// $Id: OpenFile.cpp,v 1.149 2007-07-11 16:38:36 geuzaine Exp $
+// $Id: OpenFile.cpp,v 1.188 2008-05-06 21:11:54 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@
 #include <sys/cygwin.h>
 #endif
 
-#include "Gmsh.h"
+#include <string.h>
+#include "Message.h"
 #include "Geo.h"
 #include "GModel.h"
 #include "Numeric.h"
@@ -31,40 +32,43 @@
 #include "Parser.h"
 #include "OpenFile.h"
 #include "CommandLine.h"
-#include "Views.h"
 #include "ReadImg.h"
 #include "OS.h"
 #include "HighOrder.h"
 
+#if !defined(HAVE_NO_POST)
+#include "PView.h"
+#include "PViewData.h"
+#endif
+
 #if defined(HAVE_FLTK)
 #include "GmshUI.h"
 #include "Draw.h"
 #include "SelectBuffer.h"
 #include "GUI.h"
 extern GUI *WID;
-void UpdateViewsInGUI();
 #endif
 
-extern Mesh *THEM;
-extern GModel *GMODEL;
 extern Context_T CTX;
 
-void FixRelativePath(char *in, char *out){
+void FixRelativePath(const char *in, char *out)
+{
   if(in[0] == '/' || in[0] == '\\' || (strlen(in)>2 && in[1] == ':')){
     // do nothing: 'in' is an absolute path
     strcpy(out, in);
   }
   else{
     // append 'in' to the path of the parent file
-    strcpy(out, yyname);
+    strcpy(out, gmsh_yyname);
     int i = strlen(out)-1 ;
-    while(i >= 0 && yyname[i] != '/' && yyname[i] != '\\') i-- ;
+    while(i >= 0 && gmsh_yyname[i] != '/' && gmsh_yyname[i] != '\\') i-- ;
     out[i+1] = '\0';
     strcat(out, in);
   }
 }
 
-void FixWindowsPath(char *in, char *out){
+void FixWindowsPath(const char *in, char *out)
+{
 #if defined(__CYGWIN__)
   cygwin_conv_to_win32_path(in, out);
 #else
@@ -72,14 +76,21 @@ void FixWindowsPath(char *in, char *out){
 #endif
 }
 
-void SplitFileName(char *name, char *base, char *ext)
+void SplitFileName(const char *name, char *no_ext, char *ext, char *base)
 {
-  strcpy(base, name);
+  strcpy(no_ext, name);
   strcpy(ext, "");
-  for(int i = strlen(name)-1; i >= 0; i--){
+  for(int i = strlen(name) - 1; i >= 0; i--){
     if(name[i] == '.'){
       strcpy(ext, &name[i]);
-      base[i] = '\0';
+      no_ext[i] = '\0';
+      break;
+    }
+  }
+  strcpy(base, no_ext);
+  for(int i = strlen(no_ext) - 1; i >= 0; i--){
+    if(no_ext[i] == '/' || no_ext[i] == '\\'){
+      strcpy(base, &no_ext[i + 1]);
       break;
     }
   }
@@ -94,33 +105,38 @@ static void FinishUpBoundingBox()
     range[i] = CTX.max[i] - CTX.min[i];
   }
 
-  if(range[0] == 0. && range[1] == 0. && range[2] == 0.) {
+  if(range[0] < CTX.geom.tolerance && 
+     range[1] < CTX.geom.tolerance && 
+     range[2] < CTX.geom.tolerance) {
     CTX.min[0] -= 1.; CTX.min[1] -= 1.;
     CTX.max[0] += 1.; CTX.max[1] += 1.;
     CTX.lc = 1.;
   }
-  else if(range[0] == 0. && range[1] == 0.) {
+  else if(range[0] < CTX.geom.tolerance && 
+          range[1] < CTX.geom.tolerance) {
     CTX.lc = range[2];
     CTX.min[0] -= CTX.lc; CTX.min[1] -= CTX.lc;
     CTX.max[0] += CTX.lc; CTX.max[1] += CTX.lc;
   }
-  else if(range[0] == 0. && range[2] == 0.) {
+  else if(range[0] < CTX.geom.tolerance && 
+          range[2] < CTX.geom.tolerance) {
     CTX.lc = range[1];
     CTX.min[0] -= CTX.lc; CTX.max[0] += CTX.lc;
   }
-  else if(range[1] == 0. && range[2] == 0.) {
+  else if(range[1] < CTX.geom.tolerance && 
+          range[2] < CTX.geom.tolerance) {
     CTX.lc = range[0];
     CTX.min[1] -= CTX.lc; CTX.max[1] += CTX.lc;
   }
-  else if(range[0] == 0.) {
+  else if(range[0] < CTX.geom.tolerance) {
     CTX.lc = sqrt(DSQR(range[1]) + DSQR(range[2]));
     CTX.min[0] -= CTX.lc; CTX.max[0] += CTX.lc;
   }
-  else if(range[1] == 0.) {
+  else if(range[1] < CTX.geom.tolerance) {
     CTX.lc = sqrt(DSQR(range[0]) + DSQR(range[2]));
     CTX.min[1] -= CTX.lc; CTX.max[1] += CTX.lc;
   }
-  else if(range[2] == 0.) {
+  else if(range[2] < CTX.geom.tolerance) {
     CTX.lc = sqrt(DSQR(range[0]) + DSQR(range[1]));
   }
   else {
@@ -129,8 +145,8 @@ static void FinishUpBoundingBox()
 }
 
 void SetBoundingBox(double xmin, double xmax,
-		    double ymin, double ymax, 
-		    double zmin, double zmax)
+                    double ymin, double ymax, 
+                    double zmin, double zmax)
 {
   CTX.min[0] = xmin; CTX.max[0] = xmax;
   CTX.min[1] = ymin; CTX.max[1] = ymax;
@@ -142,24 +158,16 @@ void SetBoundingBox(void)
 {
   if(CTX.forced_bbox) return;
 
-  SBoundingBox3d bb;
-
-  bb = GMODEL->bounds();
+  SBoundingBox3d bb = GModel::current()->bounds();
   
-  if(bb.empty() && List_Nbr(CTX.post.list)) {
-    for(int i = 0; i < List_Nbr(CTX.post.list); i++){
-      Post_View *v = *(Post_View **)List_Pointer(CTX.post.list, i);
-      if(fabs(v->BBox[0]) != VAL_INF && 
-	 fabs(v->BBox[2]) != VAL_INF &&
-	 fabs(v->BBox[4]) != VAL_INF)
-	bb += SPoint3(v->BBox[0], v->BBox[2], v->BBox[4]);
-      if(fabs(v->BBox[1]) != VAL_INF && 
-	 fabs(v->BBox[3]) != VAL_INF &&
-	 fabs(v->BBox[5]) != VAL_INF)
-      bb += SPoint3(v->BBox[1], v->BBox[3], v->BBox[5]);
-    }
+#if !defined(HAVE_NO_POST)
+  if(bb.empty()) {
+    for(unsigned int i = 0; i < PView::list.size(); i++)
+      if(!PView::list[i]->getData()->getBoundingBox().empty())
+        bb += PView::list[i]->getData()->getBoundingBox();
   }
-
+#endif
+  
   if(bb.empty()){
     bb += SPoint3(-1., -1., -1.);
     bb += SPoint3(1., 1., 1.);
@@ -192,62 +200,66 @@ void AddToTemporaryBoundingBox(double x, double y, double z)
   FinishUpBoundingBox();
 }
 
-int ParseFile(char *f, int close, int warn_if_missing)
+int ParseFile(const char *f, int close, int warn_if_missing)
 {
   char yyname_old[256], tmp[256];
   FILE *yyin_old, *fp;
-  int yylineno_old, yyerrorstate_old, numviews_old;
+  int yylineno_old, yyerrorstate_old, yyviewindex_old;
 
   // add 'b' for pure Windows programs: opening in text mode messes up
   // fsetpos/fgetpos (used e.g. for user-defined functions)
   if(!(fp = fopen(f, "rb"))){
-    if(warn_if_missing) Msg(WARNING, "Unable to open file '%s'", f);
+    if(warn_if_missing) Msg::Warning("Unable to open file '%s'", f);
     return 0;
   }
 
-  strncpy(yyname_old, yyname, 255);
-  yyin_old = yyin;
-  yyerrorstate_old = yyerrorstate;
-  yylineno_old = yylineno;
-  numviews_old = List_Nbr(CTX.post.list);
+#if !defined(HAVE_NO_POST)
+  int numViewsBefore = PView::list.size();
+#endif
 
-  strncpy(yyname, f, 255);
-  yyin = fp;
-  yyerrorstate = 0;
-  yylineno = 1;
+  strncpy(yyname_old, gmsh_yyname, 255);
+  yyin_old = gmsh_yyin;
+  yyerrorstate_old = gmsh_yyerrorstate;
+  yylineno_old = gmsh_yylineno;
+  yyviewindex_old = gmsh_yyviewindex;
+
+  strncpy(gmsh_yyname, f, 255);
+  gmsh_yyin = fp;
+  gmsh_yyerrorstate = 0;
+  gmsh_yylineno = 1;
+  gmsh_yyviewindex = 0;
 
   fpos_t position;
-  fgetpos(yyin, &position);
-  fgets(tmp, sizeof(tmp), yyin);
-  fsetpos(yyin, &position);
+  fgetpos(gmsh_yyin, &position);
+  fgets(tmp, sizeof(tmp), gmsh_yyin);
+  fsetpos(gmsh_yyin, &position);
 
-  while(!feof(yyin)){
-    yyparse();
-    if(yyerrorstate > 20){
-      Msg(GERROR, "Too many errors: aborting...");
+  while(!feof(gmsh_yyin)){
+    gmsh_yyparse();
+    if(gmsh_yyerrorstate > 20){
+      Msg::Error("Too many errors: aborting...");
       force_yyflush();
       break;
     }
   }
 
-  if(close)
-    fclose(yyin);
+  if(close) fclose(gmsh_yyin);
 
-  strncpy(yyname, yyname_old, 255);
-  yyin = yyin_old;
-  yyerrorstate = yyerrorstate_old;
-  yylineno = yylineno_old;
+  strncpy(gmsh_yyname, yyname_old, 255);
+  gmsh_yyin = yyin_old;
+  gmsh_yyerrorstate = yyerrorstate_old;
+  gmsh_yylineno = yylineno_old;
+  gmsh_yyviewindex = yyviewindex_old;
 
-  if(List_Nbr(CTX.post.list) != numviews_old){
-#if defined(HAVE_FLTK)
-    UpdateViewsInGUI();
+#if defined(HAVE_FLTK) && !defined(HAVE_NO_POST)
+  if(!CTX.batch && numViewsBefore != (int)PView::list.size())
+    WID->update_views();
 #endif
-  }
-  
+
   return 1;
 }
 
-void ParseString(char *str)
+void ParseString(const char *str)
 {
   if(!str) return;
   FILE *fp;
@@ -256,30 +268,34 @@ void ParseString(char *str)
     fprintf(fp, "\n");
     fclose(fp);
     ParseFile(CTX.tmp_filename_fullpath, 1);
-    if(GMODEL) GMODEL->importTHEM();
+    GModel::current()->importGEOInternals();
   }
 }
 
-void SetProjectName(char *name)
+void SetProjectName(const char *name)
 {
-  char base[356], ext[256];
-  SplitFileName(name, base, ext);
+  char no_ext[256], ext[256], base[256];
+  SplitFileName(name, no_ext, ext, base);
 
-  strncpy(CTX.filename, name, 255);
+  if(CTX.filename != name) // yes, we mean to compare the pointers
+    strncpy(CTX.filename, name, 255);
+  strncpy(CTX.no_ext_filename, no_ext, 255);
   strncpy(CTX.base_filename, base, 255);
 
+  GModel::current()->setName(base);
+    
 #if defined(HAVE_FLTK)
   if(!CTX.batch) WID->set_title(CTX.filename);
 #endif
 }
 
-int MergeFile(char *name, int warn_if_missing)
+int MergeFile(const char *name, int warn_if_missing)
 {
   // added 'b' for pure Windows programs, since some of these files
   // contain binary data
   FILE *fp = fopen(name, "rb");
   if(!fp){
-    if(warn_if_missing) Msg(WARNING, "Unable to open file '%s'", name);
+    if(warn_if_missing) Msg::Warning("Unable to open file '%s'", name);
     return 0;
   }
 
@@ -287,10 +303,10 @@ int MergeFile(char *name, int warn_if_missing)
   fgets(header, sizeof(header), fp);
   fclose(fp);
 
-  Msg(STATUS2, "Reading '%s'", name);
+  Msg::StatusBar(2, true, "Reading '%s'", name);
 
-  char ext[256], base[256];
-  SplitFileName(name, base, ext);
+  char no_ext[256], ext[256], base[256];
+  SplitFileName(name, no_ext, ext, base);
 
 #if defined(HAVE_FLTK)
   if(!CTX.batch) {
@@ -298,62 +314,84 @@ int MergeFile(char *name, int warn_if_missing)
       // the real solution would be to rewrite all our I/O functions in
       // terms of gzFile, but until then, this is better than nothing
       if(fl_choice("File '%s' is in gzip format.\n\nDo you want to uncompress it?", 
-		   "Cancel", "Uncompress", NULL, name)){
-	char tmp[256];
-	sprintf(tmp, "gunzip -c %s > %s", name, base);
-	if(SystemCall(tmp))
-	  Msg(GERROR, "Failed to uncompress `%s': check directory permissions", name);
-	if(!strcmp(CTX.filename, name)) // this is the project file
-	  SetProjectName(base);
-	return MergeFile(base);
+                   "Cancel", "Uncompress", NULL, name)){
+        char tmp[256];
+        sprintf(tmp, "gunzip -c %s > %s", name, no_ext);
+        if(SystemCall(tmp))
+          Msg::Error("Failed to uncompress `%s': check directory permissions", name);
+        if(!strcmp(CTX.filename, name)) // this is the project file
+          SetProjectName(no_ext);
+        return MergeFile(no_ext);
       }
     }
   }
 #endif
 
   CTX.geom.draw = 0; // don't try to draw the model while reading
+
+  GModel *m = GModel::current();
+
+  // FIXME: We need to decide what do do for CAD entities, meshes,
+  // etc.  For meshes we should definitely create one new model per
+  // merge (and reset current() to the previous value after the
+  // merge). This will make multi-step multi-meshes post-pro views
+  // work out of the box.
+  // GModel *m = new GModel;
+
+#if !defined(HAVE_NO_POST)
+  int numViewsBefore = PView::list.size();
+#endif
+
   int status = 0;
   if(!strcmp(ext, ".stl") || !strcmp(ext, ".STL")){
-    status = GMODEL->readSTL(name, CTX.geom.tolerance);
+    status = m->readSTL(name, CTX.geom.tolerance);
   }
   else if(!strcmp(ext, ".brep") || !strcmp(ext, ".rle") ||
-	  !strcmp(ext, ".brp") || !strcmp(ext, ".BRP")){
-    GMODEL->readOCCBREP(std::string(name));
+          !strcmp(ext, ".brp") || !strcmp(ext, ".BRP")){
+    status = m->readOCCBREP(std::string(name));
   }
   else if(!strcmp(ext, ".iges") || !strcmp(ext, ".IGES") ||
-	  !strcmp(ext, ".igs") || !strcmp(ext, ".IGS")){
-    GMODEL->readOCCIGES(std::string(name));
+          !strcmp(ext, ".igs") || !strcmp(ext, ".IGS")){
+    status = m->readOCCIGES(std::string(name));
   }
   else if(!strcmp(ext, ".step") || !strcmp(ext, ".STEP") ||
-	  !strcmp(ext, ".stp") || !strcmp(ext, ".STP")){
-    GMODEL->readOCCSTEP(std::string(name));
+          !strcmp(ext, ".stp") || !strcmp(ext, ".STP")){
+    status = m->readOCCSTEP(std::string(name));
   }
   else if(!strcmp(ext, ".unv") || !strcmp(ext, ".UNV")){
-    status = GMODEL->readUNV(name);
+    status = m->readUNV(name);
   }
   else if(!strcmp(ext, ".wrl") || !strcmp(ext, ".WRL") || 
-	  !strcmp(ext, ".vrml") || !strcmp(ext, ".VRML") ||
-	  !strcmp(ext, ".iv") || !strcmp(ext, ".IV")){
-    status = GMODEL->readVRML(name);
+          !strcmp(ext, ".vrml") || !strcmp(ext, ".VRML") ||
+          !strcmp(ext, ".iv") || !strcmp(ext, ".IV")){
+    status = m->readVRML(name);
   }
   else if(!strcmp(ext, ".mesh") || !strcmp(ext, ".MESH")){
-    status = GMODEL->readMESH(name);
+    status = m->readMESH(name);
   }
+#if !defined(HAVE_NO_POST)
+  else if(!strcmp(ext, ".med") || !strcmp(ext, ".MED") ||
+	  !strcmp(ext, ".mmed") || !strcmp(ext, ".MMED") ||
+	  !strcmp(ext, ".rmed") || !strcmp(ext, ".RMED")){
+    status = GModel::readMED(name);
+    if(status > 1) status = PView::readMED(name);
+  }
+#endif
   else if(!strcmp(ext, ".bdf") || !strcmp(ext, ".BDF") ||
-	  !strcmp(ext, ".nas") || !strcmp(ext, ".NAS")){
-    status = GMODEL->readBDF(name);
+          !strcmp(ext, ".nas") || !strcmp(ext, ".NAS")){
+    status = m->readBDF(name);
   }
   else if(!strcmp(ext, ".p3d") || !strcmp(ext, ".P3D")){
-    status = GMODEL->readP3D(name);
+    status = m->readP3D(name);
   }
   else if(!strcmp(ext, ".fm") || !strcmp(ext, ".FM")) {
-    status = GMODEL->readF(name);
+    status = m->readFourier(name);
   }
 #if defined(HAVE_FLTK)
   else if(!strcmp(ext, ".pnm") || !strcmp(ext, ".PNM") ||
-	  !strcmp(ext, ".pbm") || !strcmp(ext, ".PBM") ||
-	  !strcmp(ext, ".pgm") || !strcmp(ext, ".PGM") ||
-	  !strcmp(ext, ".ppm") || !strcmp(ext, ".PPM")) {
+          !strcmp(ext, ".pbm") || !strcmp(ext, ".PBM") ||
+          !strcmp(ext, ".pgm") || !strcmp(ext, ".PGM") ||
+          !strcmp(ext, ".ppm") || !strcmp(ext, ".PPM")) {
     status = read_pnm(name);
   }
   else if(!strcmp(ext, ".bmp") || !strcmp(ext, ".BMP")) {
@@ -361,7 +399,7 @@ int MergeFile(char *name, int warn_if_missing)
   }
 #if defined(HAVE_LIBJPEG)
   else if(!strcmp(ext, ".jpg") || !strcmp(ext, ".JPG") ||
-	  !strcmp(ext, ".jpeg") || !strcmp(ext, ".JPEG")) {
+          !strcmp(ext, ".jpeg") || !strcmp(ext, ".JPEG")) {
     status = read_jpeg(name);
   }
 #endif
@@ -376,14 +414,19 @@ int MergeFile(char *name, int warn_if_missing)
     if(!strncmp(header, "$PTS", 4) || !strncmp(header, "$NO", 3) || 
        !strncmp(header, "$PARA", 5) || !strncmp(header, "$ELM", 4) ||
        !strncmp(header, "$MeshFormat", 11)) {
-      status = GMODEL->readMSH(name);
+      status = m->readMSH(name);
+#if !defined(HAVE_NO_POST)
+      if(status > 1) status = PView::readMSH(name);
+#endif
     }
+#if !defined(HAVE_NO_POST)
     else if(!strncmp(header, "$PostFormat", 11) || 
-	    !strncmp(header, "$View", 5)) {
-      status = ReadView(name);
+            !strncmp(header, "$View", 5)) {
+      status = PView::readPOS(name);
     }
+#endif
     else {
-      status = GMODEL->readGEO(name);
+      status = m->readGEO(name);
     }
   }
 
@@ -392,27 +435,34 @@ int MergeFile(char *name, int warn_if_missing)
   CTX.geom.draw = 1;
   CTX.mesh.changed = ENT_ALL;
 
-  checkHighOrderTriangles ( GMODEL );
+#if defined(HAVE_FLTK) && !defined(HAVE_NO_POST)
+  if(!CTX.batch && numViewsBefore != (int)PView::list.size())
+    WID->update_views();
+#endif
 
-  Msg(STATUS2, "Read '%s'", name);
+  Msg::StatusBar(2, true, "Read '%s'", name);
   return status;
 }
 
-void OpenProject(char *name)
+void OpenProject(const char *name)
 {
   if(CTX.threads_lock) {
-    Msg(INFO, "I'm busy! Ask me that later...");
+    Msg::Info("I'm busy! Ask me that later...");
     return;
   }
   CTX.threads_lock = 1;
 
-  GMODEL->destroy();
-  THEM->destroy();
+  // FIXME: this will change once we clarify Open/Merge/Clear
+#if !defined(HAVE_NO_POST)
+  for(int i = PView::list.size() - 1; i >= 0; i--)
+    if(PView::list[i]->getData()->hasModel(GModel::current()))
+      delete PView::list[i];
+#endif
+  GModel::current()->destroy();
+  GModel::current()->getGEOInternals()->destroy();
 
-  // Initialize pseudo random mesh generator to the same seed
-  srand(1);
-
-  // temporary hack until we fill GMODEL on the fly during parsing
+  // temporary hack until we fill the current GModel on the fly during
+  // parsing
   ResetTemporaryBoundingBox();
 
   SetProjectName(name);
@@ -421,7 +471,10 @@ void OpenProject(char *name)
   CTX.threads_lock = 0;
 
 #if defined(HAVE_FLTK)
-  if(!CTX.batch) WID->reset_visibility();
+  if(!CTX.batch){
+    WID->reset_visibility();
+    WID->update_views();
+  }
   ZeroHighlight();
 #endif
 }
@@ -436,10 +489,9 @@ void OpenProjectMacFinder(const char *filename)
     first = 0;
   }
   else{
-    OpenProject((char*)filename);
+    OpenProject(filename);
 #if defined(HAVE_FLTK)
     Draw();
 #endif
   }
 }
-
