@@ -1,7 +1,7 @@
 #ifndef _OPENFILE_H_
 #define _OPENFILE_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,17 +20,17 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-int ParseFile(char *filename, int close, int warn_if_missing=0);
-void ParseString(char *str);
-void OpenProject(char *filename);
+int ParseFile(const char *filename, int close, int warn_if_missing=0);
+void ParseString(const char *str);
+void OpenProject(const char *filename);
 void OpenProjectMacFinder(const char *filename);
-int MergeFile(char *filename, int warn_if_missing=0);
-void FixRelativePath(char *in, char *out);
-void FixWindowsPath(char *in, char *out);
-void SplitFileName(char *name, char *base, char *ext);
+int MergeFile(const char *filename, int warn_if_missing=0);
+void FixRelativePath(const char *in, char *out);
+void FixWindowsPath(const char *in, char *out);
+void SplitFileName(const char *name, char *no_ext, char *ext, char *base);
 void SetBoundingBox(double xmin, double xmax,
-		    double ymin, double ymax, 
-		    double zmin, double zmax);
+                    double ymin, double ymax, 
+                    double zmin, double zmax);
 void SetBoundingBox(void);
 void AddToTemporaryBoundingBox(double x, double y, double z);
 
