@@ -1,7 +1,7 @@
 #ifndef _PARSER_H_
 #define _PARSER_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "List.h"
-#include "Tree.h"
+#include "ListUtils.h"
+#include "TreeUtils.h"
 
 typedef struct {
   char *Name;
@@ -32,14 +32,17 @@ void InitSymbols();
 
 extern Tree_T *Symbol_T;
 
-int yyparse();
-int yylex();
+int gmsh_yyparse();
+int gmsh_yylex();
+
 void force_yyflush();
 
-extern FILE *yyin;
-extern int yylineno;
-extern char yyname[256];
-extern char *yytext;
-extern int yyerrorstate;
+extern FILE *gmsh_yyin;
+extern int gmsh_yylineno;
+extern char *gmsh_yytext;
+
+extern int gmsh_yyviewindex;
+extern char gmsh_yyname[256];
+extern int gmsh_yyerrorstate;
 
 #endif
