@@ -1,6 +1,6 @@
-// $Id: Annotate.cpp,v 1.16 2007-05-04 10:45:08 geuzaine Exp $
+// $Id: Annotate.cpp,v 1.22 2008-04-05 17:49:23 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,14 +19,9 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include <iostream>
 #include <vector>
-#include "Plugin.h"
 #include "Annotate.h"
-#include "List.h"
-#include "Views.h"
 #include "Context.h"
-#include "Numeric.h"
 
 #if defined(HAVE_FLTK)
 #include "GmshUI.h"
@@ -40,7 +35,7 @@ StringXNumber AnnotateOptions_Number[] = {
   {GMSH_FULLRC, "X", GMSH_AnnotatePlugin::callbackX, 50.},
   {GMSH_FULLRC, "Y", GMSH_AnnotatePlugin::callbackY, 30.},
   {GMSH_FULLRC, "Z", GMSH_AnnotatePlugin::callbackZ, 0.},
-  {GMSH_FULLRC, "3D", GMSH_AnnotatePlugin::callback3D, 0.},
+  {GMSH_FULLRC, "ThereD", GMSH_AnnotatePlugin::callback3D, 0.},
   {GMSH_FULLRC, "FontSize", GMSH_AnnotatePlugin::callbackFontSize, 14.},
   {GMSH_FULLRC, "iView", NULL, -1.}
 };
@@ -59,11 +54,6 @@ extern "C"
   }
 }
 
-GMSH_AnnotatePlugin::GMSH_AnnotatePlugin()
-{
-  ;
-}
-
 static double getStyle()
 {
   int fontsize = (int)AnnotateOptions_Number[4].def, font = 0, align = 0;
@@ -102,7 +92,7 @@ void GMSH_AnnotatePlugin::draw()
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
     glOrtho((double)CTX.viewport[0], (double)CTX.viewport[2],
-	    (double)CTX.viewport[1], (double)CTX.viewport[3], -1., 1.);
+            (double)CTX.viewport[1], (double)CTX.viewport[3], -1., 1.);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
     Fix2DCoordinates(&X, &Y);
@@ -123,7 +113,7 @@ void GMSH_AnnotatePlugin::draw()
 }
 
 double GMSH_AnnotatePlugin::callback(int num, int action, double value, double *opt,
-				     double step, double min, double max)
+                                     double step, double min, double max)
 {
   switch(action){ // configure the input field
   case 1: return step;
@@ -138,7 +128,8 @@ double GMSH_AnnotatePlugin::callback(int num, int action, double value, double *
   return 0.;
 }
 
-char *GMSH_AnnotatePlugin::callbackStr(int num, int action, char *value, char **opt)
+const char *GMSH_AnnotatePlugin::callbackStr(int num, int action, const char *value,
+                                             const char **opt)
 {
   *opt = value;
 #if defined(HAVE_FLTK)
@@ -152,9 +143,9 @@ double GMSH_AnnotatePlugin::callbackX(int num, int action, double value)
   // not perfect: the change will only take place if we reopen the dialog...
   int dim3 = (int)AnnotateOptions_Number[3].def;
   return callback(num, action, value, &AnnotateOptions_Number[0].def,
-		  dim3 ? CTX.lc/200. : 0.5, 
-		  dim3 ? -CTX.lc : -100., 
-		  dim3 ? CTX.lc : 100000.);
+                  dim3 ? CTX.lc/200. : 0.5, 
+                  dim3 ? -CTX.lc : -100., 
+                  dim3 ? CTX.lc : 100000.);
 }
 
 double GMSH_AnnotatePlugin::callbackY(int num, int action, double value)
@@ -162,9 +153,9 @@ double GMSH_AnnotatePlugin::callbackY(int num, int action, double value)
   // not perfect: the change will only take place if we reopen the dialog...
   int dim3 = (int)AnnotateOptions_Number[3].def;
   return callback(num, action, value, &AnnotateOptions_Number[1].def,
-		  dim3 ? CTX.lc/200. : 0.5, 
-		  dim3 ? -CTX.lc : -100., 
-		  dim3 ? CTX.lc : 100000.);
+                  dim3 ? CTX.lc/200. : 0.5, 
+                  dim3 ? -CTX.lc : -100., 
+                  dim3 ? CTX.lc : 100000.);
 }
 
 double GMSH_AnnotatePlugin::callbackZ(int num, int action, double value)
@@ -172,34 +163,34 @@ double GMSH_AnnotatePlugin::callbackZ(int num, int action, double value)
   // not perfect: the change will only take place if we reopen the dialog...
   int dim3 = (int)AnnotateOptions_Number[3].def;
   return callback(num, action, value, &AnnotateOptions_Number[2].def,
-		  dim3 ? CTX.lc/200. : 0.5, 
-		  dim3 ? -CTX.lc : -100., 
-		  dim3 ? CTX.lc : 100000.);
+                  dim3 ? CTX.lc/200. : 0.5, 
+                  dim3 ? -CTX.lc : -100., 
+                  dim3 ? CTX.lc : 100000.);
 }
 
 double GMSH_AnnotatePlugin::callback3D(int num, int action, double value)
 {
   return callback(num, action, value, &AnnotateOptions_Number[3].def,
-		  1, 0, 1);
+                  1, 0, 1);
 }
 
 double GMSH_AnnotatePlugin::callbackFontSize(int num, int action, double value)
 {
   return callback(num, action, value, &AnnotateOptions_Number[4].def,
-		  1, 5, 100);
+                  1, 5, 100);
 }
 
-char *GMSH_AnnotatePlugin::callbackText(int num, int action, char *value)
+const char *GMSH_AnnotatePlugin::callbackText(int num, int action, const char *value)
 {
   return callbackStr(num, action, value, &AnnotateOptions_String[0].def);
 }
 
-char *GMSH_AnnotatePlugin::callbackFont(int num, int action, char *value)
+const char *GMSH_AnnotatePlugin::callbackFont(int num, int action, const char *value)
 {
   return callbackStr(num, action, value, &AnnotateOptions_String[1].def);
 }
 
-char *GMSH_AnnotatePlugin::callbackAlign(int num, int action, char *value)
+const char *GMSH_AnnotatePlugin::callbackAlign(int num, int action, const char *value)
 {
   return callbackStr(num, action, value, &AnnotateOptions_String[2].def);
 }
@@ -210,22 +201,22 @@ void GMSH_AnnotatePlugin::getName(char *name) const
 }
 
 void GMSH_AnnotatePlugin::getInfos(char *author, char *copyright,
-				   char *help_text) const
+                                   char *help_text) const
 {
   strcpy(author, "C. Geuzaine");
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
          "Plugin(Annotate) adds the text string `Text',\n"
-	 "in font `Font' and size `FontSize', in the view\n"
-	 "`iView'. If `3D' is equal to 1, the plugin inserts\n"
-	 "the string in model coordinates at the position\n"
-	 "(`X',`Y',`Z'). If `3D' is equal to 0, the plugin\n"
-	 "inserts the string in screen coordinates at\n"
-	 "the position (`X',`Y'). The string is aligned\n"
-	 "according to `Align'. If `iView' < 0, the plugin\n"
-	 "is run on the current view.\n"
-	 "\n"
-	 "Plugin(Annotate) is executed in-place.\n");
+         "in font `Font' and size `FontSize', in the view\n"
+         "`iView'. If `ThreeD' is equal to 1, the plugin inserts\n"
+         "the string in model coordinates at the position\n"
+         "(`X',`Y',`Z'). If `ThreeD' is equal to 0, the plugin\n"
+         "inserts the string in screen coordinates at\n"
+         "the position (`X',`Y'). The string is aligned\n"
+         "according to `Align'. If `iView' < 0, the plugin\n"
+         "is run on the current view.\n"
+         "\n"
+         "Plugin(Annotate) is executed in-place.\n");
 }
 
 int GMSH_AnnotatePlugin::getNbOptions() const
@@ -253,46 +244,42 @@ void GMSH_AnnotatePlugin::catchErrorMessage(char *errorMessage) const
   strcpy(errorMessage, "Annotate failed...");
 }
 
-Post_View *GMSH_AnnotatePlugin::execute(Post_View * v)
+PView *GMSH_AnnotatePlugin::execute(PView *v)
 {
   double X = AnnotateOptions_Number[0].def;
   double Y = AnnotateOptions_Number[1].def;
   double Z = AnnotateOptions_Number[2].def;
   int dim3 = (int)AnnotateOptions_Number[3].def;
   int iView = (int)AnnotateOptions_Number[5].def;
-  char *text = AnnotateOptions_String[0].def;
+  const char *text = AnnotateOptions_String[0].def;
   double style = getStyle();
 
-  if(iView < 0)
-    iView = v ? v->Index : 0;
+  PView *v1 = getView(iView, v);
+  if(!v1) return v;
 
-  if(!List_Pointer_Test(CTX.post.list, iView)) {
-    Msg(GERROR, "View[%d] does not exist", iView);
-    return v;
-  }
-
-  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
+  PViewDataList *data1 = getDataList(v1);
+  if(!data1) return v;
 
   if(dim3){
-    List_Add(v1->T3D, &X);
-    List_Add(v1->T3D, &Y);
-    List_Add(v1->T3D, &Z);
-    List_Add(v1->T3D, &style); 
-    double d = List_Nbr(v1->T3C);
-    List_Add(v1->T3D, &d); 
-    for(int i = 0; i < (int)strlen(text)+1; i++) 
-      List_Add(v1->T3C, &text[i]); 
-    v1->NbT3++;
+    List_Add(data1->T3D, &X);
+    List_Add(data1->T3D, &Y);
+    List_Add(data1->T3D, &Z);
+    List_Add(data1->T3D, &style); 
+    double d = List_Nbr(data1->T3C);
+    List_Add(data1->T3D, &d); 
+    for(int i = 0; i < (int)strlen(text) + 1; i++) 
+      List_Add(data1->T3C, (void*)&text[i]); 
+    data1->NbT3++;
   }
   else{
-    List_Add(v1->T2D, &X);
-    List_Add(v1->T2D, &Y);
-    List_Add(v1->T2D, &style); 
-    double d = List_Nbr(v1->T2C);
-    List_Add(v1->T2D, &d); 
-    for(int i = 0; i < (int)strlen(text)+1; i++) 
-      List_Add(v1->T2C, &text[i]); 
-    v1->NbT2++;
+    List_Add(data1->T2D, &X);
+    List_Add(data1->T2D, &Y);
+    List_Add(data1->T2D, &style); 
+    double d = List_Nbr(data1->T2C);
+    List_Add(data1->T2D, &d); 
+    for(int i = 0; i < (int)strlen(text) + 1; i++) 
+      List_Add(data1->T2C, (void*)&text[i]); 
+    data1->NbT2++;
   }
 
   return v1;
