@@ -1,7 +1,7 @@
 #ifndef _ANNOTATE_H_
 #define _ANNOTATE_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,27 +31,27 @@ class GMSH_AnnotatePlugin : public GMSH_Post_Plugin
 {
 private:
   static double callback(int num, int action, double value, double *opt,
-			 double step, double min, double max);
-  static char *callbackStr(int num, int action, char *value, char **opt);
+                         double step, double min, double max);
+  static const char *callbackStr(int num, int action, const char *value, const char **opt);
 public:
-  GMSH_AnnotatePlugin();
+  GMSH_AnnotatePlugin(){}
   void getName(char *name) const;
   void getInfos(char *author, char *copyright, char *helpText) const;
   void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
-  StringXNumber* getOption(int iopt);  
+  StringXNumber *getOption(int iopt);  
   int getNbOptionsStr() const;
-  StringXString* getOptionStr(int iopt);  
-  Post_View *execute(Post_View *);
+  StringXString *getOptionStr(int iopt);  
+  PView *execute(PView *);
 
   static double callbackX(int, int, double);
   static double callbackY(int, int, double);
   static double callbackZ(int, int, double);
   static double callback3D(int, int, double);
   static double callbackFontSize(int, int, double);
-  static char *callbackText(int, int, char *);
-  static char *callbackFont(int, int, char *);
-  static char *callbackAlign(int, int, char *);
+  static const char *callbackText(int, int, const char *);
+  static const char *callbackFont(int, int, const char *);
+  static const char *callbackAlign(int, int, const char *);
   static void draw();
 };
 
