@@ -1,7 +1,7 @@
 #ifndef _CUT_GRID_H_
 #define _CUT_GRID_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,23 +30,21 @@ extern "C"
 class GMSH_CutGridPlugin : public GMSH_Post_Plugin
 {
   static double callback(int num, int action, double value, double *opt,
-			 double step, double min, double max);
-  void addInView(Post_View *v, int connect, int nbcomp, 
-		 double ***pnts, double ***vals, 
-		 List_T *P, int *nP, 
-		 List_T *L, int *nL, 
-		 List_T *Q, int *nQ);
-  Post_View * GenerateView (Post_View * v, int connectPoints);
+                         double step, double min, double max);
+  void addInView(int numsteps, int connect, int nbcomp, 
+                 double ***pnts, double ***vals, 
+                 List_T *P, int *nP, 
+                 List_T *L, int *nL, 
+                 List_T *Q, int *nQ);
+  PView *GenerateView (PView *v, int connectPoints);
 public:
-  GMSH_CutGridPlugin();
-  void getName  (char *name) const;
-  void getInfos (char *author, 
-  		 char *copyright,
-  		 char *help_text) const;
-  void catchErrorMessage (char *errorMessage) const;
+  GMSH_CutGridPlugin(){}
+  void getName(char *name) const;
+  void getInfos(char *author, char *copyright, char *help_text) const;
+  void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
-  StringXNumber *getOption (int iopt);  
-  Post_View *execute (Post_View *);
+  StringXNumber *getOption(int iopt);  
+  PView *execute(PView *);
 
   static int getNbU ();
   static int getNbV ();
