@@ -1,6 +1,6 @@
-// $Id: CutMap.cpp,v 1.53 2007-05-04 10:45:08 geuzaine Exp $
+// $Id: CutMap.cpp,v 1.59 2008-04-05 17:49:23 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,13 +20,12 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "CutMap.h"
-#include "List.h"
 #include "Context.h"
 
 extern Context_T CTX;
 
 StringXNumber CutMapOptions_Number[] = {
-  {GMSH_FULLRC, "A", GMSH_CutMapPlugin::callbackA, 1.},
+  {GMSH_FULLRC, "A", GMSH_CutMapPlugin::callbackA, 0.},
   {GMSH_FULLRC, "dTimeStep", NULL, -1.},
   {GMSH_FULLRC, "dView", NULL, -1.},
   {GMSH_FULLRC, "ExtractVolume", GMSH_CutMapPlugin::callbackVol, 0.},
@@ -43,25 +42,19 @@ extern "C"
   }
 }
 
-GMSH_CutMapPlugin::GMSH_CutMapPlugin()
-{
-  ;
-}
-
 double GMSH_CutMapPlugin::callbackA(int num, int action, double value)
 {
   double min = 0., max = 1.;
   if(action > 0){
     int iview = (int)CutMapOptions_Number[6].def;
     if(iview < 0) iview = num;
-    Post_View **vv = (Post_View **)List_Pointer_Test(CTX.post.list, iview);
-    if(vv){
-      min = (*vv)->Min;
-      max = (*vv)->Max;
+    if(iview >= 0 && iview < (int)PView::list.size()){
+      min = PView::list[iview]->getData()->getMin();
+      max = PView::list[iview]->getData()->getMax();
     }
   }
   switch(action){ // configure the input field
-  case 1: return (min-max)/200.;
+  case 1: return (min - max) / 200.;
   case 2: return min;
   case 3: return max;
   default: break;
@@ -115,19 +108,19 @@ void GMSH_CutMapPlugin::getInfos(char *author, char *copyright,
   strcpy(help_text,
          "Plugin(CutMap) extracts the isosurface of value\n"
          "`A' from the view `iView' and draws the\n"
-	 "`dTimeStep'-th value of the view `dView' on the\n"
-	 "isosurface. If `iView' < 0, the plugin is run\n"
-	 "on the current view. If `dTimeStep' < 0, the\n"
-	 "plugin uses, for each time step in `iView', the\n"
-	 "corresponding time step in `dView'. If `dView'\n"
-	 "< 0, the plugin uses `iView' as the value source.\n"
-	 "If `ExtractVolume' is nonzero, the plugin\n" 
-	 "extracts the isovolume with values greater (if\n"
-	 "`ExtractVolume' > 0) or smaller (if `ExtractVolume'\n"
-	 "< 0) than the isosurface `A'.\n"
-	 "\n"
-	 "Plugin(CutMap) creates as many views as there\n"
-	 "are time steps in `iView'.\n");
+         "`dTimeStep'-th value of the view `dView' on the\n"
+         "isosurface. If `iView' < 0, the plugin is run\n"
+         "on the current view. If `dTimeStep' < 0, the\n"
+         "plugin uses, for each time step in `iView', the\n"
+         "corresponding time step in `dView'. If `dView'\n"
+         "< 0, the plugin uses `iView' as the value source.\n"
+         "If `ExtractVolume' is nonzero, the plugin\n" 
+         "extracts the isovolume with values greater (if\n"
+         "`ExtractVolume' > 0) or smaller (if `ExtractVolume'\n"
+         "< 0) than the isosurface `A'.\n"
+         "\n"
+         "Plugin(CutMap) creates as many views as there\n"
+         "are time steps in `iView'.\n");
 }
 
 int GMSH_CutMapPlugin::getNbOptions() const
@@ -153,7 +146,7 @@ double GMSH_CutMapPlugin::levelset(double x, double y, double z, double val) con
   return val - CutMapOptions_Number[0].def;
 }
 
-Post_View *GMSH_CutMapPlugin::execute(Post_View * v)
+PView *GMSH_CutMapPlugin::execute(PView *v)
 {
   int iView = (int)CutMapOptions_Number[6].def;
   _valueIndependent = 0;
@@ -164,15 +157,8 @@ Post_View *GMSH_CutMapPlugin::execute(Post_View * v)
   _targetError = CutMapOptions_Number[5].def;
   _orientation = GMSH_LevelsetPlugin::MAP;
   
-  if(iView < 0)
-    iView = v ? v->Index : 0;
-  
-  if(!List_Pointer_Test(CTX.post.list, iView)) {
-    Msg(GERROR, "View[%d] does not exist", iView);
-    return v;
-  }
-  
-  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
-  
+  PView *v1 = getView(iView, v);
+  if(!v1) return v;
+
   return GMSH_LevelsetPlugin::execute(v1);
 }
