@@ -1,7 +1,7 @@
 #ifndef _CUT_PARAMETRIC_H_
 #define _CUT_PARAMETRIC_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,31 +33,30 @@ class GMSH_CutParametricPlugin : public GMSH_Post_Plugin
 { 
 private:
   static double callback(int num, int action, double value, double *opt,
-			 double step, double min, double max);
-  static char *callbackStr(int num, int action, char *value, char **opt);
+                         double step, double min, double max);
+  static const char *callbackStr(int num, int action, const char *value, 
+                                 const char **opt);
   static int fillXYZ();
   static int recompute;
   static std::vector<double> x, y, z;
 public:
-  GMSH_CutParametricPlugin();
-  void getName  (char *name) const;
-  void getInfos (char *author, 
-  		 char *copyright,
-  		 char *help_text) const;
-  void catchErrorMessage (char *errorMessage) const;
+  GMSH_CutParametricPlugin(){}
+  void getName(char *name) const;
+  void getInfos(char *author, char *copyright, char *help_text) const;
+  void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
-  StringXNumber *getOption (int iopt);  
+  StringXNumber *getOption(int iopt);  
   int getNbOptionsStr() const;
-  StringXString* getOptionStr(int iopt);  
-  Post_View *execute (Post_View *);
+  StringXString *getOptionStr(int iopt);  
+  PView *execute(PView *);
 
   static double callbackMinU(int, int, double);
   static double callbackMaxU(int, int, double);
   static double callbackN(int, int, double);
   static double callbackConnect(int, int, double);
-  static char *callbackX(int, int, char *);
-  static char *callbackY(int, int, char *);
-  static char *callbackZ(int, int, char *);
+  static const char *callbackX(int, int, const char *);
+  static const char *callbackY(int, int, const char *);
+  static const char *callbackZ(int, int, const char *);
   static void draw();
 };
 
