@@ -1,6 +1,6 @@
-// $Id: CutPlane.cpp,v 1.52 2007-05-04 10:45:08 geuzaine Exp $
+// $Id: CutPlane.cpp,v 1.58 2008-06-05 13:57:47 samtech Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "CutPlane.h"
-#include "List.h"
 #include "Context.h"
 
 #if defined(HAVE_FLTK)
@@ -51,31 +50,27 @@ extern "C"
   }
 }
 
-GMSH_CutPlanePlugin::GMSH_CutPlanePlugin()
-{
-  ;
-}
-
 void GMSH_CutPlanePlugin::draw()
 {
 #if defined(HAVE_FLTK)
   int num = (int)CutPlaneOptions_Number[7].def;
   if(num < 0) num = iview;
-  Post_View **vv = (Post_View **)List_Pointer_Test(CTX.post.list, num);
-  if(!vv) return;
-  glColor4ubv((GLubyte *) & CTX.color.fg);
-  glLineWidth(CTX.line_width);
-  Draw_PlaneInBoundingBox((*vv)->BBox[0], (*vv)->BBox[2], (*vv)->BBox[4],
-			  (*vv)->BBox[1], (*vv)->BBox[3], (*vv)->BBox[5],
-			  CutPlaneOptions_Number[0].def,
-			  CutPlaneOptions_Number[1].def,
-			  CutPlaneOptions_Number[2].def,
-			  CutPlaneOptions_Number[3].def);
+  if(num >= 0 && num < (int)PView::list.size()){
+    glColor4ubv((GLubyte *) & CTX.color.fg);
+    glLineWidth(CTX.line_width);
+    SBoundingBox3d bb = PView::list[num]->getData()->getBoundingBox();
+    Draw_PlaneInBoundingBox(bb.min().x(), bb.min().y(), bb.min().z(), 
+                            bb.max().x(), bb.max().y(), bb.max().z(), 
+                            CutPlaneOptions_Number[0].def,
+                            CutPlaneOptions_Number[1].def,
+                            CutPlaneOptions_Number[2].def,
+                            CutPlaneOptions_Number[3].def);
+  }
 #endif
 }
 
 double GMSH_CutPlanePlugin::callback(int num, int action, double value, double *opt,
-				     double step, double min, double max)
+                                     double step, double min, double max)
 {
   if(action > 0) iview = num;
   switch(action){ // configure the input field
@@ -94,43 +89,43 @@ double GMSH_CutPlanePlugin::callback(int num, int action, double value, double *
 double GMSH_CutPlanePlugin::callbackA(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[0].def,
-		  0.01, -1, 1);
+                  0.01, -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackB(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[1].def,
-		  0.01, -1, 1);
+                  0.01, -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackC(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[2].def,
-		  0.01, -1, 1);
+                  0.01, -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackD(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[3].def,
-		  CTX.lc/200., -CTX.lc, CTX.lc);
+                  CTX.lc/200., -CTX.lc, CTX.lc);
 }
 
 double GMSH_CutPlanePlugin::callbackVol(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[4].def,
-		  1., -1, 1);
+                  1., -1, 1);
 }
 
 double GMSH_CutPlanePlugin::callbackRecur(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[5].def,
-		  1, 0, 10);
+                  1, 0, 10);
 }
 
 double GMSH_CutPlanePlugin::callbackTarget(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[6].def,
-		  0.01, 0., 1.);
+                  0.01, 0., 1.);
 }
 
 void GMSH_CutPlanePlugin::getName(char *name) const
@@ -145,13 +140,13 @@ void GMSH_CutPlanePlugin::getInfos(char *author, char *copyright,
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
          "Plugin(CutPlane) cuts the view `iView' with\n"
-	 "the plane `A'*X + `B'*Y + `C'*Z + `D' = 0. If\n"
-	 "`ExtractVolume' is nonzero, the plugin extracts\n"
-	 "the elements on one side of the plane (depending\n"
-	 "on the sign of `ExtractVolume'). If `iView' < 0,\n"
-	 "the plugin is run on the current view.\n"
-	 "\n"
-	 "Plugin(CutPlane) creates one new view.\n");
+         "the plane `A'*X + `B'*Y + `C'*Z + `D' = 0. If\n"
+         "`ExtractVolume' is nonzero, the plugin extracts\n"
+         "the elements on one side of the plane (depending\n"
+         "on the sign of `ExtractVolume'). If `iView' < 0,\n"
+         "the plugin is run on the current view.\n"
+         "\n"
+         "Plugin(CutPlane) creates one new view.\n");
 }
 
 int GMSH_CutPlanePlugin::getNbOptions() const
@@ -176,19 +171,19 @@ double GMSH_CutPlanePlugin::levelset(double x, double y, double z, double val) c
     CutPlaneOptions_Number[2].def * z + CutPlaneOptions_Number[3].def;
 }
 
-bool GMSH_CutPlanePlugin::geometrical_filter(Double_Matrix *geometrical_nodes_positions) const
+bool GMSH_CutPlanePlugin::geometricalFilter(Double_Matrix *node_positions) const
 {
-  const double l0 = levelset((*geometrical_nodes_positions)(0,0),
-			     (*geometrical_nodes_positions)(0,1),
-			     (*geometrical_nodes_positions)(0,2),1);
-  for (int i=1;i<geometrical_nodes_positions->size1();i++)
-    if (levelset((*geometrical_nodes_positions)(i,0),
-		 (*geometrical_nodes_positions)(i,1),
-		 (*geometrical_nodes_positions)(i,2),1) * l0 < 0) return true;
+  const double l0 = levelset((*node_positions)(0, 0),
+                             (*node_positions)(0, 1),
+                             (*node_positions)(0, 2), 1);
+  for (int i = 1; i < node_positions->size1(); i++)
+    if (levelset((*node_positions)(i, 0),
+                 (*node_positions)(i, 1),
+                 (*node_positions)(i, 2), 1) * l0 < 0) return true;
   return false;
 }
 
-Post_View *GMSH_CutPlanePlugin::execute(Post_View * v)
+PView *GMSH_CutPlanePlugin::execute(PView *v)
 {
   int iView = (int)CutPlaneOptions_Number[7].def;
   _ref[0] = CutPlaneOptions_Number[0].def;
@@ -202,15 +197,8 @@ Post_View *GMSH_CutPlanePlugin::execute(Post_View * v)
   _recurLevel = (int)CutPlaneOptions_Number[5].def;
   _targetError = CutPlaneOptions_Number[6].def;
   
-  if(iView < 0)
-    iView = v ? v->Index : 0;
-  
-  if(!List_Pointer_Test(CTX.post.list, iView)) {
-    Msg(GERROR, "View[%d] does not exist", iView);
-    return v;
-  }
-  
-  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
-  
+  PView *v1 = getView(iView, v);
+  if(!v1) return v;
+
   return GMSH_LevelsetPlugin::execute(v1);
 }
