@@ -1,6 +1,6 @@
-// $Id: CutSphere.cpp,v 1.47 2007-05-04 10:45:08 geuzaine Exp $
+// $Id: CutSphere.cpp,v 1.52 2008-06-05 13:57:47 samtech Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 #include <string.h>
 #include "CutSphere.h"
-#include "List.h"
 #include "Context.h"
 
 #if defined(HAVE_FLTK)
@@ -49,11 +48,6 @@ extern "C"
   }
 }
 
-GMSH_CutSpherePlugin::GMSH_CutSpherePlugin()
-{
-  ;
-}
-
 void GMSH_CutSpherePlugin::draw()
 {
 #if defined(HAVE_FLTK)
@@ -70,8 +64,8 @@ void GMSH_CutSpherePlugin::draw()
   glColor4ubv((GLubyte *) & CTX.color.fg);
   glLineWidth(CTX.line_width);
   glTranslated(CutSphereOptions_Number[0].def,
-	       CutSphereOptions_Number[1].def,
-	       CutSphereOptions_Number[2].def);
+               CutSphereOptions_Number[1].def,
+               CutSphereOptions_Number[2].def);
   gluSphere(qua, CutSphereOptions_Number[3].def, 40, 40);
   glPopMatrix();
   glPolygonMode(GL_FRONT_AND_BACK, mode[1]);
@@ -79,7 +73,7 @@ void GMSH_CutSpherePlugin::draw()
 }
 
 double GMSH_CutSpherePlugin::callback(int num, int action, double value, double *opt,
-				      double step, double min, double max)
+                                      double step, double min, double max)
 {
   switch(action){ // configure the input field
   case 1: return step;
@@ -97,37 +91,37 @@ double GMSH_CutSpherePlugin::callback(int num, int action, double value, double
 double GMSH_CutSpherePlugin::callbackX(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[0].def,
-		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+                  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackY(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[1].def,
-		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+                  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackZ(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[2].def,
-		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+                  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackR(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[3].def,
-		  CTX.lc/100., 0., 2*CTX.lc);
+                  CTX.lc/100., 0., 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackVol(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[4].def,
-		  1., -1., 1.);
+                  1., -1., 1.);
 }
 
 double GMSH_CutSpherePlugin::callbackRecur(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[5].def,
-		  1, 0, 10);
+                  1, 0, 10);
 }
 
 void GMSH_CutSpherePlugin::getName(char *name) const
@@ -142,13 +136,13 @@ void GMSH_CutSpherePlugin::getInfos(char *author, char *copyright,
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
          "Plugin(CutSphere) cuts the view `iView' with the\n"
-	 "sphere (X-`Xc')^2 + (Y-`Yc')^2 + (Z-`Zc')^2 = `R'^2.\n"
-	 "If `ExtractVolume' is nonzero, the plugin extracts\n"
-	 "the elements inside (if `ExtractVolume' < 0) or\n"
-	 "outside (if `ExtractVolume' > 0) the sphere. If\n"
-	 "`iView' < 0, the plugin is run on the current view.\n"
-	 "\n"
-	 "Plugin(CutSphere) creates one new view.\n");
+         "sphere (X-`Xc')^2 + (Y-`Yc')^2 + (Z-`Zc')^2 = `R'^2.\n"
+         "If `ExtractVolume' is nonzero, the plugin extracts\n"
+         "the elements inside (if `ExtractVolume' < 0) or\n"
+         "outside (if `ExtractVolume' > 0) the sphere. If\n"
+         "`iView' < 0, the plugin is run on the current view.\n"
+         "\n"
+         "Plugin(CutSphere) creates one new view.\n");
 }
 
 int GMSH_CutSpherePlugin::getNbOptions() const
@@ -176,7 +170,7 @@ double GMSH_CutSpherePlugin::levelset(double x, double y, double z,
   return (x - a) * (x - a) + (y - b) * (y - b) + (z - c) * (z - c) - r * r;
 }
 
-Post_View *GMSH_CutSpherePlugin::execute(Post_View * v)
+PView *GMSH_CutSpherePlugin::execute(PView *v)
 {
   int iView = (int)CutSphereOptions_Number[6].def;
   _ref[0] = CutSphereOptions_Number[0].def;
@@ -190,15 +184,8 @@ Post_View *GMSH_CutSpherePlugin::execute(Post_View * v)
   _valueTimeStep = -1;
   _orientation = GMSH_LevelsetPlugin::SPHERE;
 
-  if(iView < 0)
-    iView = v ? v->Index : 0;
-
-  if(!List_Pointer_Test(CTX.post.list, iView)) {
-    Msg(GERROR, "View[%d] does not exist", iView);
-    return v;
-  }
-
-  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
+  PView *v1 = getView(iView, v);
+  if(!v1) return v;
 
   return GMSH_LevelsetPlugin::execute(v1);
 }
