@@ -1,7 +1,7 @@
 #ifndef _EIGENVECTORS_H_
 #define _EIGENVECTORS_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "Plugin.h"
-#include "List.h"
 
 extern "C"
 {
@@ -31,13 +30,13 @@ extern "C"
 class GMSH_EigenvectorsPlugin : public GMSH_Post_Plugin
 {
  public:
-  GMSH_EigenvectorsPlugin();
+  GMSH_EigenvectorsPlugin(){}
   void getName(char *name) const;
   void getInfos(char *author, char *copyright, char *helpText) const;
   void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
-  StringXNumber* getOption(int iopt);  
-  Post_View *execute(Post_View *);
+  StringXNumber *getOption(int iopt);  
+  PView *execute(PView *);
 };
 
 #endif
