@@ -1,7 +1,7 @@
 #ifndef _EXTRACT_H_
 #define _EXTRACT_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "Plugin.h"
-#include "List.h"
 
 extern "C"
 {
@@ -31,7 +30,7 @@ extern "C"
 class GMSH_ExtractPlugin : public GMSH_Post_Plugin
 {
  public:
-  GMSH_ExtractPlugin();
+  GMSH_ExtractPlugin(){}
   void getName(char *name) const;
   void getInfos(char *author, char *copyright, char *helpText) const;
   void catchErrorMessage(char *errorMessage) const;
@@ -39,7 +38,7 @@ class GMSH_ExtractPlugin : public GMSH_Post_Plugin
   StringXNumber* getOption(int iopt);  
   int getNbOptionsStr() const;
   StringXString* getOptionStr(int iopt);  
-  Post_View *execute(Post_View *);
+  PView *execute(PView *);
 };
 
 #endif
