@@ -1,7 +1,7 @@
 #ifndef _LAMBDA2_H_
 #define _LAMBDA2_H
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,12 +21,6 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "Plugin.h"
-#include "List.h"
-#include "Numeric.h"
-#include <algorithm>
-#include <functional>
-#include <math.h>
-#include <stdio.h>
 
 extern "C"
 {
@@ -42,8 +36,7 @@ public:
   void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
   StringXNumber* getOption(int iopt);  
-
-  Post_View *execute(Post_View *);
+  PView *execute(PView *);
 };
 
 #endif
