@@ -1,7 +1,7 @@
 #ifndef _REMOVE_H_
 #define _REMOVE_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,13 +30,13 @@ extern "C"
 class GMSH_RemovePlugin : public GMSH_Post_Plugin
 {
 public:
-  GMSH_RemovePlugin();
+  GMSH_RemovePlugin(){}
   void getName(char *name) const;
   void getInfos(char *author, char *copyright, char *helpText) const;
   void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
-  StringXNumber* getOption(int iopt);  
-  Post_View *execute(Post_View *);
+  StringXNumber *getOption(int iopt);  
+  PView *execute(PView *);
 };
 
 #endif
