@@ -1,7 +1,7 @@
 #ifndef _SKIN_H_
 #define _SKIN_H_
 
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
 #include "Plugin.h"
-#include "List.h"
-#include "Tree.h"
+#include "ListUtils.h"
+#include "TreeUtils.h"
 
 extern "C"
 {
@@ -41,15 +41,15 @@ class GMSH_SkinPlugin : public GMSH_Post_Plugin
   static int fcmpElm(const void *a, const void *b);
   static void addInView(void *a, void *b);
   void skinList(List_T *inList, int inNbList, 
-		int inNbNod, int inNbFac, int fxn[6][4]);
+                int inNbNod, int inNbFac, int fxn[6][4]);
  public:
-  GMSH_SkinPlugin();
+  GMSH_SkinPlugin(){}
   void getName(char *name) const;
   void getInfos(char *author, char *copyright, char *helpText) const;
   void catchErrorMessage(char *errorMessage) const;
   int getNbOptions() const;
-  StringXNumber* getOption(int iopt);  
-  Post_View *execute(Post_View *);
+  StringXNumber *getOption(int iopt);  
+  PView *execute(PView *);
 };
 
 #endif
