@@ -1,6 +1,6 @@
-// $Id: Smooth.cpp,v 1.24 2007-05-04 10:45:09 geuzaine Exp $
+// $Id: Smooth.cpp,v 1.32 2008-04-05 17:49:23 geuzaine Exp $
 //
-// Copyright (C) 1997-2007 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,13 +19,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
-#include "Plugin.h"
 #include "Smooth.h"
-#include "List.h"
-#include "Views.h"
-#include "Context.h"
-
-extern Context_T CTX;
 
 StringXNumber SmoothOptions_Number[] = {
   {GMSH_FULLRC, "iView", NULL, -1.}
@@ -39,12 +33,6 @@ extern "C"
   }
 }
 
-
-GMSH_SmoothPlugin::GMSH_SmoothPlugin()
-{
-  ;
-}
-
 void GMSH_SmoothPlugin::getName(char *name) const
 {
   strcpy(name, "Smooth");
@@ -57,10 +45,10 @@ void GMSH_SmoothPlugin::getInfos(char *author, char *copyright,
   strcpy(copyright, "DGR (www.multiphysics.com)");
   strcpy(help_text,
          "Plugin(Smooth) averages the values at the nodes\n"
-	 "of the scalar view `iView'. If `iView' < 0, the\n"
-	 "plugin is run on the current view.\n"
-	 "\n"
-	 "Plugin(Smooth) is executed in-place.\n");
+         "of the scalar view `iView'. If `iView' < 0, the\n"
+         "plugin is run on the current view.\n"
+         "\n"
+         "Plugin(Smooth) is executed in-place.\n");
 }
 
 int GMSH_SmoothPlugin::getNbOptions() const
@@ -78,21 +66,15 @@ void GMSH_SmoothPlugin::catchErrorMessage(char *errorMessage) const
   strcpy(errorMessage, "Smooth failed...");
 }
 
-Post_View *GMSH_SmoothPlugin::execute(Post_View * v)
+PView *GMSH_SmoothPlugin::execute(PView *v)
 {
   int iView = (int)SmoothOptions_Number[0].def;
 
-  if(iView < 0)
-    iView = v ? v->Index : 0;
+  PView *v1 = getView(iView, v);
+  if(!v1) return v;
 
-  if(!List_Pointer_Test(CTX.post.list, iView)) {
-    Msg(GERROR, "View[%d] does not exist", iView);
-    return v;
-  }
+  v1->getData()->smooth();
+  v1->setChanged(true);
 
-  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
-
-  v1->smooth();
   return v1;
 }
-
